/*
	TBase.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.03	1.0			Takeshi Yoneki
*/

#include "csystem.h"

static char safetyEmptyBuffer[16] = "\0";

/*
	TBase
*/

TBase::TBase() {
	m_next = NULL;
	m_prev = NULL;
}

TBase::~TBase() {
}

/*
	TListBase
*/

TListBase::TListBase() {
	ClearListBase();
}

TListBase::~TListBase() {
}

void TListBase::ClearListBase() {
	m_top = NULL;
	m_last = NULL;
	m_count = 0;
}

void TListBase::CopyFromArray(TArrayBase& array) {
	RemoveAll();
	for (int i = 0; i < array.GetCount(); i++) {
		TBase *item = array.GetNth(i);
		if (item != NULL)
			Append(item);
	}
}

void TListBase::Append(TBase *item) {
	if (NULL == item)
		return;
	item->m_next = NULL;
	if (NULL == m_last)		/* ŏAppend */
	{
		item->m_prev = NULL;
		m_top = item;
		m_last = item;
	}
	else	/* ʏ */
	{
		m_last->m_next = item;
		item->m_prev = m_last;
		m_last = item;
	}
	m_count += 1;
}

void TListBase::Insert(TBase *prev, TBase *item) {
	if (item == NULL || prev == item)
		return;
	if (NULL == prev)	/* 擪ɑ} */
	{
		item->m_next = m_top;
		m_top = item;
	}
	else	/* ʏ */
	{
		item->m_next = prev->m_next;
		prev->m_next = item;
	}
	item->m_prev = prev;
	if (item->m_next != NULL)	/*  */
		item->m_next->m_prev = item;
	else	/* Ȃ */
		m_last = item;
	m_count += 1;
}

void TListBase::Insert(int n, TBase *item) {
	if (n < 0 || n > m_count || item == NULL)
		return;
	Insert(GetNth(n - 1), item);
}

Boolean TListBase::Remove(int n) {
	return Remove(GetNth(n));
}

Boolean TListBase::LinkCheck(TBase *item) {
	if (item == m_last)
		return TRUE;
	TBase *current = m_top;
	for (; current != NULL;) {
		if (current == item)
			return TRUE;
		current = current->m_next;
	}
	return FALSE;
}

Boolean TListBase::Remove(TBase *item) {
	if (LinkCheck(item) == FALSE)
		return FALSE;
	if (m_top == item || item->m_prev != NULL) {
		if (m_top == item)	/* 擪 */
			m_top = item->m_next;
		else	/* ʏ */
			item->m_prev->m_next = item->m_next;
		if (item->m_next != NULL)	/*  */
			item->m_next->m_prev = item->m_prev;
		else	/* Ȃ */
			m_last = item->m_prev;
		item->m_next = NULL;
		item->m_prev = NULL;
		m_count -= 1;
		return TRUE;
	}
	return FALSE;
}

void TListBase::RemoveDelete(int n) {
	RemoveDelete(GetNth(n));
}

void TListBase::RemoveDelete(TBase *item) {
	if (item != NULL) {
		if (Remove(item))
			delete item;
	}
}

void TListBase::RemoveAll() {
	for (; GetCount() > 0;)
		Remove(m_last/*m_top*/);
}

void TListBase::RemoveDeleteAll() {
	for (; GetCount() > 0;)
		RemoveDelete(m_last/*m_top*/);
}

int TListBase::GetOrder(TBase *item) {
	TBase *current = m_top;
	for (int i = 0; i < m_count; i++) {
		if (item == current || NULL == current)
			return i;
		current = current->m_next;
	}
	return -1;
}

void TListBase::MoveToTop(TBase *item) {
	Remove(item);
	Insert((TBase*)NULL, item);
}

TBase *TListBase::GetNth(int n) {
	TBase *current = m_top;
	for (int i = 0; i < m_count; i++) {
		if (i == n || NULL == current)
			return current;
		current = current->m_next;
	}
	return NULL;
}

/*
	TArrayBase
*/

void TArrayBase::Initial() {
	m_count = 0;
	m_array = NULL;
	m_expandCount = 16;
}

TArrayBase::TArrayBase() {
	Initial();
}

TArrayBase::TArrayBase(int count) {
	Initial();
	m_expandCount = count;
	ChangeCount(count);
}

TArrayBase::~TArrayBase() {
	if (m_array != NULL)
		delete [] m_array;
}

void TArrayBase::ChangeCount(int count) {
	if (m_array != NULL)
		delete [] m_array;
	m_array = count == 0 ? NULL : new TBase *[count];
	m_count = count;
	for (int i = 0; i < count; i++)
		m_array[i] = NULL;
}

void TArrayBase::CopyFromList(TListBase& list) {
	ChangeCount(list.GetCount());
	TBase *item = list.GetTop();
	for (int i = 0; item != NULL; i++, item = item->GetNext())
		SetNth(i, item);
}

void TArrayBase::Expand(int count) {
	if (m_count >= count)
		return;
	TBase **array = new TBase *[count];
	int i;
	for (i = 0; i < count; i++)
		array[i] = NULL;
	if (m_array != NULL) {
		for (i = 0; i < m_count; i++)
			array[i] = m_array[i];
		delete [] m_array;
	}
	m_array = array;
	m_count = count;
}

void TArrayBase::SetNth(int n, TBase *item) {
	if (n >= 0 && n < m_count)
		m_array[n] = item;
}

TBase *TArrayBase::GetNth(int n) {
	if (n >= 0 && n < m_count)
		return m_array[n];
	else
		return NULL;
}

int TArrayBase::GetOrder(TBase *item) {
	for (int i = 0; i < m_count; i++) {
		if (m_array[i] == item)
			return i;
	}
	return -1;
}

void TArrayBase::Append(TBase *item) {
	int n = -1;
	for (; n == -1;) {
		for (int i = 0; i < GetCount(); i++) {
			if (GetNth(i) == NULL) {
				n = i;
				break;
			}
		}
		if (n == -1)
			Expand(GetCount() + m_expandCount);
	}
	SetNth(n, item);
}

void TArrayBase::Remove(TBase *item) {
	for (int i = 0; i < GetCount(); i++) {
		if (GetNth(i) == item) {
			SetNth(i, NULL);
			break;
		}
	}
}

void TArrayBase::RemoveDeleteAll() {
	for (int i = 0; i < GetCount(); i++) {
		TBase *item = GetNth(i);
		if (item != NULL) {
			delete item;
			SetNth(i, NULL);
		}
	}
}

/*
	TMemory
*/

void TMemory::InitTMemory() {
	m_data = NULL;
	m_length = 0;
}

TMemory::TMemory() {
	InitTMemory();
}

TMemory::TMemory(long length) {
	InitTMemory();
	ChangeLength(length);
}

TMemory::~TMemory() {
	if (m_data != NULL)
		delete [] m_data;
}

void TMemory::ChangeLength(long length) {
	if (m_data != NULL)
		delete [] m_data;
	m_data = length == 0 ? NULL : new char[length];
	m_length = length;
	if (m_length > 0)
		memset();
}

void TMemory::ExpandLength(long addLength) {
	if (addLength == 0)
		return;
	long newLength = m_length + addLength;
	char *data = new char[newLength];
	if (m_length > 0 && m_data != NULL)
		memcpy(data, m_data, m_length);
	if (m_data != NULL)
		delete [] m_data;
	::memset(&data[m_length], 0, addLength);
	m_data = data;
	m_length = newLength;
}

//TMemory::operator char *() const
//{
//	return m_data == NULL ? safetyEmptyBuffer : m_data;
//}

Ptr TMemory::GetPtr() const {
	return m_data == NULL ? safetyEmptyBuffer : m_data;
}

void TMemory::operator=(TString& cs)	/* TString̃Rs[ */
{
	ChangeLength(cs.GetLength());
	memcpy((char *)*this, (char *)cs, cs.GetLength());
}

void TMemory::operator=(const TMemory& bf)	/* TMemorỹRs[ */
{
	ChangeLength(bf.GetLength());
	memcpy((char *)*this, (char *)bf, bf.GetLength());
}

char TMemory::GetAt(int n) {
	if (n >= 0 && n < m_length)
		return m_data[n];
	else
		return '\0';
}

void TMemory::SetAt(int n, char c) {
	if (n >= 0 && n < m_length)
		m_data[n] = c;
}

/*
	TTempMemory
*/

void TTempMemory::InitTTempMemory() {
	m_h = NULL;
	m_p = NULL;
	m_size = 0;
}

TTempMemory::TTempMemory() {
	InitTTempMemory();
}

TTempMemory::TTempMemory(size_t sz) {
	InitTTempMemory();
	Create(sz);
}

void TTempMemory::Create(size_t sz) {
	Dispose();
	if (theSystem->hasTempMemory) {
		m_h = TempNewHandle(sz, &m_error);
		if (m_error == noErr && m_h != NULL) {
			HLock(m_h);
			m_p = *m_h;
			m_size = sz;
		}
		else
			m_h = NULL;
	}
}

void TTempMemory::Dispose() {
	if (m_h != NULL) {
		if (m_p != NULL)
			HUnlock(m_h);
		DisposeHandle(m_h);
	}
	InitTTempMemory();
}

Ptr TTempMemory::GetPtr() const {
	return m_p == NULL ? safetyEmptyBuffer : m_p;
}

TTempMemory::~TTempMemory() {
	Dispose();
}

/*
	TAppleObject
	OSAIuWFNg{NX
*/

/*
TAppleObjectpNXŎȂ΂ȂȂ֐
 
TAppleObjectpNX͊eȂĂ͂Ȃz֐B
ASɑSĎȂĂ͂Ȃ킯ł͂ȂB
ftHg̓̂܂܂ŗǂƂB
 
Ŏguԍv́uvłAformAbsolutePositionŗL
lłȂĂ͂ȂB
Ȃ킿Aԍł̃ANZX͂P`ƂȂB
Window̓EBhEj[̕тł͂Ȃʏ̑O֌WiyI[_[j
łB
 
eIuWFNg̃fXNv^^CvԂ
virtual DescType GetObjectType();
	AvP[VƎ̌^ibbsƂforumƂjȂ΁ANXƃ^Cv
	v̂AႦ΁uOvȂĂ̂'cnum''TEXT'Ȃ̂ŁA
	ꍇ'TEXT'ԂBƂ'TEXT'ԂȂł낤BuOv
	߂̃NXpӂ킯ł͂Ȃ̂B
	Gg̓NXƃ^CvvăvpeB͈vȂX邪A
	ȂƂł͂ȂBIuWFNgXyVt@CA쐬鎞ɎQƂ邪
	mȎǂ͂ȂƂȂB
	eNXƗȂ̂ŕK{B
	ǋL
	NXȂǂ'TEXT'ɂȂł낤Ɨ\ĂAɂ炸B
	OIuWFNgƓō쐬IuWFNg̎ʂKvȂ
	ႦΕNX'Str 'ƂƎ̃^CvԂƂɂȂB
 
eIuWFNg̖OԂ@ÔȂNX͂肦
virtual char *GetObjectName();
	ŏchar *łȂTStringŎATStringgTAppleObject
	TuNXɂłȂƂ킩Achar *ɂBȂ킯ŁA
	z֐ŕԂO̓X^bNiϐjł͂ȂƂɂȂB
	IvVGg͕K{ɋ߂B
	char *CFStringRefɕύXB2006.02.03
 
IuWFNgg̏Ԃ
virtual long GetObjectNumber();
	ʏ͐eIuWFNgɃXgǗĂł낤ƂO
	ftHgm_containerɑ΂GetOrder(this) + 1ԂĂB
	cWindow͂yI[_[B
	Gg͕K{AʏftHgŗǂB
 
Cӂ̃IuWFNgԍ𓾂iJgӖĂ悢j
virtual long GetAnyContainedObjectNumber();
	{_ȒlԂ̂ƌ܂Ă邪AʎƂăJg
	Ӗ悤ɂƎvB
	ꉞႦApplicatioñvpeBcurrent bbsƂ̂pӂ
	ƂŁAɗ͎͂gȂ悤ɂB
 
ŊܗLIuWFNg𓾂
virtual TAppleObject *GetContainedObjectByNumber(
	DescType desiredClass, int n);
	ʏ͐eIuWFNgɃXgǗĂł낤ƂO
	ftHgGetNth(n - 1)ԂĂB
	cWindow͂yI[_[B
	Gg͕K{AʏftHgŗǂB
 
ܗLIuWFNgL[ň
virtual TAppleObject *GetContainedObject(
	DescType desiredClass, DescType keyForm, const AEDesc *keyData);
	IuWFNgĂIuWFNgԂ
	ȂׂTAppleObject̎Ă郁\bhŉł悤ɂĂ̂ŁA
	̂܂܎gėǂB
	Gg͕K{AʏftHgŗǂB
 
vpeB𓾂
virtual TAppleObject *GetObjectProperty(
	DescType desiredClass, DescType keyForm, const AEDesc *keyData);
	vpeBNX͕KȂĂ͂ȂB
	Gg͕K{B
 
ܗLIuWFNg̐𓾂
	virtual long CountContainedObjects(DescType desiredClass);
	GgނȂNX̓ftHgGetCount()݂̂ŗǂB
	Gg͕K{AʏftHgŗǂB
 
IuWFNg̔r
	virtual Boolean CompareObjects(DescType oper, TAppleObject *obj);
	ftHgł̓IuWFNg̏ł̂ݔr悤ɎĂ̂ŁA
	ł͈ӖȂꍇ̓I[o[ChȂĂ͂ȂB
	Gg͕K{AʏftHgŗǂB
 
Cxg
virtual OSErr OnAppleEvent(
	AEEventID eventID, DescType desiredClass, 
	const AppleEvent *aev, AppleEvent *reply);
	eIuWFNgɑ΂Cxg𒼐ړnB
	̎XNvgΉ̐SB
	Cxg󂯎NX͕K{B
 
eւ̃CxgtbN
virtual OSErr OnAdhocForContainer(
	TToken *token, AEEventID eventID, 
	const AppleEvent *aev, AppleEvent *reply);
	ړĨIuWFNgɃCxgnł͓ƂĊłȂƂ
	߁A̐ẽIuWFNgɂCxgnB
	ႦΖO̕ύX͕NXɒڍsAʂt@Cɔf
	錠͂̐eNXɂȂ̂ʏB
	Cxg󂯎NX̐eNX͂قڕK{B
 
IuWFNgXyVt@CA𓾂
virtual OSErr GetObjSpecifier(DescType desiredClass, AEDesc *objSpec);
	̎ł͏ł̎wƂȂB
	create element̖߂lȂǂŎgIuWFNgXyVt@CA邪A
	ʏ̓ftHg̎ŗǂƎvBIuWFNgԂ̓
	\bhƂċAȂ񂾂Ⴄ񂾂ǂ킩ȂɂȂĂB
	IɃNX݂ĂAXNvgɌȂ̂GetObjSpecifier
	ʂ蔲ȂĂ͂Ȃ̂ThroughObjSpecifierƂ\bh
	pӂĂB
*/

TAppleObject::TAppleObject() {
	m_isTemporary = FALSE;
	m_container = NULL;
	//	m_desiredClass = 'null';
}

/*
	IuWFNgg[N
	쐬ꎞIIuWFNgȂm_isTemporaryTRUEɂB
*/
OSErr TAppleObject::CreateToken(DescType desiredClass, AEDesc *token) {
	AObjRecord aobj;
	aobj.m_obj = this;
	aobj.m_desiredClass = desiredClass;
	return ::AECreateDesc(classAppleObject, &aobj, sizeof (AObjRecord), token);
}

/*
	GetObjectClass
	eIuWFNg̃NX̎ʂԂ
	'TEXT'Ȃ
*/
DescType TAppleObject::GetObjectType() {
	return typeNull;
}

/*
	IuWFNgg̖O
*/
static TCFString TAppleObject_emptyName("");

CFStringRef TAppleObject::GetObjectName() {
	return TAppleObject_emptyName;
}

/*
	IuWFNgg̏
*/
long TAppleObject::GetObjectNumber() {
	return m_container == NULL ? 0 : m_container->GetOrder(this) + 1;
}

/*
	ŊܗLIuWFNg𓾂
*/
TAppleObject *TAppleObject::GetContainedObjectByNumber(DescType desiredClass, int n) {
	return (TAppleObject *)GetNth(n - 1);
}

/*
	OnAppleEvent
	ۂ̃\bhƌѕt
*/
OSErr TAppleObject::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	return errAEEventNotHandled;
}

/*
	OnAdhocForContainer
	OnAppleEvenťɐeiReijɑ΂ČĂ΂܂
	IuWFNggł͊łȂƂ
*/
OSErr TAppleObject::OnAdhocForContainer(TToken *token, AEEventID eventID, const AppleEvent *aev, TAEResult *result) {
	return noErr;
}

/*
*/
long TAppleObject::GetAnyContainedObjectNumber() {
	return 1;
}

/*
	GetContainedObject
	IuWFNgĂIuWFNgԂ
*/
TAppleObject *TAppleObject::GetContainedObject(DescType desiredClass, DescType keyForm, const AEDesc *keyData) {
	TAEDesc key(keyData);
	TAppleObject *target = NULL;
	long n = 0;
	if (keyForm == formAbsolutePosition) {
		long count = CountContainedObjects(desiredClass);

		if (key.descriptorType == typeLongInteger) {
			n = key.GetInteger();
			if (n < 0)
				n = count + 1 + n;
		}
		else if (key.descriptorType == typeAbsoluteOrdinal) {
			DescType order = key.GetDescType();
			if (order == kAEFirst)
				n = 1;
			else if (order == kAEMiddle)
				n = count / 2 + count % 2;
			else if (order == kAELast)
				n = count;
			else if (order == kAEAny)
				n = GetAnyContainedObjectNumber();
			else if (order == kAEAll) {
				TAppleObjectArray *array = new TAppleObjectArray(count, this);
				array->m_isTemporary = TRUE;
				array->m_container = NULL;
				for (int i = 0; i < count; i++)
					array->SetNthObj(i, GetContainedObjectByNumber(desiredClass, i + 1));
				target = (TAppleObject *)array;
			}
		}
		if (n > 0 && n <= count)
			target = GetContainedObjectByNumber(desiredClass, n);
	}
	else if (keyForm == formName) {
		TString name = key.GetString();
		for (target = GetTop(); target != NULL; target = target->GetNext()) {
			TCFString targetName(target->GetObjectName());
			if (targetName.EqualsIgnoreCase(name))
				break;
		}
	}
	else if (keyForm == formRelativePosition) {
		DescType rel = key.GetDescType();
		n = GetObjectNumber();
		if (rel == kAEPrevious)
			target = m_container->GetContainedObjectByNumber(desiredClass, n - 1);
		else if (rel == kAENext)
			target = m_container->GetContainedObjectByNumber(desiredClass, n + 1);
	}
	return target;
}

TAppleObject *TAppleObject::GetObjectProperty(DescType desiredClass, DescType keyForm, const AEDesc *keyData) {
	return NULL;
}

long TAppleObject::CountContainedObjects(DescType desiredClass) {
	return GetCount();
}

Boolean TAppleObject::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	int n1 = GetObjectNumber();
	int n2 = obj->GetObjectNumber();
	if (oper == kAEGreaterThan)
		return n1 > n2;
	else if (oper == kAEGreaterThanEquals)
		return n1 >= n2;
	else if (oper == kAEEquals)
		return n1 == n2;
	else if (oper == kAELessThan)
		return n1 < n2;
	else if (oper == kAELessThanEquals)
		return n1 < n2;
	else
		return FALSE;
}

/*
	GetObjSpecifier
	IuWFNgXyVt@CA𓾂
	ẽIuWFNgXyVt@CA𓾂Ă玩̕
	̍ċAł͂낤
	NX̎DescTypełȂAvꂽDescTypeg
	Ƃ肠ō
*/
OSErr TAppleObject::GetObjSpecifier(DescType desiredClass, AEDesc *objSpec) {
	OSErr err;
	TAEDesc container;
	if (m_container != NULL) {
		err = m_container->GetObjSpecifier(m_container->GetObjectType(), container);
		if (err != noErr)
			return err;
	}
	TAEDesc keyData;
	long num = GetObjectNumber();
	err = keyData.AECreateDesc(typeLongInteger, (Ptr)&num, sizeof num);
	if (err != noErr)
		return err;
	err = ::CreateObjSpecifier(desiredClass, container, formAbsolutePosition, keyData, FALSE, objSpec);
	return err;
}

/*
	ThroughObjSpecifier
	ۂ̃IuWFNg̊KwƘ_IȊKwႤƂAԂ̃NX
	XyVt@CAȂ悤ɂȂĂ͂ȂȂ
	ŃXyVt@CA쐬eɈ˗邾̃\bhpӂĂ
	GetObjSpecifierI[o[Ch֐ŌĂł炤
*/
OSErr TAppleObject::ThroughObjSpecifier(DescType desiredClass, AEDesc *objSpec) {
	if (m_container != NULL)
		return m_container->GetObjSpecifier(m_container->GetObjectType(), objSpec);
	return noErr;
}

/*
	TAppleObjectArray
	XgԂ߂̃NX
*/

TAppleObjectArray::TAppleObjectArray(int n, TAppleObject *container) {
	m_container = container;
	m_obj = NULL;
	m_count = 0;
	if (n > 0) {
		m_obj = new TAppleObject *[n];
		m_count = n;
	}
}

TAppleObjectArray::~TAppleObjectArray() {
	if (m_obj != NULL)
		delete [] m_obj;
}

TAppleObject *TAppleObjectArray::GetNthObj(int n) {
	if (n >= 0 && n < m_count)
		return m_obj[n];
	else
		return NULL;
}

void TAppleObjectArray::SetNthObj(int n, TAppleObject *obj) {
	if (n >= 0 && n < m_count)
		m_obj[n] = obj;
}

OSErr TAppleObjectArray::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	if (m_count == 0)
		return noErr;
	TAEDescList objList;
	objList.AECreateList();
	if (objList.IsGood() == FALSE)
		return -1;
	AEKeyword keyword = typeNull;
	for (int i = 0; i < m_count; i++) {
		if (m_obj[i] == NULL)
			return -1;
		TAEResult r;
		OSErr err = m_obj[i]->OnAppleEvent(eventID, desiredClass, aev, &r);
		if (err != noErr)
			return err;

		objList.AEPutDesc(0, &r);
		if (objList.IsGood() == FALSE)
			return -1;
		keyword = r.m_keyword;
	}
	result->PutDescWithKeyword(keyword, objList);
	if (result->IsGood() == FALSE)
		return -1;
	return noErr;
}

TAppleObject *TAppleObjectArray::GetContainedObject(DescType desiredClass, DescType keyForm, const AEDesc *keyData) {
	if (m_count == 0)
		return NULL;
	TAppleObjectArray *array = new TAppleObjectArray(m_count, m_container);
	for (int i = 0; i < m_count; i++) {
		TAppleObject *obj = m_obj[i]->GetContainedObject(desiredClass, keyForm, keyData);
		if (obj == NULL) {
			delete array;
			return NULL;
		}
		array->SetNthObj(i, obj);
	}
	return array;
}

TAppleObject *TAppleObjectArray::GetObjectProperty(DescType desiredClass, DescType keyForm, const AEDesc *keyData) {
	if (m_count == 0)
		return NULL;
	TAppleObjectArray *array = new TAppleObjectArray(m_count, m_container);
	for (int i = 0; i < m_count; i++) {
		TAppleObject *obj = m_obj[i]->GetObjectProperty(desiredClass, keyForm, keyData);
		if (obj == NULL) {
			delete array;
			return NULL;
		}
		array->SetNthObj(i, obj);
	}
	return array;
}

OSErr TAppleObjectArray::OnAdhocForContainer(TToken *token, AEEventID eventID, const AppleEvent *aev, TAEResult *result) {
	return m_container == NULL ? noErr : m_container->OnAdhocForContainer(token, eventID, aev, result);
}
