/*
	CStd.cpp
 
	1998.05.16
	1999.11.04
*/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"

extern "C" {
	void *CCMN_memcpy(void *dst, const void *src, size_t count);
	char *CCMN_strcpy(char *dst, const char *src);
	void *CCMN_malloc(size_t count);
	void *CCMN_calloc(size_t nmemb, size_t count);
	void CCMN_free(void *p);
}

extern "C" void DebugLog(char *s);
extern void FatalErrorDlg(int errorNumber);

/*
	malloc  free
*/

//#define MEMORY_SELF_CONTROL

extern "C" void *CCMN_malloc(size_t byteCount) {
	if (byteCount == 0)
		FatalErrorDlg(1);
	void *p = NULL;
#ifdef MEMORY_SELF_CONTROL

	p = ::NewPtr(byteCount);
#else

	p = malloc(byteCount);
#endif

	if (p == NULL)
		FatalErrorDlg(2);
	return p;
}

extern "C" void *CCMN_calloc(size_t nmemb, size_t count) {
	return CCMN_malloc(nmemb * count);
}

extern "C" void CCMN_free(void *p) {
	if (p == NULL)
		FatalErrorDlg(3);
#ifdef MEMORY_SELF_CONTROL

	::DisposePtr((char *)p);
#else

	free(p);
#endif
}

/*
	new  delete
*/

void *operator new(size_t byteCount) {
	return CCMN_malloc(byteCount);
}

void operator delete(void *p) {
	CCMN_free(p);
}

void *operator new[](size_t byteCount) {
	return CCMN_malloc(byteCount);
}

void operator delete[](void *p) {
	CCMN_free(p);
}

/*
	memcpy  strcpy
*/

extern "C" void *CCMN_memcpy(void *dst, const void *src, size_t count) {
	if (count <= 0)
		return dst;
	if (src == NULL || dst == NULL)
		FatalErrorDlg(10);
	return memcpy(dst, src, count);
}

extern "C" char *CCMN_strcpy(char *dst, const char *src) {
	if (src == NULL || dst == NULL)
		FatalErrorDlg(11);
	//return std::strcpy(dst, src);
	return strcpy(dst, src);
}


