/*
	TDialog.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.03.08	1.0			Takeshi Yoneki
*/

#include "csystem.h"

pascal Boolean TDialog_DialogFilter(DialogRef dref, EventRecord *ev, short *itemHit);
pascal Boolean TDialog_DialogFilter(DialogRef dref, EventRecord *ev, short *itemHit) {
	Boolean result = FALSE;
	TDialog *dlg = (TDialog *)theWindowArray->GetByWindowRef(::GetDialogWindow(dref));
	if (dlg != NULL) {
		result = dlg->DialogFilter(dref, ev, itemHit);
		if (theSystem->hasThreadMgr)
			YieldToAnyThread();
	}
	return result;
}

void TDialog::Initial() {
	m_created = FALSE;
	m_uppModalFilter.SetUPP(TDialog_DialogFilter);
}

TDialog::TDialog()
		: TWindow() {
	Initial();
}

TDialog::TDialog(DialogPtr dlg)
		: TWindow() {
	Initial();
	m_port = ::GetDialogWindow(dlg);
}

TDialog::TDialog(short dialogID, TPoint position, WindowRef behind)
		: TWindow() {
	Initial();
	GetNewDialog(dialogID, position, behind);
}

TDialog::~TDialog() {
	DisposeDialog();
}

void TDialog::NewDialog(const Rect *boundsRect, ConstStr255Param title, Boolean visible, short procID, WindowRef behind, Boolean goAwayFlag, long refCon, Handle itmLstHndl) {
	m_port = ::GetDialogWindow(::NewDialog(NULL, boundsRect, title, visible, procID, behind, goAwayFlag, refCon, itmLstHndl));
	m_created = TRUE;
}

void TDialog::GetNewDialog(short dialogID, TPoint position, WindowRef behind) {
	m_port = ::GetDialogWindow(::GetNewDialog(dialogID, NULL, behind));
	m_created = TRUE;
	SetPort();
	if (position.h == 0 && position.v == 0) {
		TRect box = GetPortRect();
		box.LocalToGlobal();
		TRect sr = theSystem->GetDesktopRect();
		position.h = (sr.right - box.GetWidth()) / 2;
		position.v = box.top;
	}
	MoveWindow(position.h, position.v);
	ShowWindow();
}

void TDialog::DisposeDialog() {
	if (m_created && m_port != NULL)
		::DisposeDialog(*this);
	m_port = NULL;
	theCursor->Reset();
}

Boolean TDialog::OnItemHit(short itemHit) {
	return FALSE;
}

void TDialog::Update(int message) {
	TWindow *win = theWindowArray->GetByWindowRef((WindowRef)message);
	if (win != NULL)
		win->OnUpdate();
}

void TDialog::OnUpdate() {
	TSavePort savePort(this);
	BeginUpdate();
	UpdateDialog(GetVisRgn());
	DrawOKWaku();
	EndUpdate();
	UpdateCallback();	/* 2000.10.30 \łƔfȂBegin-End͂ */
}

void TDialog::UpdateCallback() {
}

void TDialog::OnCut() {
	ClearCurrentScrap();
	DialogCut();
	TEToScrap();
}

void TDialog::OnCopy() {
	ClearCurrentScrap();
	DialogCopy();
	TEToScrap();
}

void TDialog::OnPaste() {
	TScrapStuff scrapInfo;
	int n = scrapInfo.GetScrapTextSize();
	if (n > 255) {
		//		TString s;
		//		s.sprintf("TDialog::OnPaste %d", n);
		//		theDebug->Log(s);
		SysBeep(0);
		return;
	}
	OSErr err = TEFromScrap();
	if (err != noErr) {
		//		SysBeep(0);
	}
	else {
		DialogPaste();
	}
}

Handle TDialog::GetDialogItem(short itemNo) {
	Handle item;
	Rect box;
	short itemType;
	OSErr err = -1;
	if (theSystem->hasAppearance)
		err = ::GetDialogItemAsControl(*this, itemNo, (ControlRef*)&item);
	if (err != noErr)
		::GetDialogItem(*this, itemNo, &itemType, &item, &box);
	return item;
}

void TDialog::SetDIHilite(short itemNumber, short itemValue) {
	Handle item = GetDialogItem(itemNumber);
	HiliteControl((ControlRef)item, itemValue);
}

void TDialog::SetDITitle(short itemNumber, TString itemText) {
	Rect box = GetDIRect(itemNumber);
	::SetControlTitle((ControlHandle)GetDialogItem(itemNumber), itemText);
	ValidWindowRect(m_port, &box);
}

short TDialog::SetDIText(short itemNumber, TString itemText, Boolean useControl) {
	if (useControl)
		return SetDIControlText(itemNumber, itemText);
	SetDialogItemText(GetDialogItem(itemNumber), itemText);
	return itemText.GetLength();
}

void TDialog::SetDITextAndSelect(short itemNumber, TString itemText, Boolean useControl) {
	if (useControl)
		return SetDIControlTextAndSelect(itemNumber, itemText);
	SelectDialogItemText(itemNumber, 0, SetDIText(itemNumber, itemText));
}

TString TDialog::GetDIText(short itemNumber, Boolean useControl) {
	if (useControl)
		return GetDIControlText(itemNumber);
	Str255 itemText;
	GetDialogItemText(GetDialogItem(itemNumber), itemText);
	return itemText;
}

short TDialog::SetDITextCFString(short itemNumber, TCFString itemText, Boolean dmy) {
	CFStringRef r = itemText.GetCFStringRef();
	SetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextCFStringTag, sizeof(CFStringRef), (Ptr)&r);
	return itemText.GetLength();
}

void TDialog::SetDITextCFStringAndSelect(short itemNumber, TCFString itemText, Boolean dmy) {
	ControlEditTextSelectionRec sel;
	sel.selStart = 0;
	sel.selEnd = SetDITextCFString(itemNumber, itemText);
	SetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextSelectionTag, sizeof(sel), (Ptr)&sel);
}

TCFString TDialog::GetDITextCFString(short itemNumber, Boolean dmy) {
	CFStringRef r = NULL;
	Size outActualSize = 0;
	GetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextCFStringTag, sizeof(CFStringRef), (Ptr)&r, &outActualSize);
	if (r == NULL)
		return "";
	return r;
}

short TDialog::SetDINumber(short itemNumber, long itemValue) {
	TString s;
	s.sprintf("%ld", itemValue);
	return SetDIText(itemNumber, s);
}

void TDialog::SetDINumberAndSelect(short itemNumber, long itemValue) {
	SelectDialogItemText(itemNumber, 0, SetDINumber(itemNumber, itemValue));
}

long TDialog::GetDINumber(short itemNumber) {
	return GetDIText(itemNumber).atoi();
}

void TDialog::SetDIValue(short itemNumber, short value) {
	Handle item = GetDialogItem(itemNumber);
	SetControlValue((ControlRef)item, value);
}

short TDialog::GetDIValue(short itemNumber) {
	Handle item = GetDialogItem(itemNumber);
	return GetControlValue((ControlRef)item);
}

OSErr TDialog::GetDIControlData(short itemNumber, ControlPartCode inPart, ResType inTagName, Size inBufferSize, Ptr inBuffer, Size *outActualSize) {
	ControlRef item = GetDialogItemAsControl(itemNumber);
	return GetControlData(item, inPart, inTagName, inBufferSize, inBuffer, outActualSize);
}

OSErr TDialog::SetDIControlData(short itemNumber, ControlPartCode inPart, ResType inTagName, Size inBufferSize, Ptr inBuffer) {
	ControlRef item = GetDialogItemAsControl(itemNumber);
	OSErr err = SetControlData(item, inPart, inTagName, inBufferSize, inBuffer);
	if (err == noErr)
		DrawOneControl(item);
	return err;
}

short TDialog::SetDIControlText(short itemNumber, TString itemText) {
	OSErr err = SetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextTextTag, itemText.GetLength(), (char *)itemText);
	return itemText.GetLength();
}

void TDialog::SetDIControlTextAndSelect(short itemNumber, TString itemText) {
	ControlEditTextSelectionRec sel;
	sel.selStart = 0;
	sel.selEnd = SetDIControlText(itemNumber, itemText);
	SetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextSelectionTag, sizeof(sel), (Ptr)&sel);
}

TString TDialog::GetDIControlText(short itemNumber) {
	char itemText[256];
	long sz;
	memset(itemText, 0, sizeof itemText);
	OSErr err = GetDIControlData(itemNumber, kControlEditTextPart, kControlEditTextTextTag, 255, itemText, &sz);
	return itemText;
}

short TDialog::GetDIBevelButtonMenuValue(short itemNumber) {
	short n;
	Size sz;
	if (GetDIControlData(itemNumber, kControlMenuPart, kControlBevelButtonMenuValueTag, 2, (Ptr)&n, &sz) == noErr)
		return n;
	return 0;
}

void TDialog::SetDIKeyboardFocus(short itemNumber) {
	if (theSystem->hasAppearance && theSystem->systemVersion >= 0x0800) {
		ControlRef item = GetDialogItemAsControl(itemNumber);
		::SetKeyboardFocus(*this, item, kControlEditTextPart);
	}
}

void TDialog::DrawOKWaku(short itemNumber) {
	Rect box;
	short itemType;
	Handle item = GetDialogItem(itemNumber, &itemType, &box);
	if (itemType == ctrlItem + btnCtrl) {
		if (theSystem->systemVersion >= 0x0800) {
			::SetDialogDefaultItem(*this, itemNumber);
		}
		else {
			SetPort();
			PenSize(3, 3);
			TRect r(box.left - 4, box.top - 4, box.right + 4, box.bottom + 4);
			r.FrameRoundRect(15, 15);
			PenSize(1, 1);
		}
	}
}

Boolean TDialog::DialogFilter(DialogRef dref, EventRecord *ev, short *itemHit) {
	TSavePort savePort(this);

	Boolean flg = FALSE;
	if (ev->what == keyDown) {
		Boolean command = (ev->modifiers & cmdKey) != 0;
		int code = (ev->message & charCodeMask);
		if (command && code == '.') {
			flg = TRUE;
			*itemHit = DEFAULT_CANCEL_DITEM;
			SetDIHilite(*itemHit, 1);
		}
		else if (command) {
			flg = TRUE;
			switch (code) {
			case 'x':
			case 'X':
				OnCut();
				break;
			case 'c':
			case 'C':
				OnCopy();
				break;
			case 'v':
			case 'V':
				OnPaste();
				break;
			default:
				flg = FALSE;
				break;
			}
		}
		else {
			flg = TRUE;
			switch (code) {
			case 0x0D:
			case 0x03:
				*itemHit = DEFAULT_OK_DITEM;
				SetDIHilite(*itemHit, 1);
				break;
			case 0x1B:
				*itemHit = DEFAULT_CANCEL_DITEM;
				SetDIHilite(*itemHit, 1);
				break;
			default:
				flg = FALSE;
			}
		}
	}
	else if (ev->what == updateEvt) {
		if (dref == (DialogRef)ev->message)
			OnUpdate();
		else
			Update(ev->message);
	}

	return flg;
}

short TDialog::ModalDialog() {
	theCursor->Reset();
	short itemHit;
	//	::ModalDialog(NULL, &itemHit);
	::ModalDialog((ModalFilterUPP)m_uppModalFilter.GetUPP(), &itemHit);
	return itemHit;
}

Boolean TSaveConfirmDlg::DialogFilter(DialogRef dref, EventRecord *ev, short *itemHit) {
	TSavePort savePort(this);

	Boolean flg = FALSE;
	if (ev->what == keyDown) {
		Boolean command = (ev->modifiers & cmdKey) != 0;
		int code = (ev->message & charCodeMask);
		if (command && code == '.') {
			flg = TRUE;
			*itemHit = DEFAULT_CANCEL_DITEM;
			SetDIHilite(*itemHit, 1);
		}
		else {
			flg = TRUE;
			switch (code) {
			case 0x0D:
			case 0x03:
			case 'S':
			case 's':
				*itemHit = DEFAULT_OK_DITEM;
				SetDIHilite(*itemHit, 1);
				break;
			case 0x1B:
			case 'C':
			case 'c':
				*itemHit = DEFAULT_CANCEL_DITEM;
				SetDIHilite(*itemHit, 1);
				break;
			case 'D':
			case 'd':
				*itemHit = DEFAULT_DISCARD_DITEM;
				SetDIHilite(*itemHit, 1);
				break;
			default:
				flg = FALSE;
			}
		}
	}
	else if (ev->what == updateEvt) {
		if (dref == (DialogRef)ev->message)
			OnUpdate();
		else
			Update(ev->message);
	}
	return flg;
}
