/*
	TDrag.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1997.01.06	1.5			Takeshi Yoneki
*/

#include "csystem.h"

/*
	TDrag
*/


void TDrag::Initial() {
	m_drag = NULL;
	m_created = FALSE;
	m_error = noErr;
}

TDrag::TDrag() {
	Initial();
	m_error = NewDrag(&m_drag);
	if (m_error == noErr)
		m_created = TRUE;
}

TDrag::TDrag(DragReference drag) {
	Initial();
	m_drag = drag;
}

TDrag::~TDrag() {
	if (m_created)
		DisposeDrag(m_drag);
}

DragAttributes TDrag::GetDragAttributes() {
	DragAttributes attr;
	m_error = ::GetDragAttributes(m_drag, &attr);
	return attr;
}

unsigned short TDrag::CountDragItems() {
	unsigned short count;
	m_error = ::CountDragItems(m_drag, &count);
	return count;
}

ItemReference TDrag::GetDragItemReferenceNumber(unsigned short index) {
	ItemReference item;
	m_error = ::GetDragItemReferenceNumber(m_drag, index, &item);
	return item;
}

FlavorType TDrag::GetFlavorType(ItemReference theItemRef, unsigned short index) {
	FlavorType theType;
	m_error = ::GetFlavorType(m_drag, theItemRef, index, &theType);
	return theType;
}

FlavorFlags TDrag::GetFlavorFlags(ItemReference theItemRef, FlavorType theType) {
	FlavorFlags theFlags;
	m_error = ::GetFlavorFlags(m_drag, theItemRef, theType, &theFlags);
	return theFlags;
}

Size TDrag::GetFlavorDataSize(ItemReference theItemRef, FlavorType theType) {
	Size dataSize;
	m_error = ::GetFlavorDataSize(m_drag, theItemRef, theType, &dataSize);
	return dataSize;
}

Point TDrag::GetDragMouse(Point *pinnedMouse) {
	Point mouse;
	m_error = ::GetDragMouse(m_drag, &mouse, pinnedMouse);
	return mouse;
}

unsigned short TDrag::CountDragItemFlavors(ItemReference theItemRef) {
	unsigned short count;
	m_error = ::CountDragItemFlavors(m_drag, theItemRef, &count);
	return count;
}
/*
	TAutoOffScreen
*/

TAutoOffScreen::TAutoOffScreen() {
	m_port = NULL;
	m_device = NULL;
	m_world = NULL;
	m_good = FALSE;
}

TAutoOffScreen::~TAutoOffScreen() {
	if (m_world != NULL)
		DisposeGWorld(m_world);
}

RGBColor rgbWhiteColor = {0xFFFF, 0xFFFF, 0xFFFF};
RGBColor rgbBlackColor = {0, 0, 0};

GrafPtr TAutoOffScreen::DrawOff(GrafPtr inPort) {
	m_good = FALSE;
	m_savePort = inPort;
	SetPort(m_savePort);

	::GetBackColor(&m_saveBgColor);
	::GetForeColor(&m_saveFgColor);

	GetGWorld(&m_port, &m_device);

	TRect r;
	::GetPortBounds((CGrafPtr)m_savePort, r);
	r.LocalToGlobal();

	if (NewGWorld(&m_world, 0, r, 0L, 0L, 0) != noErr)
		return m_savePort;

	SetGWorld(m_world, 0L);
	if (LockPixels(GetPortPixMap(m_world)) == FALSE)
		return m_savePort;

	TRect rw;
	::GetPortBounds((CGrafPtr)m_savePort, r);
	::GetPortBounds((CGrafPtr)m_world, rw);
	CopyBits(GetPortBitMapForCopyBits((CGrafPtr)m_savePort),
	         GetPortBitMapForCopyBits((CGrafPtr)m_world),
	         r,
	         rw,
	         srcCopy, NULL);

	RGBBackColor(&m_saveBgColor);
	RGBForeColor(&m_saveFgColor);

	m_good = TRUE;
	return (GrafPtr)m_world;
}

GrafPtr TAutoOffScreen::DrawOn() {
	if (m_good == FALSE)
		return m_savePort;
	SetGWorld(m_port, m_device);

	RGBBackColor(&rgbWhiteColor);
	RGBForeColor(&rgbBlackColor);

	TRect rw, r;
	::GetPortBounds((CGrafPtr)m_savePort, r);
	::GetPortBounds((CGrafPtr)m_world, rw);

	CopyBits(GetPortBitMapForCopyBits((CGrafPtr)m_world),
	         GetPortBitMapForCopyBits(m_port),
	         rw,
	         r,
	         srcCopy, NULL);

	RGBBackColor(&m_saveBgColor);
	RGBForeColor(&m_saveFgColor);

	UnlockPixels(GetPortPixMap(m_world));
	DisposeGWorld(m_world);
	m_world = NULL;

	return m_savePort;
}
