/*
	TRect.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.12	1.0			Takeshi Yoneki
*/

#include "csystem.h"

/*
	TRect
*/

TRect TRect::SectRect(TRect& cr) {
	TRect dest;
	::SectRect(this, &cr, &dest);
	return dest;
}

void TRect::FrameRectWithShadow() {
	MoveTo(left + 2, bottom);
	LineTo(right, bottom);
	LineTo(right, top + 2);
	FrameRect();
}

void TRect::FrameRectOut() {
	TRect r(left - 1, top - 1, right + 1, bottom + 1);
	r.FrameRect();
}

TRect TRect::GetInRect() {
	TRect r(left + 1, top + 1, right - 1, bottom - 1);
	if (r.left >= r.right || r.top >= r.bottom)
		r.SetRect(0, 0, 0, 0);
	return r;
}

void TRect::EraseRectIn() {
	GetInRect().EraseRect();
}

void TRect::InvertRectIn(Boolean color) {
	GetInRect().InvertRect(color);
}

void TRect::InvalRectIn() {
	GetInRect().InvalRect();
}

void TRect::InvertRect(Boolean color) {
	if (color)
		LMSetHiliteMode(LMGetHiliteMode() & hiliteBit);
	::InvertRect(this);
}

void TRect::LocalToGlobal() {
	TPoint pt1(left, top);
	TPoint pt2(right, bottom);
	pt1.LocalToGlobal();
	pt2.LocalToGlobal();
	Pt2Rect(pt1, pt2);
}

void TRect::GlobalToLocal() {
	TPoint pt1(left, top);
	TPoint pt2(right, bottom);
	pt1.GlobalToLocal();
	pt2.GlobalToLocal();
	Pt2Rect(pt1, pt2);
}

TPoint TRect::GetWidthHeight() {
	return TPoint(GetWidth(), GetHeight());
}

TPoint TRect::GetLeftTop() {
	return TPoint(left, top);
}

TPoint TRect::GetRightBottom() {
	return TPoint(right, bottom);
}

void TRect::PaintGrayPanel(TWindow *win, Boolean vertical) {
	if (EmptyRect())
		return;

	int depth = (win == NULL ? theSystem->GetMainDepth() : depth = win->GetDepth());

	if (depth >= 8 && theSystem->systemVersion >= 0x0700) {
		TRect r;
		Boolean erased = FALSE;
		if (theSystem->appearanceVersion >= 0x0101 && theSystem->systemVersion >= 0x0850) {
			::SetThemeBackground(kThemeBrushDialogBackgroundActive, depth, TRUE);
			EraseRect();
			::NormalizeThemeDrawingState();
			erased = TRUE;
		}
		if (erased == FALSE)
			FillCRect(*theSystem->grayPixPat);

		if (theSystem->systemVersion >= 0x1000) {
			if (vertical)
				r.FillCRect(*theSystem->grayPixPat, right - 1, top, right, bottom);
			else
				r.FillCRect(*theSystem->grayPixPat, left, bottom - 1, right, bottom);
		}
		else {
			if (vertical) {
				r.FillCRect(*theSystem->blackPixPat, right - 1, top, right, bottom);
				if (GetWidth() > 1)
					r.FillCRect(*theSystem->deepPixPat, right - 2, top, right - 1, bottom);
				if (GetWidth() > 3)
					r.FillCRect(*theSystem->whitePixPat, left, top, left + 1, bottom);
			}
			else {
				r.FillCRect(*theSystem->blackPixPat, left, bottom - 1, right, bottom);
				if (GetHeight() > 1)
					r.FillCRect(*theSystem->deepPixPat, left, bottom - 2, right, bottom - 1);
				if (GetHeight() > 3)
					r.FillCRect(*theSystem->whitePixPat, left, top, right, top + 1);
			}
		}
	}
	else {
		EraseRect();
		MoveTo(left, bottom - 1);
		LineTo(right, bottom - 1);
	}
}

/*
	TRgn
*/

TRgn::TRgn() {
	NewRgn();
}

TRgn::TRgn(const TRgn& crgn) {
	NewRgn();
	*this = crgn;
}

TRgn::TRgn(RgnHandle rgn) {
	NewRgn();
	*this = rgn;
}

TRgn::TRgn(Rect r) {
	NewRgn();
	*this = r;
}

TRgn::~TRgn() {
	if (m_rgn != NULL)
		DisposeRgn();
}

TRgn TRgn::UnionRgn(RgnHandle rgn) {
	TRgn dst;
	::UnionRgn(m_rgn, rgn, dst);
	return dst;
}

TRgn TRgn::SectRgn(RgnHandle rgn) {
	TRgn dst;
	::SectRgn(m_rgn, rgn, dst);
	return dst;
}

void TRgn::BeFrame() {
	TRgn tmpRgn(m_rgn);
	tmpRgn.InsetRgn(1, 1);
	DiffRgn(m_rgn, tmpRgn);
}

void TRgn::LocalToGlobal() {
	TPoint pt(0, 0);
	pt.LocalToGlobal();
	OffsetRgn(pt.h, pt.v);
}

TRect TRgn::GetBBox() {
	Rect r;
	::GetRegionBounds(m_rgn, &r);
	return r;
}

/*
	TCursor
*/

void TCursor::Initial() {
	m_cur = NULL;
	m_cursorID = 0;
	m_count = 0;
	m_intervalTick = 0;
	m_prevTick = 0;
	m_current = 0;
}

TCursor::TCursor() {
	Initial();
}

TCursor::TCursor(short cursorID, int count, int intervalTick) {
	Initial();
	GetCursor(cursorID, count, intervalTick);
}

void TCursor::GetCursor(short cursorID, int count, int intervalTick) {
	if (count == 0)
		return;
	m_count = count;
	m_intervalTick = intervalTick;
	m_cur = new CursHandle[m_count];
	m_cursorID = cursorID;
	for (int i = 0; i < m_count; i++) {
		m_cur[i] = ::GetCursor(cursorID + i);
		HNoPurge((Handle)m_cur[i]);
	}
}

void TCursor::SetCursor() {
	if (m_count == 0)
		return;
	if (TickCount() - m_prevTick >= m_intervalTick) {
		::SetCursor(*m_cur[m_current]);
		m_prevTick = TickCount();
		m_current += 1;
		if (m_current == m_count)
			m_current = 0;
	}
}

TCursorList::TCursorList() {
	m_current = 0;
}

void TCursorList::Load(short cursorID, int count, int intervalTick) {
	if (cursorID != arrowCursor) {
		TCursor *cursor = new TCursor(cursorID, count, intervalTick);
		Append(cursor);
	}
}

void TCursorList::Set(short cursorID, Boolean force) {
	if ((!force && m_current == cursorID) || cursorID == otherCursor) {
	}
	else if (cursorID == arrowCursor) {
		InitCursor();
	}
	else {
		TCursor *cursor = GetByID(cursorID);
		if (cursor != NULL)
			cursor->SetCursor();
	}
	m_current = cursorID;
}

void TCursorList::Reset() {
	m_current = arrowCursor;
	InitCursor();
}

TCursor *TCursorList::GetByID(short cursorID) {
	TCursor *cursor;
	for (cursor = GetTop(); cursor != NULL; cursor = cursor->GetNext()) {
		if (cursor->m_cursorID == cursorID)
			return cursor;
	}
	return NULL;
}

/*
	tHgNX
*/

TFont::TFont() {
	m_fontNum = 0;
}

TFont::TFont(TFont& font) {
	m_fontNum = font.m_fontNum;
}

TFont::TFont(short fontNum) {
	m_fontNum = fontNum;
}

TFont::TFont(TString name) {
	GetFNum(name);
}

TString TFont::GetFontName() {
	Str255 name;
	::GetFontName(m_fontNum, name);
	return name;
}

/*
	sN`[NX
*/

TPicture::TPicture() {
	m_pic = NULL;
	m_isResource = FALSE;
}

TPicture::~TPicture() {
	if (m_pic != NULL) {
		if (m_isResource)
			ReleaseResource((Handle)m_pic);
		else
			KillPicture(m_pic);
	}
}

void TPicture::GetPicture(short pictureID) {
	m_pic = ::GetPicture(pictureID);
	HNoPurge((Handle)m_pic);
	m_isResource = TRUE;
}

/*
	TIconSuite
	ACRgNX
*/

TIconSuite::TIconSuite() {
	m_icon = NULL;
}

TIconSuite::~TIconSuite() {
	if (m_icon != NULL)
		DisposeIconSuite(m_icon, TRUE);
}

TIconRef::TIconRef() {
	m_ref = NULL;
	m_label = 0;
	m_creator = 0;
	m_iconType = 0;
}

TIconRef::~TIconRef() {
	if (m_creator != 0 || m_iconType != 0)
		UnregisterIconRef(m_creator, m_iconType);
	if (m_ref != NULL)
		ReleaseIconRef(m_ref);
}

OSErr TIconRef::GetFromResource(short resID, OSType creator, OSType iconType) {
	OSErr err;
	TIconSuite suite;
	suite.GetIconSuite(resID);
	IconFamilyHandle fam;
	err = IconSuiteToIconFamily(suite, kSelectorAllAvailableData, &fam);
	if (err != noErr)
		return err;
	err = RegisterIconRefFromIconFamily(creator, iconType, fam);
	::ReleaseResource((Handle)fam);
	if (err != noErr)
		return err;

	m_creator = creator;
	m_iconType = iconType;

	return err;
}

/*
	TPixPat
	J[sNZp^[
*/

void TPixPat::NewPixPat() {
	m_ppat = ::NewPixPat();
	HNoPurge((Handle)m_ppat);
}

void TPixPat::GetPixPat(short patID) {
	m_ppat = ::GetPixPat(patID);
	HNoPurge((Handle)m_ppat);
}

void TPixPat::InitAndMake(const RGBColor *myColor) {
	NewPixPat();
	MakeRGBPat(myColor);
}

TPixPat::~TPixPat() {
	if (m_ppat != NULL)
		DisposePixPat(m_ppat);
}

/*
*/

void TLongPoint::Initial() {
	h = 0;
	v = 0;
}

TLongPoint::TLongPoint() {
	Initial();
}

TLongPoint::TLongPoint(long x, long y) {
	Initial();
	SetPt(x, y);
}

void TLongPoint::operator=(const TPoint& pt) {
	h = pt.h;
	v = pt.v;
}

TLongPoint::operator TPoint() const {
	return TPoint(h, v);
}

void TLongPoint::SetPt(long x, long y) {
	h = x;
	v = y;
}

void TLongRect::Initial() {
	left = 0;
	top = 0;
	right = 0;
	bottom = 0;
}

TLongRect::TLongRect() {
	Initial();
}

TLongRect::TLongRect(const TRect& r) {
	Initial();
	*this = r;
}

void TLongRect::operator=(const TRect& r) {
	left = r.left;
	top = r.top;
	right = r.right;
	bottom = r.bottom;
}

TLongRect::operator TRect() const {
	return TRect(left, top, right, bottom);
}

void TLongRect::SetRect(long l, long t, long r, long b) {
	left = l;
	top = t;
	right = r;
	bottom = b;
}

void TLongRect::OffsetRect(long dh, long dv) {
	left += dh;
	right += dh;
	top += dv;
	bottom += dv;
}

Boolean TLongRect::PtInRect(TLongPoint pt) {
	if (pt.h <= left)
		return FALSE;
	if (pt.h >= right)
		return FALSE;
	if (pt.v <= top)
		return FALSE;
	if (pt.v >= bottom)
		return FALSE;
	return TRUE;
}
