/*
	CFile.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.12	1.0			Takeshi Yoneki
*/

#include "csystem.h"

void TDirectory::Init() {
	m_volume = 0;
	m_dirID = 0;
	m_error = noErr;
}

TDirectory::TDirectory() {
	Init();
}

TDirectory::TDirectory(const TDirectory& dir) {
	Init();
	*this = dir;
}

TDirectory::TDirectory(const TDirectory& dir, const TCFString dirName) {
	*this = TDirectory(dir.m_volume, dir.m_dirID, dirName);
}

TDirectory::TDirectory(const TFSSpec& spec, Boolean useName) {
	Init();
	if (useName) {
		*this = TDirectory(spec.GetDirectory(), spec.GetName());
	}
	else {
		m_volume = spec.GetVRefNum();
		m_dirID = spec.GetParID();
	}
}

TDirectory::TDirectory(const TString volumeName, long dirID) {
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	Str63 name;
	TString tmp;
	tmp.sprintf("%s:", (char *)StripPath(volumeName));
	tmp.CopyTo(name);
	pb.ioParam.ioNamePtr = name;
	pb.volumeParam.ioVolIndex = -1;
	m_error = PBHGetVInfoSync(&pb);
	if (m_error != noErr)
		return;
	m_volume = pb.ioParam.ioVRefNum;
	m_dirID = dirID;
	if (IsReal() == FALSE)		/* ݃`FbN */
		m_error = -1;
}

TDirectory::TDirectory(short volume, long dirID) {
	Init();
	m_volume = volume;
	m_dirID = dirID;
}

TDirectory::TDirectory(short volume, long dirID, TCFString dirName) {
	Init();
	m_volume = volume;
	m_dirID = dirID;
	if (dirName.IsEmpty() == FALSE) {
		CInfoPBRec cb;
		if (GetCatInfoByName(dirName, &cb) && (cb.dirInfo.ioFlAttrib & fsSBDrNmFls) != 0)
			m_dirID = cb.dirInfo.ioDrDirID;
	}
}

TDirectory::TDirectory(const TCFString& fullPath) {
	Init();
	*this = fullPath;
}

void TDirectory::operator=(const TDirectory& dir) {
	m_volume = dir.m_volume;
	m_dirID = dir.m_dirID;
	m_error = dir.m_error;
}

void TDirectory::operator=(const TFSSpec& spec) {
	m_volume = spec.GetVRefNum();
	m_dirID = spec.GetParID();
}

void TDirectory::operator=(const TFSRef& fsRef) {
	TFSSpec spec = fsRef.GetFSSpec();
	*this = TDirectory(spec.GetDirectory(), spec.GetName());
	//*this = spec;
}

void TDirectory::operator=(const TCFString& fullPath) {
	TFSSpec spec(TFSRef(fullPath, true));
	*this = TDirectory(spec, true);
	//	m_volume = spec.GetVRefNum();
	//	m_dirID = spec.GetParID();
}

short TDirectory::GetCatFileNum() {
	CInfoPBRec cb;
	memset(&cb, 0, sizeof cb);
	m_error = noErr;
	cb.dirInfo.ioCompletion = NULL;
	cb.dirInfo.ioNamePtr = NULL;
	cb.dirInfo.ioVRefNum = m_volume;
	cb.dirInfo.ioDrDirID = m_dirID;
	cb.dirInfo.ioFDirIndex = -1;
	m_error = PBGetCatInfoSync(&cb);
	if (m_error == noErr)
		return cb.dirInfo.ioDrNmFls;
	else
		return 0;
}

TDirectory TDirectory::GetParent() {
	TDirectory parent;
	parent.m_error = noErr;
	if (IsRoot()) {
		parent = *this;
		parent.m_dirID = 1;
		return parent;
	}
	CInfoPBRec cb;
	memset(&cb, 0, sizeof cb);
	cb.dirInfo.ioCompletion = NULL;
	cb.dirInfo.ioNamePtr = NULL;
	cb.dirInfo.ioVRefNum = m_volume;
	cb.dirInfo.ioDrDirID = m_dirID;
	cb.dirInfo.ioFDirIndex = -1;
	parent.m_error = PBGetCatInfoSync(&cb);
	if (parent.m_error == noErr) {
		parent.m_volume = cb.dirInfo.ioVRefNum;
		parent.m_dirID = cb.dirInfo.ioDrParID;
	}
	return parent;
}

Boolean TDirectory::GetPermissions(UInt32 *permissions) {
	if (theSystem->hasHFSPlus == FALSE)
		return FALSE;
	FSCatalogInfo cInfo;
	m_error = noErr;
	memset(&cInfo, 0, sizeof cInfo);
	//TFSSpec spec(*this);
	TFSRef fsRef(TFSSpec(*this));
	FSCatalogInfoBitmap which = kFSCatInfoPermissions;
	m_error = FSGetCatalogInfo(fsRef, which, &cInfo, NULL, NULL, NULL);
	if (m_error != noErr)
		return FALSE;
	if (permissions != NULL)
		memcpy(permissions, cInfo.permissions, sizeof (UInt32) * 4);
	return TRUE;
}

Boolean TDirectory::GetCatInfoByName(TCFString name, CInfoPBRec *pcb) {
	TFilePath filePath(name, *this);
	TString fileName = filePath.GetRealShortName();
	if (fileName.GetLength() == 0) {
		m_error = -1;
		return false;
	}
	CInfoPBRec cb;
	memset(&cb, 0, sizeof cb);
	m_error = noErr;
	Str63 fname;
	fileName.CopyTo(fname);
	cb.dirInfo.ioCompletion = NULL;
	cb.dirInfo.ioNamePtr = fname;
	cb.dirInfo.ioVRefNum = m_volume;
	cb.dirInfo.ioDrDirID = m_dirID;
	cb.dirInfo.ioFDirIndex = 0;
	m_error = PBGetCatInfoSync(&cb);
	if (m_error != noErr)
		return FALSE;
	if (pcb != NULL)
		memcpy(pcb, &cb, sizeof cb);
	return TRUE;
}

/*
Boolean TDirectory::GetCatInfoByLongName(CUniString name, FSCatalogInfo *catInfo)
{
	// w肵OFSRef
	TFSSpec parent(*this);
	FSRef fileRef;
	m_error = FSMakeFSRefUnicode(
		parent, name.GetLength(), name, name.GetHint(), &fileRef);
	// 2004.09.04 ݂Ȃt@CFSRef͍Ȃ悤B
	// Ȃ킿At@C̑݃`FbNFSMakeFSRefUnicodeŉ\ƂƂB
	if (m_error != noErr)
		return FALSE;
	// J^O𓾂
	FSCatalogInfo ci;
	memset(&ci, 0, sizeof ci);
	m_error = FSGetCatalogInfo(
		&fileRef, 
		kFSCatInfoGettableInfo, 
		&ci,
		NULL, NULL, NULL);
	if (m_error != noErr)
		return FALSE;
	if (catInfo != NULL)
		memcpy(catInfo, &ci, sizeof ci);
	return TRUE;
}
*/

TCFString TDirectory::GetCatInfoByDirIndex(short dirIndex, CInfoPBRec *cb) {
	memset(cb, 0, sizeof (CInfoPBRec));
	m_error = noErr;
	Str63 fname = "\p";
	cb->dirInfo.ioCompletion = NULL;
	cb->dirInfo.ioNamePtr = fname;
	cb->dirInfo.ioVRefNum = m_volume;
	cb->dirInfo.ioFDirIndex = dirIndex + 1;
	cb->dirInfo.ioDrDirID = m_dirID;
	m_error = PBGetCatInfoSync(cb);
	cb->dirInfo.ioNamePtr = NULL;
	if (m_error == noErr) {
		TFSSpec spec(*this, fname);
		TFSRef fsRef(spec);
		return fsRef.GetName();
		//return fname;
	}
	return "";
}

TCFString TDirectory::GetCatInfoByDirIndex(short dirIndex, long *attrib, long *fFlags, long *dirID) {
	CInfoPBRec cb;
	TCFString fname = GetCatInfoByDirIndex(dirIndex, &cb);
	if (m_error == noErr) {
		if (fFlags != NULL)
			*fFlags = cb.dirInfo.ioDrUsrWds.frFlags;
		if (attrib != NULL)
			*attrib = cb.dirInfo.ioFlAttrib;
		if (dirID != NULL)
			*dirID = cb.dirInfo.ioDrDirID;
	}
	return fname;
}

TCFString TDirectory::GetFInfoByDirIndex(short dirIndex, FInfo *info) {
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	m_error = noErr;
	Str63 name = "\p";
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = name;
	pb.fileParam.ioVRefNum = m_volume;
	pb.fileParam.ioDirID = m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = dirIndex + 1;
	m_error = PBHGetFInfoSync(&pb);
	if (m_error == noErr) {
		*info = pb.fileParam.ioFlFndrInfo;
		TFSSpec spec(*this, name);
		TFSRef fsRef(spec);
		return fsRef.GetName();
		//return name;
	}
	return "";
}

OSErr TDirectory::GetFInfoByName(TCFString fileName, FInfo *info) {
	TFilePath filePath(fileName, *this);
	TString shortName = filePath.GetRealShortName();
	if (shortName.GetLength() == 0) {
		m_error = -1;
		return -1;
	}
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	m_error = noErr;
	Str63 name;
	shortName.CopyTo(name);
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = name;
	pb.fileParam.ioVRefNum = m_volume;
	pb.fileParam.ioDirID = m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	m_error = PBHGetFInfoSync(&pb);
	if (m_error == noErr)
		*info = pb.fileParam.ioFlFndrInfo;
	return m_error;
}

OSErr TDirectory::DeleteFile(TCFString fileName) {
	if (IsNull())
		return FALSE;
	TFilePath filePath(fileName, *this);
	TFSRef fsRef(filePath);
	return FSDeleteObject(fsRef);
	/*
		HParamBlockRec pb;
		memset(&pb, 0, sizeof pb);
		pb.ioParam.ioNamePtr = (StringPtr)(ConstStr255Param)fileName;
		pb.ioParam.ioVRefNum = m_volume;
		pb.fileParam.ioDirID = m_dirID;
		OSErr err = PBHDeleteSync(&pb);
		return err;
	*/
}

OSErr TDirectory::RenameFile(TCFString oldName, TCFString newName) {
	TFilePath oldPath(oldName, *this);
	TFSRef oldFSRef(oldPath);
	THFSUniStr255 newUniName(newName);
	return FSRenameUnicode(oldFSRef, newUniName.GetLength(), newUniName, kTextEncodingUnknown, NULL);
	/*
		HParamBlockRec pb;
		memset(&pb, 0, sizeof pb);
		pb.ioParam.ioNamePtr = (StringPtr)(ConstStr255Param)oldName;
		pb.ioParam.ioVRefNum = m_volume;
		pb.fileParam.ioDirID = m_dirID;
		pb.ioParam.ioMisc = (char *)(StringPtr)(ConstStr255Param)newName;
		m_error = PBHRenameSync(&pb);
		return m_error;
	*/
}

OSErr TDirectory::RenameDirectory(TCFString newName) {
	TFSRef oldFSRef(*this);
	THFSUniStr255 newUniName(newName);
	return FSRenameUnicode(oldFSRef, newUniName.GetLength(), newUniName, kTextEncodingUnknown, NULL);
}

TDirectory TDirectory::CreateDir(TCFString dirName) {
	TDirectory dir;
	THFSUniStr255 newUniName(dirName);
	UInt32 dirID;
	TFSRef parent(*this);
	dir.m_error = FSCreateDirectoryUnicode(parent, newUniName.GetLength(), newUniName, kFSCatInfoNone, NULL, NULL, NULL, &dirID);
	if (dir.m_error == noErr)
		dir = TDirectory(m_volume, dirID);
	return dir;
	/*
		TDirectory dir;
		HParamBlockRec pb;
		memset(&pb, 0, sizeof pb);
		pb.ioParam.ioNamePtr = (StringPtr)(ConstStr255Param)dirName;
		pb.ioParam.ioVRefNum = m_volume;
		pb.fileParam.ioDirID = m_dirID;
		dir.m_error = PBDirCreateSync(&pb);
		if (dir.m_error == noErr)
			dir = TDirectory(m_volume, pb.fileParam.ioDirID);
		return dir;
	*/
}

void TDirectory::BeEmptyDir() {
	if (IsNull())
		return;
	for (;;) {
		short count = GetCatFileNum();
		if (count == 0)
			break;
		CInfoPBRec cb;
		TString name = GetCatInfoByDirIndex(0, &cb);
		if (cb.dirInfo.ioFlAttrib & fsSBDrNmFls) {
			TDirectory dir(*this, name);
			dir.BeEmptyDir();
		}
		DeleteFile(name);
	}
	return;
}

// pXΉ
TString TDirectory::PathToDirectory(TString path, TDirectory parent) {
	Init();
	/* ŏ͂̂܂܂̃pX */
	CInfoPBRec pcb;
	if (parent.GetCatInfoByName(path, &pcb)) {
		m_volume = pcb.dirInfo.ioVRefNum;
		m_dirID = pcb.dirInfo.ioDrDirID;
		if (IsReal())
			return path;
	}

	/* ̓tpXƂO */
	path = StripPath(path);
	TString volName = StripPathVol(path);
	TDirectory dir(volName, 0);
	if (dir.IsGood()) {
		*this = dir;
		if (IsVolumeNameOnly(path))
			return path;
		if (GetCatInfoByName(path, &pcb)) {
			m_volume = pcb.dirInfo.ioVRefNum;
			m_dirID = pcb.dirInfo.ioDrDirID;
			if (IsReal())
				return path;
		}
	}
	/* ς蕔pX */
	path.sprintf(":%s", (char *)path);
	if (GetCatInfoByName(path, &pcb)) {
		m_volume = pcb.dirInfo.ioVRefNum;
		m_dirID = pcb.dirInfo.ioDrDirID;
		if (IsReal())
			return path;
	}
	m_error = -1;
	return "";
}

TString TDirectory::GetVolumeName() {
	HParamBlockRec pb;
	Str63 name = "\p";
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioNamePtr = name;
	pb.volumeParam.ioVolIndex = 0;
	pb.ioParam.ioVRefNum = m_volume;
	m_error = PBHGetVInfoSync(&pb);
	if (m_error == noErr) {
		return name;
	}
	return "";
}

TString TDirectory::StripPath(TString path) {
	TString s = path.strtok_first(":");
	for (;;) {
		TString tmp = path.strtok_next(":");
		if (tmp.IsEmpty())
			return s;
		s.sprintf("%s:%s", (char *)s, (char *)tmp);
	}
}

TString TDirectory::StripPathVol(TString path) {
	return path.strtok_first(":");
}

Boolean TDirectory::IsVolumeNameOnly(TString path) {
	for (int i = 0; i < path.GetLength(); i++) {
		if (path[i] == ':')
			return FALSE;
	}
	return TRUE;
}

TCFString TDirectory::GetFullPath() {
	CInfoPBRec cpb;
	memset(&cpb, 0, sizeof cpb);
	Str255 nm = "\p";
	short volume = m_volume;
	long dirID = m_dirID;
	TCFString path, name;
	for (; dirID != 1;) {
		cpb.dirInfo.ioCompletion = NULL;
		cpb.dirInfo.ioNamePtr = nm;
		cpb.dirInfo.ioVRefNum = volume;
		cpb.dirInfo.ioDrDirID = dirID;
		cpb.dirInfo.ioFDirIndex = -1;
		m_error = PBGetCatInfoSync(&cpb);
		if (m_error != noErr)
			return "";
		TFSSpec spec(volume, cpb.dirInfo.ioDrParID, nm);
		TFSRef fsRef(spec);
		name = fsRef.GetName();
		if (path.IsEmpty()) {
			path = name;
		}
		else {
			if (path.GetLength() + 1 + name.GetLength() < 256)
				path = name + ":" + path;
			//path.sprintf("%s:%s", (char *)name, (char *)path);
			else
				return "";
		}
		dirID = cpb.dirInfo.ioDrParID;
	}
	return path;
}

TCFString TDirectory::GetLastToken() {
	if (IsReal() == FALSE)
		return "";
	TFSRef fsRef(*this);
	return fsRef.GetName();

	/*
		TString path = GetFullPath();
		if (path.IsEmpty())
			return "";
		char *delim = ":";
		TString token = path.strtok_first(delim);
		for (;;)
		{
			TString tmp = path.strtok_next(delim);
			if (tmp.IsEmpty())
				return token;
			token = tmp;
		}
	*/
}

Boolean TDirectory::IsReal() const {
	TFSSpec spec(*this);
	TFSRef fsRef;
	return fsRef.MakeByFSSpec(spec) == noErr;
	/*
		return GetLastToken().IsEmpty() == FALSE;
	*/
}

TString TDirectory::GetVInfo(short drvNum) {
	HParamBlockRec pb;
	Str63 name = "\p";
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioNamePtr = name;
	pb.volumeParam.ioVolIndex = drvNum;
	m_error = PBHGetVInfoSync(&pb);
	if (m_error == noErr) {
		m_volume = pb.ioParam.ioVRefNum;
		return name;
	}
	return "";
}

Boolean TDirectory::HasSubDirectory(Boolean ignoreInvisible) {
	short n = GetCatFileNum();
	for (int i = 0; i < n; i++) {
		long attrib;
		long fFlags;
		long dirID;
		TString name = GetCatInfoByDirIndex(i, &attrib, &fFlags, &dirID);
		if (name.IsEmpty() == FALSE && (attrib & fsSBDrNmFls) != 0) {
			if ((fFlags & fInvisible) == 0 && (fFlags & 0x1000) == 0)
				return TRUE;	/* 0x1000̓VXẽtH_̂悤ł */
			if (ignoreInvisible == FALSE)
				return TRUE;
		}
	}
	return FALSE;
}

OSErr TDirectory::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		TFSSpec spec(*this);
		result->PutDataWithKeyword(keyAEResult, typeFSS, (Ptr)(const FSSpec *)spec, sizeof (FSSpec));
		return noErr;
	}
	else if (eventID == kAESetData) {
		TFSSpec spec = ae.GetParamFSSpec();
		if (ae.IsGood() == FALSE)
			return ae.m_error;
		*this = TDirectory(spec, TRUE);
		return noErr;
	}
	return errAEEventNotHandled;
}

Boolean TDirectory::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	TFSSpec spec = TFSSpec(*this);
	return spec.CompareObjects(oper, obj);
}

/*
	filePath
*/

void TFilePath::Init() {
	m_error = noErr;
}

TFilePath::TFilePath() {
	Init();
}

TFilePath::TFilePath(const TFilePath& filePath) {
	Init();
	*this = filePath;
}

TFilePath::TFilePath(const TCFString fileName, const TDirectory& dir) {
	Init();
	m_dir = dir;
	m_fileName = fileName;
}

TFilePath::TFilePath(const TFSSpec& spec) {
	Init();
	*this = spec;
}

void TFilePath::operator=(const TFilePath& filePath) {
	m_dir = filePath.m_dir;
	m_fileName = filePath.m_fileName;
}

void TFilePath::operator=(const TFSSpec& spec) {
	m_dir = TDirectory(spec, false);
	TFSRef fsRef(spec);
	m_fileName = fsRef.GetName();
	if (m_fileName.IsEmpty())
		m_fileName = spec.GetName();
	//	m_dir = TDirectory(spec, FALSE);
	//	m_fileName = spec.GetName();
}

Boolean TFilePath::operator==(const TFilePath& filePath) {
	return m_dir == filePath.m_dir && m_fileName.EqualsIgnoreCase(filePath.m_fileName);
	//	return m_dir == filePath.m_dir && m_fileName.stricmp(filePath.m_fileName) == 0;
}

/*
OSErr TFilePath::RenameFileLongName(const CUniString &newLongName)
{
	// 2004.09.04 qgɗ^ƒZOςɂȂ̂kTextEncodingUnknownɂĂB
	if (theSystem->hasHFSPlus && theSystem->hasTextEncodingConverter)
		return ::FSRenameUnicode(TFSSpec(*this), newLongName.GetLength(), newLongName, kTextEncodingUnknown, NULL);
	else
		return -1;
}
*/

TFinderInfo TFilePath::GetFinderInfo() {
	TFinderInfo info;
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	m_error = noErr;
	Str63 name;
	GetRealShortName().CopyTo(name);
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = name;
	pb.fileParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	m_error = PBHGetFInfoSync(&pb);
	info.m_info = pb.fileParam.ioFlFndrInfo;
	info.m_ioFlCrDat = pb.fileParam.ioFlCrDat;
	info.m_ioFlMdDat = pb.fileParam.ioFlMdDat;
	info.m_ioFlLgLen = pb.fileParam.ioFlLgLen;
	info.m_ioFlRLgLen = pb.fileParam.ioFlRLgLen;
	info.m_ioFlAttrib = pb.fileParam.ioFlAttrib;
	return info;
}

OSErr TFilePath::SetFinderInfo(TFinderInfo info) {
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	m_error = noErr;
	Str63 name;
	GetRealShortName().CopyTo(name);
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = name;
	pb.fileParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	pb.fileParam.ioFlFndrInfo = info.m_info;
	pb.fileParam.ioFlCrDat = info.m_ioFlCrDat;
	pb.fileParam.ioFlMdDat = info.m_ioFlMdDat;
	m_error = PBHSetFInfoSync(&pb);
	return m_error;
}

Boolean TFilePath::GetPermissions(UInt32 *permissions) {
	FSCatalogInfo cInfo;
	m_error = noErr;
	memset(&cInfo, 0, sizeof cInfo);
	TFSRef fsRef(TFSSpec(*this));
	FSCatalogInfoBitmap which = kFSCatInfoPermissions;
	m_error = FSGetCatalogInfo(fsRef, which, &cInfo, NULL, NULL, NULL);
	if (m_error != noErr)
		return FALSE;
	if (permissions != NULL)
		memcpy(permissions, cInfo.permissions, sizeof (UInt32) * 4);
	return TRUE;
}

Boolean TFilePath::SetPermissions(UInt32 *permissions) {
	if (permissions == NULL)
		return FALSE;
	FSCatalogInfo cInfo;
	m_error = noErr;
	memset(&cInfo, 0, sizeof cInfo);
	memcpy(cInfo.permissions, permissions, sizeof (UInt32) * 4);
	TFSRef fsRef(TFSSpec(*this));
	FSCatalogInfoBitmap which = kFSCatInfoPermissions;
	m_error = FSSetCatalogInfo(fsRef, which, &cInfo);
	if (m_error != noErr)
		return FALSE;
	return TRUE;
}

/*
CUniString TFilePath::GetLongName()
{
	TFSSpec spec(*this);
	return spec.GetLongName();
}
*/

Boolean TFilePath::IsLocked() {
	TString shortName = GetRealShortName();
	if (shortName.GetLength() == 0) {
		m_error = -1;
		return false;
	}
	HParamBlockRec pb;
	m_error = noErr;
	memset(&pb, 0, sizeof(pb));
	pb.fileParam.ioNamePtr = (StringPtr)(ConstStr255Param)shortName;
	pb.fileParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	m_error = PBHGetFInfoSync(&pb);
	if (m_error != noErr)
		return FALSE;
	return (BitAnd(pb.fileParam.ioFlAttrib, 1) != 0);
}

TCFString TFilePath::GetFullPath() {
	//
	// CFURLCopyFileSystemPathMac OS 9łCarbonLibł͒ZOɂȂB
	// 2005.12..01
	//
	//	TCFURL url(TFSSpec(*this));
	//	return url.GetFileSystemPath();
	return m_dir.GetFullPath() + ":" + m_fileName;
}

TString TFilePath::GetRealShortName() const {
	if (IsReal() == FALSE)
		return "";
	TFSRef fsRef(*this);
	TFSSpec spec(fsRef);
	return spec.GetName();
}

Boolean TFilePath::IsReal() const {
	if (m_dir.IsReal() == FALSE)
		return FALSE;
	TFSRef parent(m_dir);
	TFSRef ref;
	THFSUniStr255 us(m_fileName);
	return ref.MakeByUnicode(parent, us.GetLength(), us) == noErr;
}

/*
	TFinderInfo
*/

TFinderInfo::TFinderInfo() {
	memset(&m_info, 0, sizeof m_info);
	m_ioFlCrDat = 0;
	m_ioFlMdDat = 0;
	m_ioFlLgLen = 0;
	m_ioFlRLgLen = 0;
	m_ioFlAttrib = 0;
}

/*
	TFSSpec
*/

void TFSSpec::Init() {
	memset(&m_spec, 0, sizeof m_spec);
	m_error = noErr;
}

TFSSpec::TFSSpec() {
	Init();
}

TFSSpec::TFSSpec(const TFSSpec& spec) {
	Init();
	*this = spec;
}

TFSSpec::TFSSpec(const FSSpec& spec) {
	Init();
	*this = spec;
}

TFSSpec::TFSSpec(const TDirectory& dir) {
	Init();
	*this = dir;
}

TFSSpec::TFSSpec(const TDirectory& dir, const TString &shortName) {
	Init();
	FSMakeFSSpec(dir.m_volume, dir.m_dirID, shortName);
}

TFSSpec::TFSSpec(const TFilePath& filePath) {
	Init();
	*this = filePath;
}

TFSSpec::TFSSpec(short vRefNum, long dirID, ConstStr255Param shortName) {
	Init();
	FSMakeFSSpec(vRefNum, dirID, shortName);
}

TFSSpec::TFSSpec(const TFSRef& fsRef) {
	Init();
	*this = fsRef;
}

TFSSpec::operator const FSSpec *() const {
	return GetFSSpecPtr();
}

const FSSpec *TFSSpec::GetFSSpecPtr() const {
	return &m_spec;
}

void TFSSpec::operator=(const TFSSpec& spec) {
	m_spec = spec.m_spec;
}

void TFSSpec::operator=(const FSSpec& spec) {
	m_spec = spec;
}

void TFSSpec::operator=(const TDirectory& dir) {
	FSMakeFSSpec(dir.m_volume, dir.m_dirID, (TString)"");
}

void TFSSpec::operator=(const TFilePath& filePath) {
	TFSRef fsRef(filePath);
	*this = fsRef.GetFSSpec();
	//TString name = filePath.m_fileName;
	//FSMakeFSSpec(filePath.m_dir.m_volume, filePath.m_dir.m_dirID, name);
}

void TFSSpec::operator=(const TFSRef &fsRef) {
	*this = fsRef.GetFSSpec();
}


Boolean TFSSpec::operator==(TFSSpec& spec) {
	if (GetVRefNum() == spec.GetVRefNum() &&
	        GetParID() == spec.GetParID() &&
	        GetName() == spec.GetName())
		return TRUE;
	else
		return FALSE;
}

Boolean TFSSpec::IsGood() {
	return m_error == noErr;
}

short TFSSpec::GetVRefNum() const {
	return m_spec.vRefNum;
}

long TFSSpec::GetParID() const {
	return m_spec.parID;
}

TString TFSSpec::GetName() const {
	return m_spec.name;
}

/*
CUniString TFSSpec::GetLongName()
{
	OSErr err = -1;
	if (theSystem->hasTextEncodingConverter)
	{
		if (theSystem->hasHFSPlus)
		{
			FSCatalogInfo ci;
			memset(&ci, 0, sizeof(ci));
			HFSUniStr255 us;
			memset(&us, 0, sizeof(us));
			err = ::FSGetCatalogInfo(*this, kFSCatInfoGettableInfo, &ci, &us, NULL, NULL);
			if (err == noErr)
				return (CUniString)us;
		}
		return GetName();
	}
	return "";
}
*/

OSErr TFSSpec::FSMakeFSSpec(short vRefNum, long dirID, ConstStr255Param fileName) {
	return m_error = ::FSMakeFSSpec(vRefNum, dirID, fileName, &m_spec);
}

OSErr TFSSpec::FSpGetFInfo(FInfo *fndrInfo) {
	return m_error = ::FSpGetFInfo(&m_spec, fndrInfo);
}

TDirectory TFSSpec::GetDirectory() const {
	return TDirectory(m_spec.vRefNum, m_spec.parID);
}

Boolean TFSSpec::FindApplicationOneVolume(short volume, OSType creator) {
	DTPBRec dtpb;
	memset(&dtpb, 0, sizeof dtpb);
	dtpb.ioVRefNum = volume;
	m_error = PBDTGetPath(&dtpb);
	if (m_error != noErr)
		return FALSE;
	if (dtpb.ioDTRefNum == 0) {
		m_error = -1;
		return FALSE;
	}
	short dtRefNum = dtpb.ioDTRefNum;
	Str63 name;
	memset(&dtpb, 0, sizeof dtpb);
	dtpb.ioNamePtr = name;
	dtpb.ioDTRefNum = dtRefNum;
	dtpb.ioIndex = 0;
	dtpb.ioFileCreator = creator;
	m_error = PBDTGetAPPLSync(&dtpb);
	if (m_error != noErr)
		return FALSE;
	m_error = FSMakeFSSpec(volume, dtpb.ioAPPLParID, name);
	return m_error == noErr;
}

Boolean TFSSpec::FindApplication(OSType creator) {
	if (FindApplicationOneVolume(theSystem->applDirectory.m_volume, creator))
		return TRUE;
	if (FindApplicationOneVolume(theSystem->sysDirectory.m_volume, creator))
		return TRUE;
	for (int i = 1; i < 64; i++) {
		TDirectory dir;
		dir.GetVInfo(i);
		if (dir.IsGood() == TRUE)
			if (FindApplicationOneVolume(dir.m_volume, creator))
				return TRUE;
	}
	return FALSE;
}

OSErr TFSSpec::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		result->PutDataWithKeyword(keyAEResult, typeFSS, (Ptr)&m_spec, sizeof (FSSpec));
		return noErr;
	}
	else if (eventID == kAESetData) {
		TFSSpec spec = ae.GetParamFSSpec();
		if (ae.IsGood() == FALSE)
			return ae.m_error;
		*this = spec;
		return noErr;
	}
	return errAEEventNotHandled;
}

Boolean TFSSpec::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	TFSSpec n1 = *this;
	TFSSpec n2;
	if (obj->GetObjectType() == classFSSpec)
		n2 = *(TFSSpec *)obj;
	else if (obj->GetObjectType() == classDirectory)
		n2 = TFSSpec(*(TDirectory *)obj);
	else
		return FALSE;
	if (oper == kAEEquals)
		return n1 == n2;
	else
		return FALSE;
}

TCatInfo::TCatInfo() {
	Init();
}

void TCatInfo::Init() {
	memset(&m_cb, 0, sizeof (m_cb));
	memset(&m_ci, 0, sizeof (m_ci));
}

Boolean TCatInfo::GetCatInfoByShortName(TString shortName, TDirectory dir) {
	Init();
	m_error = noErr;
	if (theSystem->hasHFSPlus) {
		TFSSpec spec(TFilePath(shortName, dir));
		TFSRef fsRef(spec);
		m_error = ::FSGetCatalogInfo(fsRef, kFSCatInfoGettableInfo, &m_ci, &m_longName, NULL, NULL);
		if (m_error != noErr)
			return FALSE;
	}
	else {
		shortName.CopyTo(m_fName);
		m_cb.dirInfo.ioCompletion = NULL;
		m_cb.dirInfo.ioNamePtr = m_fName;
		m_cb.dirInfo.ioVRefNum = dir.m_volume;
		m_cb.dirInfo.ioDrDirID = dir.m_dirID;
		m_cb.dirInfo.ioFDirIndex = 0;
		m_error = ::PBGetCatInfoSync(&m_cb);
		if (m_error != noErr)
			return FALSE;
	}
	return TRUE;
}

/*
	File
*/

void TFileIO::Init() {
	m_fRefNum = 0;
	m_error = noErr;
	m_noFlush = CFILEIO_FLUSH_IF_WRITE;
	m_needFlush = FALSE;
	m_openMode = CFILEIO_OPEN_MODE_NORMAL;
	m_buffer = NULL;
	m_bufferSize = 0;
	m_filePos = 0;
	m_pos = 0;
	m_bfTop = 0;
	m_readEnd = FALSE;
	m_readSize = 0;
	m_noCache = FALSE;
	m_fileSize = 0;
}

TFileIO::TFileIO(Boolean noFlush) {
	Init();
	m_noFlush = noFlush;
}

TFileIO::TFileIO(TFilePath &filePath, OSType creator, OSType fileType, Boolean noFlush) {
	Init();
	OpenOrCreate(filePath, creator, fileType, noFlush);
}

TFileIO::~TFileIO() {
	Close();
}

OSErr TFileIO::OpenOrCreate(TFilePath &filePath, OSType creator, OSType fileType, Boolean noFlush) {
	m_noFlush = noFlush;
	m_error = HOpen(filePath, 0, false, false);
	if (m_error != noErr && creator != '    ') {
		m_error = FSCreate(filePath, creator, fileType);
		if (m_error == noErr) {
			m_error = HOpen(filePath, 0, false, false);
		}
	}
	return m_error;
}

OSErr TFileIO::Create(TFilePath &filePath, OSType creator, OSType fileType, Boolean noFlush) {
	return OpenOrCreate(filePath, creator, fileType, noFlush);
}

void TFileIO::Close() {
	//m_noFlush;
	if (m_fRefNum != 0)
		FSClose();
	if (m_buffer != NULL)
		delete [] m_buffer;
	m_fRefNum = 0;
	m_buffer = NULL;
}

OSErr TFileIO::Open(TFilePath &filePath, SInt8 permssn, Boolean rf, Boolean noCache) {
	//return FSOpen(filePath, permssn, rf, noCache);
	return HOpen(filePath, permssn, rf, noCache);
}

OSErr TFileIO::FSCreate(TFilePath &filePath, OSType creator, OSType fileType) {
	THFSUniStr255 name(filePath.m_fileName);
	TFSRef parent(filePath.m_dir);
	m_error = FSCreateFileUnicode(parent, name.GetLength(), name, kFSCatInfoNone, NULL, NULL, NULL);
	if (m_error == noErr) {
		TFinderInfo info = filePath.GetFinderInfo();
		info.m_info.fdCreator = creator;
		info.m_info.fdType = fileType;
		filePath.SetFinderInfo(info);
	}
	return m_error;
}

/*
// 肵Ȃ̂FSOpenForkĝ͒~
// 2005.11.23
// FSOpenFork64bitΉAPImɂgȂ悤BIɑŜňڍsKvB
// 2005.11.24
OSErr TFileIO::FSOpen(TFilePath &filePath, SInt8 permssn, Boolean rf, Boolean noCache) {
	m_noCache = noCache;
	m_dir = filePath.m_dir;
	TFSRef fsRef(filePath);
	HFSUniStr255 forkName;
	if (rf)
		FSGetResourceForkName(&forkName);
	else
		FSGetDataForkName(&forkName);
	m_error = FSOpenFork(fsRef, forkName.length, forkName.unicode, permssn, &m_fRefNum);
	return m_error;
}
*/

OSErr TFileIO::HOpen(TFilePath &filePath, SInt8 permssn, Boolean rf, Boolean noCache) {
	if (filePath.IsReal() == FALSE) {
		m_error = -1;
		return -1;
	}
	// 傤Ȃ̂PBHOpenSync𒷂OΉB
	// 2005.11.23
	TString fileName = filePath.GetRealShortName();
	if (fileName.GetLength() == 0) {
		m_error = -1;
		return -1;
	}

	m_noCache = noCache;
	m_dir = filePath.m_dir;
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioNamePtr = (StringPtr)(ConstStr255Param)fileName;
	pb.ioParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	pb.ioParam.ioPermssn = permssn;
	/* data fork */
	if (rf == FALSE)
		m_error = PBHOpenSync(&pb);
	/* resource fork */
	else
		m_error = PBHOpenRFSync(&pb);
	if (m_error == noErr)
		m_fRefNum = pb.ioParam.ioRefNum;
	return m_error;
}

OSErr TFileIO::HCreate(TFilePath &filePath, OSType creator, OSType fileType) {
	m_dir = filePath.m_dir;
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	Str255 name;
	filePath.m_fileName.CopyToPSTR(name);
	pb.ioParam.ioNamePtr = name;
	pb.ioParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	m_error = PBHCreateSync(&pb);
	if (m_error != noErr)
		return m_error;
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioNamePtr = name;
	pb.ioParam.ioVRefNum = m_dir.m_volume;
	pb.fileParam.ioDirID = m_dir.m_dirID;
	pb.fileParam.ioFlFndrInfo.fdType = fileType;
	pb.fileParam.ioFlFndrInfo.fdCreator = creator;
	TDateTime dateTime;
	pb.fileParam.ioFlCrDat = dateTime;
	pb.fileParam.ioFlMdDat = dateTime;
	m_error = PBHSetFInfoSync(&pb);
	return m_error;
}

OSErr TFileIO::FSClose() {
	if (m_openMode == CFILEIO_OPEN_MODE_WRITE && m_pos > 0)
		Write(m_pos, m_buffer);
	m_error = ::FSClose(m_fRefNum);
	m_fRefNum = 0;
	if (m_needFlush)
		m_dir.FlushVol();
	return m_error;
}

OSErr TFileIO::FSRead(long *count, void *buffPtr) {
	//	return m_error = ::FSRead(m_fRefNum, count, buffPtr);
	ParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioRefNum = m_fRefNum;
	pb.ioParam.ioBuffer = (char *)buffPtr;
	pb.ioParam.ioReqCount = *count;
	pb.ioParam.ioPosMode = fsAtMark;
	if (m_noCache)
		pb.ioParam.ioPosMode += 0x0020;
	m_error = PBReadSync(&pb);
	*count = pb.ioParam.ioActCount;
	return m_error;
}

long TFileIO::Read(long count, void *buffPtr) {
	m_error = FSRead(&count, buffPtr);
	return count;
}

OSErr TFileIO::FSWrite(long *count, void *buffPtr) {
	if (m_noFlush == CFILEIO_FLUSH_IF_WRITE)
		m_needFlush = TRUE;
	ParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	pb.ioParam.ioRefNum = m_fRefNum;
	pb.ioParam.ioBuffer = (char *)buffPtr;
	pb.ioParam.ioReqCount = *count;
	pb.ioParam.ioPosMode = fsAtMark;
	if (m_noCache)
		pb.ioParam.ioPosMode += 0x0020;
	m_error = PBWriteSync(&pb);
	*count = pb.ioParam.ioActCount;
	return m_error;
}

long TFileIO::Write(long count, void *buffPtr) {
	m_error = FSWrite(&count, buffPtr);
	return count;
}

OSErr TFileIO::GetEOF(long *logEOF) {
	m_error = ::GetEOF(m_fRefNum, logEOF);
	m_fileSize = *logEOF;
	return m_error;
}

long TFileIO::GetEOF() {
	long logEOF;
	GetEOF(&logEOF);
	return logEOF;
}

OSErr TFileIO::SetEOF(long logEOF) {
	return m_error = ::SetEOF(m_fRefNum, logEOF);
	/*	ParamBlockRec pb;
		memset(&pb, 0, sizeof pb);
		pb.ioParam.ioRefNum = m_fRefNum;
		pb.ioParam.ioMisc = (char *)logEOF;
		m_error = PBSetEOFSync(&pb);
		return m_error;
	*/
}

TFileReader::TFileReader(Boolean noFlush) {
	Init();
	m_noFlush = noFlush;
}

TFileReader::TFileReader(TFilePath &filePath, Boolean rf, long bufferSize) {
	Init();
	OpenForGet(filePath, rf, bufferSize);
}

OSErr TFileReader::OpenForGet(TFilePath &filePath, Boolean rf, long bufferSize) {
	m_error = Open(filePath, fsRdPerm, rf);
	if (m_error != noErr)
		return m_error;
	GetEOF();
	m_bufferSize = bufferSize;
	m_buffer = new Byte [bufferSize];
	m_openMode = CFILEIO_OPEN_MODE_READ;
	m_filePos = 0;
	m_pos = 0;
	m_readEnd = FALSE;
	m_readSize = 0;
	return m_error;
}

int TFileReader::GetC() {
	m_error = noErr;
	if (m_fRefNum == 0)
		return EOF;
	if (m_pos >= m_readSize) {
		if (m_readEnd)
			return EOF;
		m_pos = 0;
		m_readSize = Read(m_bufferSize, m_buffer);
		if (m_readSize < m_bufferSize)
			m_readEnd = TRUE;
		if (m_readSize == 0)
			return EOF;
	}
	int data = m_buffer[m_pos];
	m_pos += 1;
	m_filePos += 1;
	return data;
}

void TFileReader::GetP(void *p, long length) {
	for (long i = 0; i < length; i++)
		((Byte *)p)[i] = GetC();
}

long TFileReader::Seek(short posMode, long posOff) {
	SetFPos(posMode, posOff);
	GetFPos(&m_filePos);
	m_pos = 0;
	m_readEnd = FALSE;
	m_readSize = 0;
	return m_filePos;
}

long TFileReader::Tell() {
	return m_filePos;
}

TFileWriter::TFileWriter(Boolean noFlush) {
	Init();
	m_noFlush = noFlush;
}

TFileWriter::TFileWriter(TFilePath &filePath, OSType creator, OSType fileType, Boolean rf, long bufferSize, Boolean noFlush) {
	Init();
	m_noFlush = noFlush;
	OpenForPut(filePath, creator, fileType, rf, bufferSize);
}

OSErr TFileWriter::OpenForPut(TFilePath &filePath, OSType creator, OSType fileType, Boolean rf, long bufferSize) {
	m_error = Open(filePath, 0, rf);
	if (m_error != noErr) {
		m_error = FSCreate(filePath, creator, fileType);
		if (m_error != noErr)
			return m_error;
		m_error = Open(filePath, 0, rf);
		if (m_error != noErr)
			return m_error;
	}
	BeEmpty();
	m_bufferSize = bufferSize;
	m_buffer = new Byte [bufferSize];
	m_openMode = CFILEIO_OPEN_MODE_WRITE;
	m_filePos = 0;
	m_pos = 0;
	m_bfTop = 0;
	return m_error;
}

void TFileWriter::PutC(int data) {
	m_error = noErr;
	if (m_fRefNum == 0)
		return;
	if (m_pos == m_bufferSize) {
		int count = m_bufferSize;
		Write(count, m_buffer);
		m_pos = 0;
	}
	m_buffer[m_pos] = (Byte)data;
	m_pos += 1;
	m_filePos += 1;
}

void TFileWriter::PutP(void *p, long length) {
	for (long i = 0; i < length; i++)
		PutC(((Byte *)p)[i]);
}

/*
	TDateTime
*/

unsigned long TDateTime::GetDateTime() {
	unsigned long secs;
	::GetDateTime(&secs);
	return secs;
}

unsigned long TDateTime::DateToSeconds() {
	unsigned long secs;
	::DateToSeconds(this, &secs);
	return secs;
}

TString TDateTime::IUDateString(DateForm longFlag) {
	Str255 result;
	::DateString(DateToSeconds(), longDate, result, NULL);
	return result;
}
TString TDateTime::IUTimeString(Boolean wantSeconds) {
	Str255 result;
	::TimeString(DateToSeconds(), wantSeconds, result, NULL);
	return result;
}

OSErr TDateTime::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		TString s;
		if (desiredClass == classDate)
			s.sprintf("%d/%d/%d", year, month, day);
		else if (desiredClass == classTime)
			s.sprintf("%02d:%02d:%02d", hour, minute, second);
		else
			s.sprintf("%d/%d/%d %02d:%02d:%02d", year, month, day, hour, minute, second);
		char *p = s;
		result->PutDataWithKeyword(keyAEResult, typeChar, (Ptr)p, s.GetLength());
		return noErr;
	}
	else if (eventID == kAESetData) {
		TString s = ae.GetParamString();
		char *delim = "/-:,.";
		TString t[3];
		t[0] = s.strtok_first(delim);
		t[1] = s.strtok_next(delim);
		t[2] = s.strtok_next(delim);
		if (desiredClass == classDate) {
			year = t[0].atoi();
			month = t[1].atoi();
			day = t[2].atoi();
			dayOfWeek = 0;
		}
		else if (desiredClass == classTime) {
			hour = t[0].atoi();
			minute = t[1].atoi();
			second = t[2].atoi();
		}
		return noErr;
	}
	return errAEEventNotHandled;
}

/*
	TResourceFile
*/

void TResourceFile::Init() {
	m_fRefNum = 0;
}

TResourceFile::TResourceFile(TFilePath filePath, Boolean create) {
	Init();
	m_filePath = filePath;
	OpenResFile();
	if (m_fRefNum == -1 && create) {
		CreateResFile();
		OpenResFile();
	}
}

TResourceFile::~TResourceFile() {
	if (m_fRefNum > 0)
		CloseResFile();
}

void TResourceFile::OpenResFile() {
	TString name = m_filePath.GetRealShortName();
	m_fRefNum = ::HOpenResFile(m_filePath.m_dir.m_volume, m_filePath.m_dir.m_dirID, name, fsRdWrPerm);
}

void TResourceFile::CreateResFile() {
	TString name = m_filePath.GetRealShortName();
	::HCreateResFile(m_filePath.m_dir.m_volume, m_filePath.m_dir.m_dirID, name);
}

void TResourceFile::CloseResFile() {
	if (m_fRefNum > 0)
		::CloseResFile(m_fRefNum);
	m_fRefNum = 0;
}

/*
	TResource
*/

TResource::TResource() {
	m_h = NULL;
	m_detached = FALSE;
	m_size = 0;
}

TResource::TResource(OSType sig, short id) {
	Get1Resource(sig, id);
	m_detached = FALSE;
}

TResource::TResource(long sz) {
	NewHandle(sz);
}

TResource::~TResource() {
	if (m_detached)
		DisposeHandle();
	m_detached = FALSE;
}

void TResource::Get1Resource(OSType sig, short id) {
	m_h = ::Get1Resource(sig, id);
	m_size = GetResourceSizeOnDisk();
}

void TResource::DetachResource() {
	if (m_detached == FALSE) {
		::DetachResource(m_h);
		m_detached = TRUE;
	}
}

void TResource::AddResource(OSType sig, short id, ConstStr255Param name) {
	::AddResource(m_h, sig, id, name);
	m_detached = FALSE;
}

void *TResource::HLock() {
	if (m_h == NULL)
		return NULL;
	::HLock(m_h);
	return (void *)*m_h;
}

void TResource::HUnlock() {
	if (m_h == NULL)
		return;
	::HUnlock(m_h);
}

void TResource::NewHandle(long sz) {
	m_h = ::NewHandle(sz);
	m_detached = TRUE;
	m_size = sz;
}

void TResource::DisposeHandle() {
	if (m_h != NULL) {
		::DisposeHandle(m_h);
		m_h = NULL;
		m_size = 0;
	}
}

void TResource::SetHandleSize(long sz) {
	::SetHandleSize(m_h, sz);
	m_size = sz;
}

/*
	TLockHandle
*/

TLockHandle::TLockHandle(Handle h) {
	m_h = h;
	if (m_h != NULL)
		::HLock(m_h);
}

TLockHandle::~TLockHandle() {
	if (m_h != NULL)
		::HUnlock(m_h);
}

void *TLockHandle::GetPtr() {
	return m_h == NULL ? NULL : (void *)*m_h;
}

/*
	TStandardFile
*/

TStandardFileReply::TStandardFileReply() {
	m_fType = '    ';
	m_fCreator = '    ';
	m_isFolder = FALSE;
	m_isVolume = FALSE;
}

void TStandardFileReplyList::AppendFile(TFSSpec &spec) {
	FInfo info;
	spec.FSpGetFInfo(&info);
	TStandardFileReply *reply = new TStandardFileReply;
	reply->m_isFolder = spec.IsGood() == FALSE;
	TFSRef fsRef(spec);
	reply->m_fName = fsRef.GetName();
	//	reply->m_fLongName = spec.GetLongName();
	reply->m_dir = TDirectory(spec.GetVRefNum(), spec.GetParID());
	reply->m_isVolume = (reply->m_isFolder && reply->m_fName.IsEmpty() && reply->m_dir.IsRoot());
	if (reply->m_isFolder && reply->m_fName.IsEmpty()) {
		reply->m_fName = reply->m_dir.GetDirName();
		reply->m_dir = reply->m_dir.GetParent();
	}
	if (reply->m_isFolder == FALSE) {
		reply->m_fType = info.fdType;
		reply->m_fCreator = info.fdCreator;
	}
	Append(reply);
}

void TStandardFileReplyList::AppendFile(OSType fType, TDirectory dir, TCFString fName, Boolean isFolder, Boolean isVolume) {
	TStandardFileReply *reply = new TStandardFileReply;
	reply->m_fType = fType;
	reply->m_dir = dir;
	reply->m_fName = fName;
	reply->m_isFolder = isFolder;
	reply->m_isVolume = isVolume;
	Append(reply);
}


/*
	TStandardFile
*/

#define	STANDARD_FILEOPEN_DLG			1100
#define	SFD_FO_TITLE_DITEM				10

TStandardFile *currentTStandardFile;
pascal Boolean TStandardFile_Filter6(CInfoPBPtr pb);
pascal Boolean TStandardFile_Filter7(CInfoPBPtr pb, void *yourDataPtr);
pascal Boolean TStandardFile_FilterNav(AEDesc *theItem, void *info, void *callBackUD, NavFilterModes filterMode);
pascal void TStandardFile_EventNav(NavEventCallbackMessage callBackSelector, NavCBRecPtr callBackParms, void *callBackUD);
pascal short TStandardFile_DlgHook6(short item, DialogPtr dlg);
pascal short TStandardFile_DlgHook7(short item, DialogPtr dlg, void *yourDataPtr);
pascal Boolean TStandardFile_ModalFilter6(DialogPtr dlg, EventRecord *ev, short *itemHit);
pascal Boolean TStandardFile_ModalFilter7(DialogPtr dlg, EventRecord *ev, short *itemHit, void *yourDataPtr);

/* Filter */
pascal Boolean TStandardFile_Filter6(CInfoPBPtr pb) {
	TStandardFileFilterItemInfo itemInfo;
	itemInfo.m_isFolder = (pb->hFileInfo.ioFlAttrib & fsSBDrNmFls) != 0;
	if (itemInfo.m_isFolder) {
		itemInfo.m_dir = TDirectory(pb->dirInfo.ioVRefNum, pb->dirInfo.ioDrParID);
		itemInfo.m_name = pb->dirInfo.ioNamePtr;
		itemInfo.m_visible = (pb->dirInfo.ioDrUsrWds.frFlags & kIsInvisible) == 0;
	}
	else {
		itemInfo.m_dir = TDirectory(pb->hFileInfo.ioVRefNum, pb->hFileInfo.ioFlParID);
		itemInfo.m_name = pb->hFileInfo.ioNamePtr;
		itemInfo.m_visible = (pb->hFileInfo.ioFlFndrInfo.fdFlags & kIsInvisible) == 0;
		itemInfo.m_fdCreator = pb->hFileInfo.ioFlFndrInfo.fdCreator;
		itemInfo.m_fdType = pb->hFileInfo.ioFlFndrInfo.fdType;
	}
	return currentTStandardFile->Filter(itemInfo) == FALSE;
}

pascal Boolean TStandardFile_Filter7(CInfoPBPtr pb, void *yourDataPtr) {
	return TStandardFile_Filter6(pb);
}

pascal Boolean TStandardFile_FilterNav(AEDesc *item, void *info, void *callBackUD, NavFilterModes filterMode) {
	TAEDesc desc(item);
	if (desc.descriptorType != typeFSRef)
		return TRUE;
	TFSRef fsRef = desc.GetFSRef();
	TFSSpec spec(fsRef);
	NavFileOrFolderInfo *fileInfo = (NavFileOrFolderInfo *)info;
	TStandardFileFilterItemInfo itemInfo;
	itemInfo.m_isFolder = fileInfo->isFolder;
	itemInfo.m_visible = fileInfo->visible;
	itemInfo.m_name = fsRef.GetName();
	itemInfo.m_dir = spec.GetDirectory();
	if (itemInfo.m_isFolder == FALSE) {
		itemInfo.m_fdCreator = fileInfo->fileAndFolder.fileInfo.finderInfo.fdCreator;
		itemInfo.m_fdType = fileInfo->fileAndFolder.fileInfo.finderInfo.fdType;
	}
	return currentTStandardFile->Filter(itemInfo);
}

pascal void TStandardFile_EventNav(NavEventCallbackMessage callBackSelector, NavCBRecPtr callBackParms, void *callBackUD) {
	TStandardFile *std = (TStandardFile *)callBackUD;
	if (callBackSelector == kNavCBEvent) {
		short itemHit = 0;
		std->ModalFilter(GetDialogFromWindow(callBackParms->window), callBackParms->eventData.eventDataParms.event, &itemHit);
	}
}

/* DlgHook */
pascal short TStandardFile_DlgHook6(short item, DialogPtr dlg) {
	return currentTStandardFile->DlgHook(item, dlg);
}

pascal short TStandardFile_DlgHook7(short item, DialogPtr dlg, void *yourDataPtr) {
	return currentTStandardFile->DlgHook(item, dlg);
}

/* ModalFilter */
pascal Boolean TStandardFile_ModalFilter6(DialogPtr dlg, EventRecord *ev, short *itemHit) {
	return currentTStandardFile->ModalFilter(dlg, ev, itemHit);
}

pascal Boolean TStandardFile_ModalFilter7(DialogPtr dlg, EventRecord *ev, short *itemHit, void *yourDataPtr) {
	return currentTStandardFile->ModalFilter(dlg, ev, itemHit);
}

void TStandardFile::Init() {
	m_selectFolder = FALSE;
	m_firstDir = TDirectory(0, 0);
	m_firstCall = theSystem->systemVersion >= 0x0700;
	m_dlgID = 0;
	m_multi = FALSE;
	if (theSystem->hasNavigation && theSystem->m_useNavigation) {
		m_uppFileFilterNav.SetUPP(TStandardFile_FilterNav);
		m_uppEventNav.SetUPP(TStandardFile_EventNav);
	}
}

TStandardFile::TStandardFile() {
	Init();
}

TStandardFile::TStandardFile(TDirectory dir) {
	Init();
	m_firstDir = dir;
}

TStandardFile::~TStandardFile() {
	RemoveDeleteAll();
}

Boolean TStandardFile::Filter(TStandardFileFilterItemInfo& info) {
	return TRUE;
}

void TStandardFile::Update(int message) {
	TWindow *win = theWindowArray->GetByWindowRef((WindowRef)message);
	if (win != NULL)
		win->OnUpdate();
}

short TStandardFile::DlgHook(short item, DialogPtr dp) {
	return item;
}

Boolean TStandardFile::ModalFilter(DialogPtr dlg, EventRecord *ev, short *itemHit) {
	if (ev->what == updateEvt)
		Update(ev->message);
	return FALSE;
}

void TStandardFile::GetFile(TString prompt, Boolean multi) {
	if (theSystem->systemVersion >= 0x1000)
		GetFileNav2(prompt, multi);
	else
		GetFileNav1(prompt, multi);
}

void TStandardFile::GetFileNav2(TString prompt, Boolean multi) {
	if (prompt.IsEmpty() == FALSE)
		m_title = prompt;
	m_multi = multi;
	currentTStandardFile = this;
	memset(&m_replyNav, 0, sizeof m_replyNav);

	//NavObjectFilterUPP uppNavObjectFilterProc = NewNavObjectFilterUPP(xNavObjectFilterProcPtr);
	NavDialogCreationOptions opt;
	memset(&opt, 0, sizeof(opt));
	opt.version = kNavDialogCreationOptionsVersion;
	opt.optionFlags = kNavDefaultNavDlogOptions;
	opt.location.v = -1;
	opt.location.h = -1;
	opt.modality = kWindowModalityAppModal;
	opt.windowTitle = m_title;
	if (multi)
		opt.optionFlags |= kNavAllowMultipleFiles;
	else
		opt.optionFlags &= (~kNavAllowMultipleFiles);
	opt.optionFlags |= kNavDontAutoTranslate;
	opt.optionFlags |= kNavDontAddTranslateItems;
	opt.optionFlags |= kNavAllowInvisibleFiles;
	opt.optionFlags &= (~kNavAllowPreviews);
	NavDialogRef nav;
	if (m_selectFolder)
		m_good = NavCreateChooseObjectDialog(&opt, NULL, NULL, m_uppFileFilterNav, NULL, &nav) == noErr;
	else
		m_good = NavCreateGetFileDialog(&opt, NULL, NULL, NULL, m_uppFileFilterNav, NULL, &nav) == noErr;
	TFSRef r(m_firstDir);
	TAEDesc aed;
	aed.AECreateDesc(typeFSRef, r.GetFSRefPtr(), sizeof(FSRef));
	NavCustomControl(nav, kNavCtlSetLocation, aed);
	m_good = NavDialogRun(nav) == noErr;
	if (m_good) {
		m_replyNav.version = kNavReplyRecordVersion;
		NavDialogGetReply(nav, &m_replyNav);
		m_good = m_replyNav.validRecord;
		if (m_good) {
			TAEDescList select(&m_replyNav.selection);
			int count = select.GetCount();
			for (int i = 0; i < count; i++) {
				FSSpec spec;
				DescType actualType;
				Size actualSize;
				AEKeyword retKeyword;
				m_good = select.AEGetNthPtr(i + 1, typeFSS, &retKeyword, &actualType, &spec, sizeof spec, &actualSize) == noErr;
				if (m_good) {
					TFSSpec fsSpec(spec);
					AppendFile(fsSpec);
				}
			}
		}
		NavDisposeReply(&m_replyNav);
	}
	NavDialogDispose(nav);
	theCursor->Reset();
}

void TStandardFile::GetFileNav1(TString prompt, Boolean multi) {
	if (prompt.IsEmpty() == FALSE)
		m_title = prompt;
	m_multi = multi;
	currentTStandardFile = this;
	memset(&m_replyNav, 0, sizeof m_replyNav);

	if (theSystem->hasNavigation && theSystem->m_useNavigation) {
		NavDialogOptions dialogOptions;
		NavGetDefaultDialogOptions(&dialogOptions);
		FSSpec spec;
		FSMakeFSSpec(m_firstDir.m_volume, m_firstDir.m_dirID, "\pdmy", &spec);
		spec.name[0] = 0;
		AEDesc defaultLocation;
		AECreateDesc(typeFSS, &spec, sizeof(spec), &defaultLocation);

		if (m_multi)
			dialogOptions.dialogOptionFlags |= kNavAllowMultipleFiles;
		else
			dialogOptions.dialogOptionFlags &= (~kNavAllowMultipleFiles);
		dialogOptions.dialogOptionFlags |= kNavDontAutoTranslate;
		dialogOptions.dialogOptionFlags |= kNavDontAddTranslateItems;
		dialogOptions.dialogOptionFlags |= kNavAllowInvisibleFiles;
		dialogOptions.dialogOptionFlags &= (~kNavAllowPreviews);

		if (m_title.IsEmpty() == FALSE)
			m_title.CopyTo(dialogOptions.windowTitle, 255);
		if (m_selectFolder)
			m_good = NavChooseObject(&defaultLocation, &m_replyNav, &dialogOptions, m_uppEventNav.GetUPP(), m_uppFileFilterNav.GetUPP(), this) == noErr;
		else
			m_good = NavGetFile(&defaultLocation, &m_replyNav, &dialogOptions, m_uppEventNav.GetUPP(), NULL, m_uppFileFilterNav.GetUPP(), NULL, this) == noErr;
		if (m_good)
			m_good = m_replyNav.validRecord;
		if (m_good) {
			TAEDescList select(&m_replyNav.selection);
			int count = select.GetCount();
			for (int i = 0; i < count; i++) {
				FSSpec spec;
				DescType actualType;
				Size actualSize;
				AEKeyword retKeyword;
				m_good = select.AEGetNthPtr(i + 1, typeFSS, &retKeyword, &actualType, &spec, sizeof spec, &actualSize) == noErr;
				if (m_good) {
					TFSSpec fsSpec(spec);
					AppendFile(fsSpec);
				}
			}
		}
		NavDisposeReply(&m_replyNav);
	}
	theCursor->Reset();
}

void TStandardFile::PutFile(TString prompt, TString orgName) {
	if (theSystem->systemVersion >= 0x1000)
		PutFileNav2(prompt, orgName);
	else
		PutFileNav1(prompt, orgName);
}

void TStandardFile::PutFileNav2(TString prompt, TString orgName) {
	if (prompt.IsEmpty() == FALSE)
		m_title = prompt;
	currentTStandardFile = this;
	memset(&m_replyNav, 0, sizeof m_replyNav);

	NavDialogCreationOptions opt;
	memset(&opt, 0, sizeof(opt));
	opt.version = kNavDialogCreationOptionsVersion;
	opt.optionFlags = kNavDefaultNavDlogOptions;
	opt.location.v = -1;
	opt.location.h = -1;
	opt.modality = kWindowModalityAppModal;
	opt.saveFileName = orgName;
	opt.windowTitle = m_title;
	NavDialogRef nav;
	OSStatus err = NavCreatePutFileDialog(&opt, m_fType, m_fCreator, NULL, NULL, &nav);

	TFSRef r(m_firstDir);
	TAEDesc aed;
	aed.AECreateDesc(typeFSRef, r.GetFSRefPtr(), sizeof(FSRef));
	NavCustomControl(nav, kNavCtlSetLocation, aed);

	m_good = NavDialogRun(nav) == noErr;
	if (m_good) {
		m_replyNav.version = kNavReplyRecordVersion;
		NavDialogGetReply(nav, &m_replyNav);
		m_good = m_replyNav.validRecord;
		if (m_good) {
			TAEDescList select(&m_replyNav.selection);
			FSRef documentFSRef;
			DescType actualType;
			Size actualSize;
			AEKeyword retKeyword;
			m_good = select.AEGetNthPtr(1, typeFSRef, &retKeyword, &actualType, &documentFSRef, sizeof(FSRef), &actualSize) == noErr;
			if (m_good) {
				TStandardFileReply *reply = new TStandardFileReply;
				reply->m_isFolder = FALSE;
				reply->m_dir = (TFSRef)documentFSRef;
				reply->m_isVolume = FALSE;
				reply->m_fName = NavDialogGetSaveFileName(nav);
				Append(reply);
			}
			NavDisposeReply(&m_replyNav);
		}
	}
	NavDialogDispose(nav);
	theCursor->Reset();
}

void TStandardFile::PutFileNav1(TString prompt, TString orgName) {
	if (prompt.IsEmpty() == FALSE)
		m_title = prompt;
	currentTStandardFile = this;
	memset(&m_replyNav, 0, sizeof m_replyNav);
	if (theSystem->hasNavigation && theSystem->m_useNavigation) {
		NavDialogOptions dialogOptions;
		NavGetDefaultDialogOptions(&dialogOptions);
		FSSpec spec;
		FSMakeFSSpec(m_firstDir.m_volume, m_firstDir.m_dirID, "\pdmy", &spec);
		spec.name[0] = 0;
		AEDesc defaultLocation;
		AECreateDesc(typeFSS, &spec, sizeof(spec), &defaultLocation);
		dialogOptions.dialogOptionFlags &= (~kNavAllowStationery);
		orgName.CopyTo(dialogOptions.savedFileName, 255);
		if (m_title.IsEmpty() == FALSE)
			m_title.CopyTo(dialogOptions.windowTitle, 255);
		m_good = NavPutFile(&defaultLocation, &m_replyNav, &dialogOptions, m_uppEventNav.GetUPP(), m_fType, m_fCreator, this) == noErr;
		if (m_good)
			m_good = m_replyNav.validRecord;
		if (m_good) {
			TAEDescList select(&m_replyNav.selection);
			FSSpec spec;
			DescType actualType;
			Size actualSize;
			AEKeyword retKeyword;
			m_good = select.AEGetNthPtr(1, typeFSS, &retKeyword, &actualType, &spec, sizeof spec, &actualSize) == noErr;
			if (m_good) {
				TFSSpec fsSpec(spec);
				TStandardFileReply *reply = new TStandardFileReply;
				reply->m_isFolder = FALSE;
				reply->m_dir = TDirectory(fsSpec.GetVRefNum(), fsSpec.GetParID());
				reply->m_isVolume = FALSE;
				reply->m_fName = fsSpec.GetName();
				Append(reply);
			}
		}
		NavDisposeReply(&m_replyNav);
	}
	theCursor->Reset();
}

OSType TStandardFile::GetFType(int n) {
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_fType : '    ';
}

TDirectory TStandardFile::GetDirectory(int n) {
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_dir : TDirectory();
}

TCFString TStandardFile::GetFName(int n) {
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_fName : (TCFString)"";
}

/*
CUniString TStandardFile::GetFLongName(int n)
{
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_fLongName : (CUniString)"";
}
*/

TFilePath TStandardFile::GetFilePath(int n) {
	TStandardFileReply *reply = GetNth(n);
	if (reply == NULL)
		return TFilePath();
	return TFilePath(reply->m_fName, reply->m_dir);
}

Boolean TStandardFile::IsFolder(int n) {
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_isFolder : FALSE;
}

Boolean TStandardFile::IsVolume(int n) {
	TStandardFileReply *reply = GetNth(n);
	return reply != NULL ? reply->m_isVolume : FALSE;
}

#define	STANDARD_DIRECTORY_7_DLG		1101
#define	STANDARD_DIRECTORY_6_DLG		1111
#define	SFD7_DIR_SELECT_DITEM			10
#define	SFD7_DIR_DIRECTORY_DITEM		11
#define	SFD7_DIR_TITLE_DITEM			12
#define	SFD6_DIR_SELECT_DITEM			11
#define	SFD6_DIR_DIRECTORY_DITEM		12

void TStandardFileDirectory::Init() {
	if (theSystem->systemVersion < 0x0700)
		m_dlgID = STANDARD_DIRECTORY_6_DLG;
	else
		m_dlgID = STANDARD_DIRECTORY_7_DLG;
	m_selectFolder = TRUE;
}

Boolean TStandardFileDirectory::Filter(TStandardFileFilterItemInfo& info) {
	return info.m_isFolder;
}

short TStandardFileDirectory::DlgHook(short item, DialogPtr dp) {
	return TStandardFile::DlgHook(item, dp);
}

Boolean TStandardFileDirectory::ModalFilter(DialogPtr dlg, EventRecord *ev, short *itemHit) {
	if (ev->what == keyDown) {
		Boolean command = (ev->modifiers & cmdKey) != 0;
		int code = (ev->message & charCodeMask);
		switch (code) {
		case 'S':
		case 's':
			*itemHit = SFD7_DIR_SELECT_DITEM;
			return TRUE;

		}
	}
	return FALSE;
}

CSaveFinderInfo::CSaveFinderInfo(TFilePath& filePath) {
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	m_error = noErr;
	filePath.m_fileName.CopyTo(m_name);
	m_volume = filePath.m_dir.m_volume;
	m_dirID = filePath.m_dir.m_dirID;
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = m_name;
	pb.fileParam.ioVRefNum = m_volume;
	pb.fileParam.ioDirID = m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	m_error = PBHGetFInfoSync(&pb);
	m_info = pb.fileParam.ioFlFndrInfo;
	m_ioFlCrDat = pb.fileParam.ioFlCrDat;
	m_ioFlMdDat = pb.fileParam.ioFlMdDat;
}

CSaveFinderInfo::~CSaveFinderInfo() {
	if (m_error != noErr)
		return;
	HParamBlockRec pb;
	memset(&pb, 0, sizeof pb);
	pb.fileParam.ioCompletion = NULL;
	pb.fileParam.ioNamePtr = m_name;
	pb.fileParam.ioVRefNum = m_volume;
	pb.fileParam.ioDirID = m_dirID;
	pb.fileParam.ioFVersNum = 0;
	pb.fileParam.ioFDirIndex = -1;
	pb.fileParam.ioFlFndrInfo = m_info;
	pb.fileParam.ioFlCrDat = m_ioFlCrDat;
	pb.fileParam.ioFlMdDat = m_ioFlMdDat;
	m_error = PBHSetFInfoSync(&pb);
}

/*
	TDebug
*/

TDebug::TDebug(TFilePath &filePath) {
	m_filePath = filePath;
	m_create = FALSE;
}

void TDebug::Log(TString s) {
	TString data;
	TDateTime secs;
	if (m_create == FALSE) {
		TFileIO file(m_filePath, 'ttxt', 'TEXT');
		file.BeEmpty();
		m_create = TRUE;
		data.sprintf("%s\r", (char *)secs.IUDateString(shortDate));
		file.Write(data);
		//		data.sprintf("MachineType %d\r", theSystem->machineType);
		//		file.Write(data);
		data.sprintf("SystemVersion %04x\r", theSystem->systemVersion);
		file.Write(data);
		//		data.sprintf("HasFPU %d\r", theSystem->hasFPU);
		//		file.Write(data);
		data.sprintf("HasColorQD %d\r", theSystem->hasColorQD);
		file.Write(data);
		//		data.sprintf("KeyboardType %d\r", theSystem->keyBoardType);
		//		file.Write(data);
		//		data.sprintf("AppleTalkDriverVersion %d\r", theSystem->atDrvrVersNum);
		//		file.Write(data);
	}
	data.sprintf("%s-%d %s\r", (char *)secs.IUTimeString(TRUE), TickCount(), (char *)s);
	TFileIO file(m_filePath);
	file.SetFPos(fsFromLEOF, 0);
	file.Write(data);
}

void TDebug::LogAlertExit(TString s) {
	Log(s);
	SysBeep(0);
	TDialog dlg(ERR_EXIT_DLG);
	dlg.SetDIText(ERR_EXIT_MSG_DITEM, s);
	dlg.ModalDialog();
	ExitToShell();
}

extern "C" void DebugLog(char *s) {
	if (theDebug == NULL)
		return;
	theDebug->Log(s);
}
