/*
 *  cftools.cpp
 *  HUNDOSHI-EDIT
 *
 *  Created by ostra on 2005.08.10.
 *  Copyright (C) Takeshi Yoneki / OSTRA / OSTRACISM CO., 2005
 *
 */

#include "csystem.h"
#include "cftools.h"
//#include "ctools.h"

/*
 * TCFURL
 * t@CVXẽpX
 */

void TCFURL::Init() {
	m_ref = NULL;
}

void TCFURL::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

TCFURL::TCFURL() {
	Init();
}

TCFURL::TCFURL(const TCFURL &url) {
	Init();
	*this = url;
}

TCFURL::TCFURL(const CFURLRef relativeURL) {
	Init();
	*this = relativeURL;
}

TCFURL::TCFURL(const CFStringRef filePath, Boolean isDirectory, CFURLPathStyle pathStyle) {
	Init();
	CopyFromString(filePath, isDirectory, pathStyle);
}

TCFURL::TCFURL(const CFBundleRef bundle) {
	Init();
	*this = bundle;
}

TCFURL::TCFURL(const TFSRef &fsRef) {
	Init();
	*this = fsRef;
}

TCFURL::TCFURL(const TFSSpec &spec) {
	Init();
	*this = spec;
}

TCFURL::operator CFURLRef() const {
	return GetCFURLRef();
}

TCFURL::operator CFStringRef() {
	return GetFileSystemPath();
}

void TCFURL::operator=(const TCFURL &url) {
	CopyFromURL(url.m_ref);
}

void TCFURL::operator=(const CFURLRef relativeURL) {
	CopyFromURL(relativeURL);
}

void TCFURL::operator=(const CFStringRef filePath) {
	CopyFromString(filePath);
}

void TCFURL::operator=(const CFBundleRef bundle) {
	CopyFromBundle(bundle);
}

void TCFURL::operator=(const TFSRef &fsRef) {
	CreateFromFSRef(fsRef);
}

void TCFURL::operator=(const TFSSpec &spec) {
	CreateFromFSRef(TFSRef(spec));
}

void TCFURL::CopyFromURL(CFURLRef relativeURL) {
	DeleteRef();
	if (relativeURL != NULL)
		m_ref = CFURLCopyAbsoluteURL(relativeURL);
}

void TCFURL::CopyFromString(CFStringRef filePath, Boolean isDirectory, CFURLPathStyle pathStyle) {
	DeleteRef();
	if (filePath != NULL)
		m_ref = CFURLCreateWithFileSystemPath(kCFAllocatorDefault, filePath, pathStyle, isDirectory);
}

void TCFURL::CopyFromBundle(CFBundleRef bundle) {
	DeleteRef();
	if (bundle != NULL)
		m_ref = CFBundleCopyBundleURL(bundle);
}

void TCFURL::CreateFromFSRef(const TFSRef &fsRef) {
	DeleteRef();
	m_ref = CFURLCreateFromFSRef(kCFAllocatorDefault, fsRef);
}

void TCFURL::CreateFromFSSpec(const TFSSpec &spec) {
	CreateFromFSRef(TFSRef(spec));
}

UInt32 TCFURL::GetPackageType() {
	UInt32 packageType = 0;
	UInt32 packageCreator = 0;
	CFBundleGetPackageInfoInDirectory(m_ref, &packageType, &packageCreator);
	return packageType;
}

UInt32 TCFURL::GetPackageCreator() {
	UInt32 packageType = 0;
	UInt32 packageCreator = 0;
	CFBundleGetPackageInfoInDirectory(m_ref, &packageType, &packageCreator);
	return packageCreator;
}

CFURLRef TCFURL::GetCFURLRef() const {
	return m_ref;
}

CFStringRef TCFURL::GetFileSystemPath(CFURLPathStyle pathStyle) {
	return CFURLCopyFileSystemPath(m_ref, pathStyle);
}

/*
TCFString TCFURL::GetString() {
	return CFURLGetString(m_ref);
}
*/

/*
 *	TFSRef
 *	̂̂t@CVXeIuWFNg
 */

void TFSRef::Init() {
	memset(&m_ref, 0, sizeof(m_ref));
}

TFSRef::TFSRef() {
	Init();
}

TFSRef::TFSRef(const FSRef& fsRef) {
	Init();
	*this = fsRef;
}

TFSRef::TFSRef(const TFSRef& fsRef) {
	Init();
	*this = fsRef;
}

TFSRef::TFSRef(const TCFString& path, Boolean isDirectory, CFURLPathStyle pathStyle) {
	Init();
	MakeByPath(path, isDirectory, pathStyle);
}

TFSRef::TFSRef(const TCFURL &url) {
	Init();
	*this = url;
}

TFSRef::TFSRef(const TFSSpec &spec) {
	Init();
	*this = spec;
}

TFSRef::TFSRef(const TFilePath &filePath) {
	Init();
	*this = filePath;
}

TFSRef::TFSRef(const TDirectory &dir) {
	Init();
	*this = dir;
}

TFSRef::TFSRef(const TFSRef &parent, const TCFString &name) {
	Init();
	THFSUniStr255 us(name);
	MakeByUnicode(parent, us.GetLength(), us);
}

TFSRef::~TFSRef() {
}

TFSRef::operator const FSRef *() const {
	return GetFSRefPtr();
}

//TFSRef::operator const TFSSpec() {
//	return GetFSSpec();
//}

void TFSRef::operator=(const FSRef& fsRef) {
	m_ref = fsRef;
}

void TFSRef::operator=(const TFSRef& fsRef) {
	m_ref = fsRef.m_ref;
}

void TFSRef::operator=(const TCFString &path) {
	MakeByPath(path);
}

void TFSRef::operator=(const TCFURL &url) {
	CopyFromURL(url);
}

void TFSRef::operator=(const TFSSpec &spec) {
	MakeByFSSpec(spec);
}

void TFSRef::operator=(const TFilePath &filePath) {
	HFSUniStr255 us;
	filePath.m_fileName.CopyToHFSUniStr255(&us);
	FSSpec spec;
	FSMakeFSSpec(filePath.m_dir.m_volume, filePath.m_dir.m_dirID, "\p", &spec);
	TFSRef parent(spec);
	if (MakeByUnicode(parent, us.length, us.unicode) != noErr) {
		Init();
	}
}

void TFSRef::operator=(const TDirectory &dir) {
	MakeByFSSpec(TFSSpec(dir));
}

Boolean TFSRef::CopyFromURL(const TCFURL &url) {
	return CFURLGetFSRef(url, &m_ref);
}

OSStatus TFSRef::MakeByPath(const TCFString& path, Boolean isDirectory, CFURLPathStyle pathStyle) {
	TCFURL url(path, isDirectory, pathStyle);
	return CopyFromURL(url);
}

OSStatus TFSRef::MakeByFSSpec(const TFSSpec &spec) {
	return FSpMakeFSRef(spec, &m_ref);
}

OSErr TFSRef::MakeByUnicode(const FSRef *parentRef, int nameLength, const UniChar *name) {
	return FSMakeFSRefUnicode(parentRef, (UniCharCount)nameLength, name, kTextEncodingUnknown, &m_ref);
}

OSErr TFSRef::MakeByHFSUniStr255(const FSRef *parentRef, HFSUniStr255* name) {
	return FSMakeFSRefUnicode(parentRef, (UniCharCount)name->length, name->unicode, kTextEncodingUnknown, &m_ref);
}

OSErr TFSRef::GetCatalogInfo(FSCatalogInfo *catalogInfo, HFSUniStr255 *outName, FSSpec *fsSpec, FSRef *parentRef) {
	return FSGetCatalogInfo(&m_ref, kFSCatInfoGettableInfo, catalogInfo, outName, fsSpec, parentRef);
}

const FSRef *TFSRef::GetFSRefPtr() const {
	return &m_ref;
}

TFSSpec TFSRef::GetFSSpec() const {
	FSCatalogInfo catalogInfo;
	FSSpec fsSpec;
	memset(&fsSpec, 0, sizeof(fsSpec));
	if (FSGetCatalogInfo(&m_ref, kFSCatInfoGettableInfo, &catalogInfo, NULL, &fsSpec, NULL) != noErr)
		memset(&fsSpec, 0, sizeof(fsSpec));
	return fsSpec;
}

TCFString TFSRef::GetName() {
	FSCatalogInfo catalogInfo;
	HFSUniStr255 outName;
	memset(&outName, 0, sizeof(outName));
	if (GetCatalogInfo(&catalogInfo, &outName) == noErr)
		return outName;
	return "";
}

/*
void TCFReadStream::Init() {
	m_ref = NULL;
}
 
void TCFReadStream::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}
*/

void THFSUniStr255::Init() {
	memset(&m_data, 0, sizeof(m_data));
}

THFSUniStr255::THFSUniStr255() {
	Init();
}

THFSUniStr255::THFSUniStr255(const TCFString &s) {
	Init();
	*this = s;
}

THFSUniStr255::THFSUniStr255(const CFStringRef cs) {
	Init();
	*this = cs;
}

THFSUniStr255::THFSUniStr255(const HFSUniStr255 *us) {
	Init();
	*this = us;
}

void THFSUniStr255::operator=(const TCFString &s) {
	operator=(s.GetCFStringRef());
}

void THFSUniStr255::operator=(const CFStringRef cs) {
	CFRange range = {0, 255};
	CFStringGetCharacters(cs, range, m_data.unicode);
	m_data.length = CFStringGetLength(cs);
}

void THFSUniStr255::operator=(const HFSUniStr255 *us) {
	memcpy(&m_data, us, sizeof(m_data));
}

/*
 * TCFString
 * RAt@f[V
 */

static CFStringRef emptyString = CFSTR("");

void TCFString::Init() {
	m_ref = emptyString;
	m_scriptCode = ::IntlScript();
}

void TCFString::DeleteRef() {
	if (m_ref != NULL && m_ref != emptyString)
		CFRelease(m_ref);
	m_ref = emptyString;
}

TCFString::TCFString() {
	Init();
}

TCFString::~TCFString() {
	DeleteRef();
}

TCFString::TCFString(const TCFString &cs) {
	Init();
	*this = cs;
}

TCFString::TCFString(const TString &s) {
	Init();
	*this = s;
}

TCFString::TCFString(const char *cstr) {
	Init();
	*this = cstr;
}

TCFString::TCFString(ConstStr255Param pStr) {
	Init();
	*this = pStr;
}

TCFString::TCFString(const CFStringRef theString) {
	Init();
	*this = theString;
}

//TCFString::TCFString(NSString *theString) {
//	Init();
//	CopyFromString((CFStringRef)theString);
//}

//TCFString::TCFString(const OSType sig) {
//	Init();
//	*this = sig;
//}

TCFString::TCFString(const CFURLRef anURL) {
	Init();
	*this = anURL;
}

TCFString::TCFString(const HFSUniStr255 &us) {
	Init();
	*this = us;
}

TCFString::TCFString(const THFSUniStr255 &us) {
	Init();
	*this = us;
}

TCFString::TCFString(const TUniString &us) {
	Init();
	*this = us;
}

void TCFString::operator=(const TCFString &cs) {
	CopyFromString(cs);
	m_scriptCode = cs.m_scriptCode;
}

void TCFString::operator=(const TString &s) {
	CopyFromCSTR(s, s.GetScriptCode());
}

void TCFString::operator=(const char *cstr) {
	CopyFromCSTR(cstr);
}

void TCFString::operator=(ConstStr255Param pStr) {
	CopyFromPSTR(pStr);
}

void TCFString::operator=(const CFStringRef theString) {
	CopyFromString(theString);
}

//void TCFString::operator=(NSString *theString) {
//	CopyFromString((CFStringRef)theString);
//}

//void TCFString::operator=(const OSType sig) {
//	CopyFromOSType(sig);
//}

void TCFString::operator=(const CFURLRef anURL) {
	CopyFromURL(anURL);
}

void TCFString::operator=(const HFSUniStr255 &us) {
	CopyFromHFSUniStr255(us);
}

void TCFString::operator=(const THFSUniStr255 &us) {
	CopyFromHFSUniStr255(us);
}

void TCFString::operator=(const TUniString &us) {
	CopyFromUSTR(us, us.GetLength());
	SetScriptCode(us.GetScriptCode());
}

//operator NSString *() const {
//	return (NSString *)m_ref;
//}

Boolean TCFString::HasSuffix(TCFString& s) {
	return CFStringHasSuffix(m_ref, s);
}

TCFString TCFString::Append(const TCFString& s) {
	CFMutableStringRef m = CFStringCreateMutableCopy(kCFAllocatorDefault, 0, m_ref);
	CFStringAppend(m, s);
	TCFString t(m);
	t.SetScriptCode(GetScriptCode());
	return t;
}

void TCFString::CopyFromCSTR(const char *cstr, /*CFStringEncoding*/ int encoding) {
	if (encoding == -1)
		encoding = m_scriptCode;
	DeleteRef();
	if (cstr != NULL)
		m_ref = CFStringCreateWithCString(kCFAllocatorDefault, cstr, encoding);
	m_scriptCode = encoding;
}

void TCFString::CopyFromPSTR(ConstStr255Param pStr, /*CFStringEncoding*/ int encoding) {
	if (encoding == -1)
		encoding = m_scriptCode;
	DeleteRef();
	m_ref = CFStringCreateWithPascalString(kCFAllocatorDefault, pStr, encoding);
	m_scriptCode = encoding;
}

void TCFString::CopyFromUSTR(const UniChar *ustr, int length) {
	DeleteRef();
	m_ref = CFStringCreateWithCharacters(kCFAllocatorDefault, ustr, (CFIndex)length);
}

void TCFString::CopyFromString(const CFStringRef theString) {
	DeleteRef();
	if (theString != NULL)
		m_ref = CFStringCreateCopy(kCFAllocatorDefault, theString);
}

void TCFString::CopyFromURL(const CFURLRef anURL) {
	CopyFromString(TCFURL(anURL).GetFileSystemPath());
}

void TCFString::CopyFromOSType(const OSType sig) {
	SetOSType(sig);
}

void TCFString::CopyFromHFSUniStr255(const HFSUniStr255 &us) {
	DeleteRef();
	m_ref = CFStringCreateWithCharacters(kCFAllocatorDefault, us.unicode, (CFIndex)us.length);
}

void TCFString::CopyToPSTR(StringPtr buffer, int bufferSize, /*CFStringEncoding*/ int encoding) const {
	if (encoding == -1)
		encoding = m_scriptCode;
	CFStringGetPascalString(m_ref, buffer, (CFIndex)bufferSize, encoding);
}

void TCFString::CopyToCSTR(char *buffer, int bufferSize, /*CFStringEncoding*/ int encoding) const {
	if (encoding == -1)
		encoding = m_scriptCode;
	CFStringGetCString(m_ref, buffer, (CFIndex)bufferSize, encoding);
}

void TCFString::CopyToUSTR(UniChar *buffer, int bufferSize) const {
	CFRange range = {0, bufferSize};
	CFStringGetCharacters(m_ref, range, buffer);
}

void TCFString::CopyToHFSUniStr255(HFSUniStr255 *us) const {
	memset(us, 0, sizeof(HFSUniStr255));
	CopyToUSTR(us->unicode, 255);
	us->length = GetLength();
}

void TCFString::SetOSType(OSType sig) {
	char bf[5];
	int shift = 24;
	for (int i = 0; i < 4; i++) {
		bf[i] = ((sig >> shift) & 0x000000FF);
		shift -= 8;
	}
	bf[4] = '\0';
	*this = bf;
}

OSType TCFString::GetOSType() {
	OSType sig = 0;
	if (GetLength() == 0)
		return sig;
	char p[16];
	CopyToCSTR(p, 15);
	char buffer[4] = {0x20, 0x20, 0x20, 0x20};
	int i;
	int count = strlen(p);
	for (i = 0; i < 4 && i < count; i++) {
		buffer[i] = p[i];
	}
	int shift = 24;
	for (i = 0; i < 4; i++) {
		unsigned long x = (buffer[i] & 0x000000FF);
		x = (x == 0 ? 0x20 : x);
		x = (x << shift);
		sig |= x;
		shift -= 8;
	}
	return sig;
}

TCFString TCFString::GetForMenu() {
	if (IsEmpty())
		return "";
	TUniString u = *this;
	UniChar *p = u;
	if (*p == ntohs('-')) {
		return (TCFString)"\r" + *this;
	}
	else {
		return *this;
	}	
}

OSErr TCFString::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		TString s = *this;
		char *p = s;
		result->PutDataWithKeyword(keyAEResult, typeChar, (Ptr)p, GetLength());
		return noErr;
	}
	else if (eventID == kAESetData) {
		*this = ae.GetParamString();
		return noErr;
	}
	else if (eventID == kAEDelete) {
		return noErr;
	}
	return errAEEventNotHandled;
}

Boolean TCFString::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	if (obj->GetObjectType() != GetObjectType())
		return FALSE;
	TCFString s = *(TCFString *)obj;
	int r = Compare(s);
	if (oper == kAEGreaterThan)
		return r > 0;
	else if (oper == kAEGreaterThanEquals)
		return r >= 0;
	else if (oper == kAEEquals)
		return r == 0;
	else if (oper == kAELessThan)
		return r < 0;
	else if (oper == kAELessThanEquals)
		return r <= 0;
	else
		return FALSE;
}

/*
 * TCFBundle
 * oh
 */

void TCFBundle::Init() {
	m_ref = NULL;
	m_created = false;
}

void TCFBundle::DeleteRef() {
	if (m_ref != NULL && m_created)
		CFRelease(m_ref);
	m_ref = NULL;
	m_created = false;
}

TCFBundle::TCFBundle() {
	Init();
}

TCFBundle::~TCFBundle() {
	DeleteRef();
}

TCFBundle::TCFBundle(const CFURLRef url) {
	Init();
	*this = url;
}

//TCFBundle::TCFBundle(const CFPlugInRef plugin) {
//	Init();
//	m_ref = CFPlugInGetBundle(plugin);
//}

//void TCFBundle::operator=(const CFURLRef url) {
//	Create(url);
//}

UInt32 TCFBundle::GetPackageType() {
	UInt32 packageType = 0;
	UInt32 packageCreator = 0;
	CFBundleGetPackageInfo(m_ref, &packageType, &packageCreator);
	return packageType;
}

UInt32 TCFBundle::GetPackageCreator() {
	UInt32 packageType = 0;
	UInt32 packageCreator = 0;
	CFBundleGetPackageInfo(m_ref, &packageType, &packageCreator);
	return packageCreator;
}

//void TCFBundle::Create(const CFURLRef url) {
//	m_ref = CFBundleCreate(kCFAllocatorDefault, url);
//	m_created = true;
//}

/*
 * TCFArray
 *@z
 */

void TCFArray::Init() {
	m_ref = NULL;
}

void TCFArray::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

TCFArray::TCFArray() {
	Init();
	m_ref = CFArrayCreateMutable(kCFAllocatorDefault, 0, NULL);
}

TCFArray::~TCFArray() {
	RemoveDeleteAllValues();
	DeleteRef();
}

TCFArray::TCFArray(const CFArrayRef theArray) {
	Init();
	*this = theArray;
}

/*
TCFArray::TCFArray(NSArray *theArray) {
	Init();
	CopyFromArray((CFArrayRef)theArray);
}
*/

void TCFArray::operator=(const CFArrayRef theArray) {
	CopyFromArray(theArray);
}

void TCFArray::CopyFromArray(CFArrayRef theArray) {
	if (theArray != NULL)
		m_ref = CFArrayCreateMutableCopy(kCFAllocatorDefault, 0, theArray);
}

const void *TCFArray::RemoveValueAtIndex(int idx) {
	const void *item = GetValueAtIndex(idx);
	CFArrayRemoveValueAtIndex(m_ref, (CFIndex)idx);
	return item;
}

void TCFArray::RemoveDeleteAllValues() {
	if (m_ref != NULL) {
		for (int i = 0; i < GetCount(); i++)
			DeleteItem(GetValueAtIndex(i));
		RemoveAllValues();
	}
}

void TCFArray::RemoveAllValues() {
	if (m_ref != NULL)
		CFArrayRemoveAllValues(m_ref);
}

void TCFArray::SortValues(CFComparatorFunction comparator, void *context) {
	CFRange range = {0, GetCount()};
	CFArraySortValues(m_ref, range, comparator, context);
}

int TCFArray::GetCountOfValue(const void *value) {
	CFRange range = {0, GetCount()};
	return (int)CFArrayGetCountOfValue(m_ref, range, value);
}

int TCFArray::GetOrder(const void *value) {
	for (int i = 0; i < GetCount(); i++) {
		if (GetValueAtIndex(i) == value)
			return i;
	}
	return -1;
}

CFComparisonResult TCFStringArray::Comparator(const void *val1, const void *val2, void *context) {
	TCFString *s1 = (TCFString *)val1;
	TCFString *s2 = (TCFString *)val2;
	return (CFComparisonResult)(*s1).Compare(*s2);
}

void TCFData::Init() {
	m_ref = NULL;
}

void TCFData::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

TCFData::TCFData() {
	Init();
}

TCFData::~TCFData() {
	DeleteRef();
}

TCFData::TCFData(int length) {
	Init();
	Create();
	SetLength(length);
}

TCFData::TCFData(const void *bytes, int length) {
	Init();
	CopyFromBytes(bytes, length);
}

/*
TCFData::TCFData(TResource res) {
	Init();
	CopyFromBytes((const UInt8 *)res.lock(), res.getSize());
	res.unlock();
}
*/

void TCFData::CopyFromBytes(const void *bytes, int length) {
	CFDataRef dt = CFDataCreateWithBytesNoCopy(kCFAllocatorDefault, (const UInt8 *)bytes, (CFIndex)length, kCFAllocatorNull);
	m_ref = CFDataCreateMutableCopy(kCFAllocatorDefault, 0, dt);
}

void TCFData::GetBytes(int offset, int length, void *buffer) {
	CFRange range = {offset, length};
	CFDataGetBytes(m_ref, range, (UInt8 *)buffer);
}

/*
TBinaryReader::TBinaryReader(const CFDataRef data) {
	m_ref = data;
	m_position = 0;
}
 
void TBinaryReader::GetBytes(int offset, int length, void *buffer) {
	CFRange range = {offset, length};
	CFDataGetBytes(m_ref, range, (UInt8 *)buffer);
}
 
int TBinaryReader::ReadInt() {
	int result;
	GetBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return ntohl(result);
}
 
short TBinaryReader::ReadShort() {
	short result;
	GetBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return ntohs(result);
}
 
char TBinaryReader::ReadChar() {
	char result;
	GetBytes(m_position, sizeof(result), &result);
	m_position += sizeof(result);
	return result;
}
*/

/*
 * TCFDictionary
 * fBNVii}bvj
 */

void TCFDictionary::Init() {
	m_ref = NULL;
}

void TCFDictionary::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

TCFDictionary::TCFDictionary() {
	Init();
	m_ref = CFDictionaryCreateMutable(kCFAllocatorDefault, 0, NULL, NULL);
}

TCFDictionary::~TCFDictionary() {
	DeleteRef();
}

TCFDictionary::TCFDictionary(CFDictionaryRef theDict) {
	Init();
	*this = theDict;
}

void TCFDictionary::operator=(const CFDictionaryRef theDict) {
	CopyFromDictionary(theDict);
}

/*
 * TCFPlugIn
 * vOC
 */

void TCFPlugIn::Init() {
	m_ref = NULL;
}

void TCFPlugIn::DeleteRef() {
	if (m_ref != NULL)
		CFRelease(m_ref);
	m_ref = NULL;
}

TCFPlugIn::TCFPlugIn() {
	Init();
}

TCFPlugIn::~TCFPlugIn() {
	DeleteRef();
}

TCFPlugIn::TCFPlugIn(const CFURLRef url) {
	Init();
	*this = url;
}

void TCFPlugIn::operator=(const CFURLRef url) {
	m_ref = CFPlugInCreate(kCFAllocatorDefault, url);
}

