/*
 *  cftools.h
 *  HUNDOSHI-EDIT
 *
 *  Created by ostra on 2005.08.09.
 *  Copyright (C) Takeshi Yoneki / OSTRA / OSTRACISM CO., 2005
 *
 */

#ifndef _CFTOOLS_H_
#define _CFTOOLS_H_

//#include "MacIF.h"
//#include "ctoolsclass.h"

class TCFURL {
	CFURLRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFURL();
	TCFURL(const TCFURL &url);
	TCFURL(const CFURLRef relativeURL);
	TCFURL(const CFStringRef filePath, Boolean isDirectory = false, CFURLPathStyle pathStyle = kCFURLHFSPathStyle);
	TCFURL(const CFBundleRef bundle);
	TCFURL(const TFSRef &fsRef);
	TCFURL(const TFSSpec &spec);
	operator CFURLRef() const;
	operator CFStringRef();
	void operator=(const TCFURL &url);
	void operator=(const CFURLRef relativeURL);
	void operator=(const CFStringRef filePath);
	void operator=(const CFBundleRef bundle);
	void operator=(const TFSRef &fsRef);
	void operator=(const TFSSpec &spec);
	void CopyFromURL(const CFURLRef relativeURL);
	void CopyFromString(const CFStringRef filePath, Boolean isDirectory = false, CFURLPathStyle pathStyle = kCFURLHFSPathStyle);
	void CopyFromBundle(const CFBundleRef bundle);
	void CreateFromFSRef(const TFSRef &fsRef);
	void CreateFromFSSpec(const TFSSpec &spec);
	UInt32 GetPackageType();
	UInt32 GetPackageCreator();
	CFURLRef GetCFURLRef() const;
	CFStringRef GetFileSystemPath(CFURLPathStyle pathStyle = kCFURLHFSPathStyle);
	//TCFString GetString();
};

class TFSRef {
	FSRef m_ref;
	void Init();
public:
	TFSRef();
	TFSRef(const FSRef& fsRef);
	TFSRef(const TFSRef& fsRef);
	TFSRef(const TCFString &path, Boolean isDirectory = false, CFURLPathStyle pathStyle = kCFURLHFSPathStyle);
	TFSRef(const TCFURL &url);
	TFSRef(const TFSSpec &spec);
	TFSRef(const TFilePath &filePath);
	TFSRef(const TDirectory &dir);
	TFSRef(const TFSRef &parent, const TCFString &name);
	~TFSRef();
	operator const FSRef *() const;
	//	operator const TFSSpec();
	void operator=(const FSRef& fsRef);
	void operator=(const TFSRef& fsRef);
	void operator=(const TCFString &path);
	void operator=(const TCFURL &url);
	void operator=(const TFSSpec &spec);
	void operator=(const TFilePath &filePath);
	void operator=(const TDirectory &dir);
	//	operator const FSRef() {
	//		return m_ref;
	//	}
	Boolean CopyFromURL(const TCFURL &url);
	OSStatus MakeByPath(const TCFString& path, Boolean isDirectory = false, CFURLPathStyle pathStyle = kCFURLHFSPathStyle);
	OSStatus MakeByFSSpec(const TFSSpec &spec);
	OSErr MakeByUnicode(const FSRef *parentRef, int nameLength, const UniChar *name);
	OSErr MakeByHFSUniStr255(const FSRef *parentRef, HFSUniStr255* name);
	OSErr GetCatalogInfo(FSCatalogInfo *catalogInfo, HFSUniStr255 *outName = NULL, FSSpec *fsSpec = NULL, FSRef *parentRef = NULL);
	const FSRef *GetFSRefPtr() const;
	TFSSpec GetFSSpec() const;
	TCFString GetName();
};

class THFSUniStr255 {
public:
	HFSUniStr255 m_data;
	THFSUniStr255();
	THFSUniStr255(const TCFString &s);
	THFSUniStr255(const CFStringRef cs);
	THFSUniStr255(const HFSUniStr255 *us);
	void operator=(const TCFString &s);
	void operator=(const CFStringRef cs);
	void operator=(const HFSUniStr255 *us);
	operator const UniChar *() const {
		return m_data.unicode;
	}
	operator  UniChar *() {
		return m_data.unicode;
	}
	operator const HFSUniStr255 *() const {
		return &m_data;
	}
	operator HFSUniStr255 *() {
		return &m_data;
	}
	operator const HFSUniStr255() const {
		return m_data;
	}
	operator HFSUniStr255() {
		return m_data;
	}
	int GetLength() const {
		return m_data.length;
	}
	const UniChar *GetUniChar() const {
		return m_data.unicode;
	}
	void GetResourceForkName() {
		FSGetResourceForkName(&m_data);
	}
	void GetDataForkName() {
		FSGetDataForkName(&m_data);
	}
protected:
	void Init();
};


/*
class TCFReadStream {
	CFReadStreamRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFReadStream(const TCFURL& url) {
		Init();
		MakeByURL(url);
	}
	operator CFReadStreamRef() const {
		return m_ref;
	}
	Boolean Open() {
		return CFReadStreamOpen(m_ref);
	}
	void Close() {
		CFReadStreamClose(m_ref);
	}
	int Read(void *buffer, int bufferLength) {
		return CFReadStreamRead(m_ref, (UInt8 *)buffer, (CFIndex)bufferLength);
	}
	void MakeByURL(const TCFURL& url) {
		m_ref = CFReadStreamCreateWithFile(kCFAllocatorDefault, url);
	}
};
*/

class TCFString : public TAppleObject {
	CFStringRef m_ref;
	void Init();
	void DeleteRef();
	int m_scriptCode;	// CPascalɕϊۂ̃XNvgR[hB
public:
	TCFString();
	~TCFString();
	TCFString(const TCFString &cs);
	TCFString(const TString &s);
	TCFString(const char *cstr);
	TCFString(ConstStr255Param pStr);
	TCFString(const CFStringRef theString);
	//	TCFString(NSString *theString);
	//	TCFString(const OSType sig);
	TCFString(const CFURLRef anURL);
	TCFString(const HFSUniStr255 &us);
	TCFString(const THFSUniStr255 &us);
	TCFString(const TUniString &us);
	void operator=(const TCFString &cs);
	void operator=(const TString &s);
	void operator=(const char *cstr);
	void operator=(ConstStr255Param pStr);
	void operator=(const CFStringRef theString);
	//	void operator=(NSString *theString);
	//	void operator=(const OSType sig);
	void operator=(const CFURLRef anURL);
	void operator=(const HFSUniStr255 &us);
	void operator=(const THFSUniStr255 &us);
	void operator=(const TUniString &us);
	//	operator NSString *() const {
	//		return (NSString *)m_ref;
	//	}
	operator CFStringRef() const {
		return GetCFStringRef();
	}
	//	operator const CFStringRef() {
	//		return m_ref;
	//	}
	//	operator const char *() {
	//		return CFStringGetCStringPtr(m_ref, IntlScript());
	//	}
	//	operator const UniChar *() const {
	//		return CFStringGetCharactersPtr(m_ref);
	//	}
	operator OSType() {
		return GetOSType();
	}
	CFStringRef GetCFStringRef() const {
		return m_ref;
	}
	int Compare(const TCFString &s, CFOptionFlags option = 0) {
		return (int)CFStringCompare(m_ref, s.m_ref, option);
	}
	//	int CompareNoCase(const TCFString &s) {
	//		return CompareIgnoreCase(s);
	//	}
	int CompareIgnoreCase(const TCFString &s) {
		return Compare(s, kCFCompareCaseInsensitive);
	}
	Boolean Equals(const TCFString &s) {
		return Compare(s) == 0;
	}
	Boolean EqualsIgnoreCase(const TCFString &s) {
		return Compare(s, kCFCompareCaseInsensitive) == 0;
	}
	Boolean operator==(const TCFString &s) {
		return Equals(s);
	}
	Boolean operator!=(const TCFString &s) {
		return !Equals(s);
	}
	int GetLength() const {
		return (int)CFStringGetLength(m_ref);
	}
	Boolean IsNull() {
		return m_ref == NULL;
	}
	Boolean IsEmpty() {
		return GetLength() == 0;
	}
	TCFString operator+(const TCFString &s) {
		return Append(s);
	}
	TCFString operator+(const char *cstr) {
		return Append(cstr);
	}
	Boolean HasSuffix(TCFString& s);
	TCFString Append(const TCFString& s);
	int GetScriptCode() const {
		return m_scriptCode;
	}
	void SetScriptCode(int scriptCode) {
		m_scriptCode = scriptCode;
	}
	void CopyFromCSTR(const char *cstr, /*CFStringEncoding*/ int encoding = -1);
	void CopyFromPSTR(ConstStr255Param pStr, /*CFStringEncoding*/ int encoding = -1);
	void CopyFromUSTR(const UniChar *ustr, int length);
	void CopyFromString(const CFStringRef theString);
	void CopyFromOSType(const OSType sig);
	void CopyFromURL(const CFURLRef anURL);
	void CopyFromHFSUniStr255(const HFSUniStr255 &us);
	void CopyToPSTR(StringPtr buffer, int bufferSize = 255, /*CFStringEncoding*/int encoding = -1) const;
	void CopyToCSTR(char *buffer, int bufferSize, /*CFStringEncoding*/ int encoding = -1) const;
	void CopyToUSTR(UniChar *buffer, int bufferSize) const;
	void CopyToHFSUniStr255(HFSUniStr255 *us) const;
	void CopyTo(Str255 ps) {
		CopyToPSTR(ps);
	}
	void SetOSType(OSType sig);
	OSType GetOSType();
	TCFString GetForMenu();
	/* for AppleScript */
	virtual DescType GetObjectType() {
		return classCFString;	/* 'CFSt' */
	}
	virtual OSErr OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result);
	virtual Boolean CompareObjects(DescType oper, TAppleObject *obj);
};

class TCFBundle {
	CFBundleRef m_ref;
	Boolean m_created;
	void Init();
	void DeleteRef();
public:
	TCFBundle();
	~TCFBundle();
	TCFBundle(const CFURLRef url);
	TCFBundle(const CFPlugInRef plugin) {
		Init();
		*this = plugin;
	}
	operator CFBundleRef() {
		return GetCFBundleRef();
	}
	CFBundleRef GetCFBundleRef() const {
		return m_ref;
	}
	void operator=(const CFPlugInRef plugin) {
		m_ref = CFPlugInGetBundle(plugin);
	}
	void operator=(const CFURLRef url) {
		Create(url);
	}
	CFStringRef GetIdentifier() {
		return CFBundleGetIdentifier(m_ref);
	}
	UInt32 GetPackageType();
	UInt32 GetPackageCreator();
	void Create(const CFURLRef url) {
		m_ref = CFBundleCreate(kCFAllocatorDefault, url);
		m_created = true;
	}
	CFDictionaryRef GetInfoDictionary() {
		return CFBundleGetInfoDictionary(m_ref);
	}
	short OpenBundleResourceMap() {
		return CFBundleOpenBundleResourceMap(m_ref);
	}
	void CloseBundleResourceMap(short refNum) {
		CFBundleCloseBundleResourceMap(m_ref, refNum);
	}
	Boolean IsNull() {
		return m_ref == NULL;
	}
};

//
// z
//
class TCFArray {
protected:
	CFMutableArrayRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFArray();
	~TCFArray();
	TCFArray(const CFArrayRef theArray);
	//TCFArray(NSArray *theArray);
	//	operator NSMutableArray *() const {
	//		return (NSMutableArray *)m_ref;
	//	}
	void operator=(const CFArrayRef theArray);
	Boolean IsNull() {
		return m_ref == NULL;
	}
	void CopyFromArray(CFArrayRef theArray);
	void AppendValue(const void *value) {
		CFArrayAppendValue(m_ref, value);
	}
	int GetCount() {
		return (int)CFArrayGetCount(m_ref);
	}
	const void *operator[](int idx) {
		return GetValueAtIndex(idx);
	}
	void Add(const void *value) {
		AppendValue(value);
	}
	const void *GetValueAtIndex(int idx) {
		return CFArrayGetValueAtIndex(m_ref, (CFIndex)idx);
	}
	void RemoveAllValues();
	void RemoveDeleteAllValues();
	const void *RemoveValueAtIndex(int idx);
	void SortValues(CFComparatorFunction comparator, void *context);
	int GetCountOfValue(const void *value);
	int GetOrder(const void *value);
	virtual void DeleteItem(const void *item) {
	}
};

//
// ̔z
//
class TCFStringArray : public TCFArray {
public:
	TCFString *operator[](int idx) {
		return (TCFString *)GetValueAtIndex(idx);
	}
	virtual void DeleteItem(const void *item) {
		delete (TCFString *)item;
	}
	static CFComparisonResult Comparator(const void *val1, const void *val2, void *context);
	void Sort() {
		SortValues(Comparator, NULL);
	}
};

//
// ubN
//
class TCFData {
protected:
	CFMutableDataRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFData();
	~TCFData();
	TCFData(int length);
	TCFData(const void *bytes, int length);
	//TCFData(TResource res);
	operator void *() {
		return GetBytePtr();
	}
	operator CFMutableDataRef() const {
		return m_ref;
	}
	operator CFDataRef() const {
		return m_ref;
	}
	void SetLength(int length) {
		CFDataSetLength(m_ref, (CFIndex)length);
	}
	void Create(int capacity = 0) {
		m_ref = CFDataCreateMutable(kCFAllocatorDefault, (CFIndex)capacity);
	}
	void CopyFromBytes(const void *bytes, int length);
	void *GetBytePtr() {
		return (void *)CFDataGetMutableBytePtr(m_ref);
	}
	int GetLength() {
		return CFDataGetLength(m_ref);
	}
	void GetBytes(int offset, int length, void *buffer);
};

//
// ubÑoCiǂݍ
//
/*
class TBinaryReader {
	CFDataRef m_ref;
	int m_position;
public:
	TBinaryReader(const CFDataRef data);
	int GetLength() {
		return CFDataGetLength(m_ref);
	}
	void ResetPosition() {
		m_position = 0;
	}
	void GetBytes(int offset, int length, void *buffer);
	int ReadInt();
	short ReadShort();
	char ReadChar();
};
*/

//
// fBNVii}bvj
//
class TCFDictionary {
protected:
	CFMutableDictionaryRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFDictionary();
	~TCFDictionary();
	TCFDictionary(const CFDictionaryRef theDict);
	operator CFMutableDictionaryRef() const {
		return m_ref;
	}
	void operator=(const CFDictionaryRef theDict);
	int GetCount() {
		return (int)CFDictionaryGetCount(m_ref);
	}
	const void *GetValue(CFStringRef key) {
		return CFDictionaryGetValue(m_ref, (const void *)key);
	}
	void GetKeysAndValues(const void **keys, const void **values) {
		CFDictionaryGetKeysAndValues(m_ref, keys, values);
	}
	void CopyFromDictionary(CFDictionaryRef theDict) {
		m_ref = CFDictionaryCreateMutableCopy(kCFAllocatorDefault, 0, theDict);
	}
};

class TCFPlugIn {
protected:
	CFPlugInRef m_ref;
	void Init();
	void DeleteRef();
public:
	TCFPlugIn();
	~TCFPlugIn();
	TCFPlugIn(const CFURLRef url);
	operator CFPlugInRef() const {
		return m_ref;
	}
	void operator=(const CFURLRef url);
	CFPlugInRef GetCFPlugInRef() {
		return m_ref;
	}
	Boolean IsNull() {
		return m_ref == NULL;
	}
};

#endif
