#include "csystem.h"
#include "ckanji.h"

UInt16 MakeBOM(char *p) {
	return (p[0] << 8) + p[1];
}

TUnicodeTextEncodingInfo::TUnicodeTextEncodingInfo() {
	Init();
}

void TUnicodeTextEncodingInfo::Init() {
	m_textEncoding = 0;
	m_unicodeEncoding = 0;
	m_unicodeToTextInfo = NULL;
	m_textToUnicodeInfo = NULL;
	m_utf8ToTextInfo = NULL;
	m_textToUTF8Info = NULL;
	m_error = 0;
}

TUnicodeTextEncodingInfo::TUnicodeTextEncodingInfo(TextEncodingBase encodingBase) {
	Init();
	Create(encodingBase);
}

OSStatus TUnicodeTextEncodingInfo::Create(TextEncodingBase encodingBase) {
	Dispose();
	Init();
	m_error = -1;
	m_textEncoding = ::CreateTextEncoding(encodingBase, kTextEncodingDefaultVariant, kTextEncodingDefaultFormat);
	m_unicodeEncoding = ::CreateTextEncoding(kTextEncodingUnicodeDefault, kTextEncodingDefaultVariant, kUnicode16BitFormat/*kTextEncodingDefaultFormat*/);
	m_utf8Encoding = ::CreateTextEncoding(kTextEncodingUnicodeDefault, kTextEncodingDefaultVariant, kUnicodeUTF8Format);
	// m_textEncoding0肤H 2004.09
	//	if (m_textEncoding == NULL || m_unicodeEncoding == NULL || m_utf8Encoding == NULL)
	//		return m_error;
	UnicodeMapping exchangeMap;
	exchangeMap.unicodeEncoding = m_unicodeEncoding;
	exchangeMap.otherEncoding = m_textEncoding;
	exchangeMap.mappingVersion = kUnicodeUseLatestMapping;
	m_error = ::CreateUnicodeToTextInfo(&exchangeMap, &m_unicodeToTextInfo);
	if (m_error != noErr)
		return m_error;
	exchangeMap.unicodeEncoding = m_unicodeEncoding;
	exchangeMap.otherEncoding = m_textEncoding;
	exchangeMap.mappingVersion = kUnicodeUseLatestMapping;
	m_error = ::CreateTextToUnicodeInfo(&exchangeMap, &m_textToUnicodeInfo);
	if (m_error != noErr)
		return m_error;
	exchangeMap.unicodeEncoding = m_utf8Encoding;
	exchangeMap.otherEncoding = m_textEncoding;
	exchangeMap.mappingVersion = kUnicodeUseLatestMapping;
	m_error = ::CreateUnicodeToTextInfo(&exchangeMap, &m_utf8ToTextInfo);
	if (m_error != noErr)
		return m_error;
	exchangeMap.unicodeEncoding = m_utf8Encoding;
	exchangeMap.otherEncoding = m_textEncoding;
	exchangeMap.mappingVersion = kUnicodeUseLatestMapping;
	m_error = ::CreateTextToUnicodeInfo(&exchangeMap, &m_textToUTF8Info);
	if (m_error != noErr)
		return m_error;
	char bf[2] = "\0";
	for (int i = 0; i < 256; i++) {
		bf[0] = i;
		if (CharacterByteType(bf, 0, encodingBase) == smFirstByte)
			m_charByteData[i] = 2;
		else
			m_charByteData[i] = 1;
	}
	return m_error;
}

TUnicodeTextEncodingInfo::~TUnicodeTextEncodingInfo() {
	Dispose();
}

void TUnicodeTextEncodingInfo::Dispose() {
	if (m_unicodeToTextInfo != NULL)
		::DisposeUnicodeToTextInfo(&m_unicodeToTextInfo);
	if (m_textToUnicodeInfo != NULL)
		::DisposeTextToUnicodeInfo(&m_textToUnicodeInfo);
	m_unicodeToTextInfo = NULL;
	m_textToUnicodeInfo = NULL;
}

OSStatus TUnicodeTextEncodingInfo::ConvertFromUnicodeToText(ByteCount iUnicodeLen, const UniChar iUnicodeStr[], ByteCount iOutputBufLen, ByteCount *oOutputLen, LogicalAddress oOutputStr) {
	OSStatus err = 0;
	ByteCount oInputRead = 0;
	ByteOffset iOffsetArray = 0;
	ItemCount oOffsetCount = 0;
	ByteOffset oOffsetArray = 0;
	err = ::ConvertFromUnicodeToText(
	          m_unicodeToTextInfo, iUnicodeLen,
	          iUnicodeStr,
	          kUnicodeUseFallbacksMask | kUnicodeLooseMappingsMask,
	          0, &iOffsetArray, &oOffsetCount,
	          &oOffsetArray,
	          iOutputBufLen, &oInputRead,
	          oOutputLen, oOutputStr);
	return err;
}

OSStatus TUnicodeTextEncodingInfo::ConvertFromTextToUnicode(ByteCount iSourceLen, ConstLogicalAddress iSourceStr, ByteCount iOutputBufLen, ByteCount *oUnicodeLen, UniChar oUnicodeStr[]) {
	OSStatus err = 0;
	ByteCount oInputRead = 0;
	ByteOffset iOffsetArray = 0;
	ItemCount oOffsetCount = 0;
	ByteOffset oOffsetArray = 0;
	err = ::ConvertFromTextToUnicode(m_textToUnicodeInfo, iSourceLen,
	                                 iSourceStr,
	                                 0,
	                                 0, &iOffsetArray, &oOffsetCount,
	                                 &oOffsetArray, iOutputBufLen, &oInputRead,
	                                 oUnicodeLen, oUnicodeStr);
	return err;
}

OSStatus TUnicodeTextEncodingInfo::ConvertFromUTF8ToText(ByteCount iUnicodeLen, const UniChar *iUnicodeStr, ByteCount iOutputBufLen, ByteCount *oOutputLen, LogicalAddress oOutputStr) {
	OSStatus err = 0;
	ByteCount oInputRead = 0;
	ByteOffset iOffsetArray = 0;
	ItemCount oOffsetCount = 0;
	ByteOffset oOffsetArray = 0;
	err = ::ConvertFromUnicodeToText(
	          m_utf8ToTextInfo, iUnicodeLen,
	          iUnicodeStr,
	          kUnicodeUseFallbacksMask | kUnicodeLooseMappingsMask,
	          0, &iOffsetArray, &oOffsetCount,
	          &oOffsetArray,
	          iOutputBufLen, &oInputRead,
	          oOutputLen, oOutputStr);
	return err;
}

OSStatus TUnicodeTextEncodingInfo::ConvertFromTextToUTF8(ByteCount iSourceLen, ConstLogicalAddress iSourceStr, ByteCount iOutputBufLen, ByteCount *oUnicodeLen, UniChar oUnicodeStr[]) {
	OSStatus err = 0;
	ByteCount oInputRead = 0;
	ByteOffset iOffsetArray = 0;
	ItemCount oOffsetCount = 0;
	ByteOffset oOffsetArray = 0;
	err = ::ConvertFromTextToUnicode(m_textToUTF8Info, iSourceLen,
	                                 iSourceStr,
	                                 0,
	                                 0, &iOffsetArray, &oOffsetCount,
	                                 &oOffsetArray, iOutputBufLen, &oInputRead,
	                                 oUnicodeLen, oUnicodeStr);
	return err;
}

TString TUnicodeTextEncodingInfo::HFSUniStrToText(HFSUniStr255 *uniText) {
	char buffer[256];
	memset(buffer, 0, 256);
	ByteCount oOutputLen = 0;
	OSStatus result = ConvertFromUnicodeToText(uniText->length * sizeof(UniChar), uniText->unicode, 255, &oOutputLen, buffer);
	if (result == noErr)
		return buffer;
	return "";
}

int TUnicodeTextEncodingInfo::UnicodeToTextChar(UInt16 bom, char *srcPtr, char *dstPtr, int *srcBytes, int *dstBytes) {
	if (*srcBytes < 2) {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_MIS;
	}

	char dst[4];
	ByteCount oInputRead = 0;
	ByteCount oOutputLen = 0;
	ByteOffset iOffsetArray = 0;
	ItemCount oOffsetCount = 0;
	ByteOffset oOffsetArray = 0;
	dst[0] = 0;
	dst[1] = 0;
	UInt16 uni;
	if (bom == 0xFFFE)	// little
		uni = (srcPtr[1] << 8) + srcPtr[0];
	else	// big
		uni = (srcPtr[0] << 8) + srcPtr[1];
	OSStatus result = ConvertFromUnicodeToText(2, &uni, 2, &oOutputLen, dst);

	*srcBytes = 2;

	/*  */
	if (dst[0] != 0 && dst[1] != 0) {
		dstPtr[0] = dst[0];
		dstPtr[1] = dst[1];
		*dstBytes = 2;
		return CHAR_KIND_KANJI;
	}
	else if (dst[0] != 0) {
		/* ANK */
		if (dst[0] >= 0x20 && dst[0] <= 0x7E) {
			dstPtr[0] = dst[0];
			*dstBytes = 1;
			return CHAR_KIND_ANK;
		}
		/* CTRL */
		else if (dst[0] < 0x80) {
			dstPtr[0] = '.';
			*dstBytes = 1;
			return CHAR_KIND_CTRL;
		}
		/*  */
		else {
			dstPtr[0] = dst[0];
			*dstBytes = 1;
			return CHAR_KIND_EXT;
		}
	}
	/* ~Xqbg */
	dstPtr[0] = '.';
	*dstBytes = 1;
	return CHAR_KIND_MIS;
}

TString TUnicodeTextEncodingInfo::UnicodeToText(char *uniText, int bytes) {
	char *buffer = new char [bytes * 2 + 10];
	char *srcText;
	char *dstText = buffer;
	int rest;

	UInt16 bom = MakeBOM(uniText);
	if (bom == BOM_LITTLE || bom == BOM_BIG) {
		srcText = &uniText[2];
		rest = bytes - 2;
	}
	else {
		bom = BOM_BIG;
		srcText = &uniText[0];
		rest = bytes;
	}

	for (; rest > 0 && (srcText[0] != 0 || srcText[1] != 0);) {
		int charKind;
		int srcBytes = rest;
		int dstBytes = 0;

		charKind = UnicodeToTextChar(bom, srcText, dstText, &srcBytes, &dstBytes);

		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	*dstText = '\0';
	TString s = buffer;
	delete [] buffer;
	return s;
}

void TUnicodeTextEncodingInfo::TextToUnicodeChar(UInt16 bom, char *srcPtr, char *dstPtr, int *srcBytes, int *dstBytes) {
	ByteCount oOutputLen = 0;
	UniChar uni[4];
	OSStatus result = -1;
	Boolean miss = FALSE;

	int bytes = m_charByteData[*srcPtr & 0xFF];
	if (bytes > *srcBytes) {
		uni[0] = '.';
		*srcBytes = 1;
	}
	else {
		result = ConvertFromTextToUnicode(bytes, srcPtr, 2, &oOutputLen, uni);
		*srcBytes = bytes;
	}

	*dstBytes = 2;

	if (oOutputLen < 2)
		uni[0] = '.';

	if (bom == BOM_LITTLE) {
		dstPtr[0] = (uni[0] & 0xFF);
		dstPtr[1] = ((uni[0] >> 8) & 0xFF);
	}
	else {
		dstPtr[0] = ((uni[0] >> 8) & 0xFF);
		dstPtr[1] = (uni[0] & 0xFF);
	}
}

void TUnicodeTextEncodingInfo::TextToUnicode(UInt16 bom, TMemory *outUni, const TString &inStr) {
	char *srcText = inStr;
	char *buffer = new char [inStr.GetLength() * 2 + 10];
	char *dstText = buffer;
	int rest = inStr.GetLength();

	dstText[0] = ((bom >> 8) & 0xFF);
	dstText[1] = (bom & 0xFF);
	dstText += 2;

	for (; rest > 0;) {
		int srcBytes = rest;
		int dstBytes = 0;

		TextToUnicodeChar(bom, srcText, dstText, &srcBytes, &dstBytes);

		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	dstText[0] = '\0';
	dstText[1] = '\0';
	dstText += 2;

	int count = dstText - buffer;
	outUni->ChangeLength(count);
	memcpy((char *)*outUni, buffer, count);
}

TString TUnicodeTextEncodingInfo::UTF8ToText(TString &inStr) {
	int inLength = inStr.GetLength();
	if (inLength == 0)
		return "";
	char *buffer = new char[inLength * 2];
	memset(buffer, 0, inLength * 2);
	ByteCount oOutputLen = 0;
	OSStatus err = ConvertFromUTF8ToText(inLength, (const UniChar *)(char *)inStr, inLength * 2, &oOutputLen, buffer);
	if (err != noErr || oOutputLen == 0)
		return "";
	buffer[oOutputLen] = 0;
	TString s(buffer);
	delete [] buffer;
	return s;
}

TString TUnicodeTextEncodingInfo::TextToUTF8(TString &inStr) {
	int inLength = inStr.GetLength();
	if (inLength == 0)
		return "";
	char *buffer = new char[inLength * 4];
	memset(buffer, 0, inLength * 4);
	ByteCount oUnicodeLen = 0;
	OSStatus err = ConvertFromTextToUTF8(inLength, (char *)inStr, inLength * 4, &oUnicodeLen, (UniChar*)buffer);
	if (err != noErr || oUnicodeLen == 0)
		return "";
	buffer[oUnicodeLen] = 0;
	TString s(buffer);
	delete [] buffer;
	return s;
}

/*
	SJISJIS7EUCɕϊ
*/

void Kanji_ToJIS7(char *srcPtr, char *dstPtr) {
	int m, n, d1, d2;
	int s1 = (srcPtr[0] & 0x00FF);
	int s2 = (srcPtr[1] & 0x00FF);
	if (s1 >= 0x81 && s1 <= 0x9F)
		m = 0x81;
	else
		m = 0xC1;
	if (s2 >= 0x40 && s2 <= 0x9E)
		n = 0x21;
	else
		n = 0x22;
	d1 = (s1 - m) * 2 + n;
	if (s2 >= 0x40 && s2 <= 0x7E)
		d2 = s2 - 0x1F;
	else if (s2 >= 0x80 && s2 <= 0x9E)
		d2 = s2 - 0x20;
	else
		d2 = s2 - 0x7E;
	dstPtr[0] = d1;
	dstPtr[1] = d2;
}

void Kanji_ToEUC(char *srcPtr, char *dstPtr) {
	Kanji_ToJIS7(srcPtr, dstPtr);
	dstPtr[0] |= 0x80;
	dstPtr[1] |= 0x80;
}

/*
	EUCJIS7̊R[hSJISɕϊ
*/

void Kanji_ToSJIS(char *srcPtr, char *dstPtr) {
	int s1 = (srcPtr[0] & 0x7F);
	int s2 = (srcPtr[1] & 0x7F);

	dstPtr[0] = ((s1 - 1) >> 1);
	if (s1 < 0x5F)
		dstPtr[0] += 0x71;
	else
		dstPtr[0] += 0xB1;

	if ((s1 & 1) == 0)
		dstPtr[1] = s2 + 0x7E;
	else if (s2 < 0x60)
		dstPtr[1] = s2 + 0x1F;
	else
		dstPtr[1] = s2 + 0x20;
}

/*
	̓|C^EUCR[h𔻒肵āASJISɕϊB
	oCgoCgɕϊ邩̏ԂB
*/

int Kanji_EUCToSJISChar(char *srcPtr, char *dstPtr, int *srcBytes, int *dstBytes) {
	/* ANK */
	if (srcPtr[0] >= 0x20 && srcPtr[0] <= 0x7E) {
		dstPtr[0] = srcPtr[0];
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_ANK;
	}
	/* CTRL */
	else if (srcPtr[0] < 0x80) {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	/* ~Xqbg */
	else if (*srcBytes < 2) {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_MIS;
	}
	/* pJi */
	else if (srcPtr[0] == 0x8E) {
		dstPtr[0] = srcPtr[1];
		*dstBytes = 1;
		*srcBytes = 2;
		return CHAR_KIND_KANA;
	}
	/* Sp */
	else if (srcPtr[0] >= 0xA1 && srcPtr[0] <= 0xFE &&
	         srcPtr[1] >= 0xA1 && srcPtr[1] <= 0xFE) {
		Kanji_ToSJIS(srcPtr, dstPtr);
		*dstBytes = 2;
		*srcBytes = 2;
		return CHAR_KIND_KANJI;
	}
	/* ~Xqbg */
	else {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_MIS;
	}
}

int Kanji_JIS7ToSJISChar(char *srcPtr, char *dstPtr,int *srcBytes, int *dstBytes, int *status) {
	if (srcPtr[0] == 0x1B) {
		*status = JIS7_STATUS_ESC;
		*dstBytes = 0;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	else if (*status == JIS7_STATUS_ESC) {
		if (*srcBytes < 2) {
			*status = JIS7_STATUS_ASCII;
			*dstBytes = 0;
			*srcBytes = 0;
			return CHAR_KIND_MIS;
		}
		else {
			if ((srcPtr[0] == '$' && srcPtr[1] == '@') ||
			        (srcPtr[0] == '$' && srcPtr[1] == 'B')) {
				*status = JIS7_STATUS_KANJI;
				*dstBytes = 0;
				*srcBytes = 2;
				return CHAR_KIND_CTRL;
			}
			else if (srcPtr[0] == '(' && srcPtr[1] == 'B') {
				*status = JIS7_STATUS_ASCII;
				*dstBytes = 0;
				*srcBytes = 2;
				return CHAR_KIND_CTRL;
			}
			else if (srcPtr[0] == '(' && srcPtr[1] == 'I') {
				*status = JIS7_STATUS_0201K;
				*dstBytes = 0;
				*srcBytes = 2;
				return CHAR_KIND_CTRL;
			}
			else if (srcPtr[0] == '(' && srcPtr[1] == 'J') {
				*status = JIS7_STATUS_0201L;
				*dstBytes = 0;
				*srcBytes = 2;
				return CHAR_KIND_CTRL;
			}
			else {
				*status = JIS7_STATUS_ASCII;
				*dstBytes = 0;
				*srcBytes = 0;
				return CHAR_KIND_MIS;
			}
		}
	}
	else if (srcPtr[0] == 0x0F) {
		*status = JIS7_STATUS_ASCII;
		*dstBytes = 0;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	else if (srcPtr[0] == 0x0E) {
		*status = JIS7_STATUS_0201K;
		*dstBytes = 0;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	else if (*status == JIS7_STATUS_KANJI) {
		if (*srcBytes < 2 || srcPtr[1] >= 0x80) {
			*status = JIS7_STATUS_ASCII;
			*dstBytes = 0;
			*srcBytes = 0;
			return CHAR_KIND_MIS;
		}
		else {
			Kanji_ToSJIS(srcPtr, dstPtr);
			*dstBytes = 2;
			*srcBytes = 2;
			return CHAR_KIND_KANJI;
		}
	}
	else if (*status == JIS7_STATUS_0201K) {
		dstPtr[0] = (srcPtr[0] | 0x80);
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_KANA;
	}
	/* ANK */
	else if (srcPtr[0] >= 0x20 && srcPtr[0] <= 0x7E) {
		dstPtr[0] = srcPtr[0];
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_ANK;
	}
	/* CTRL */
	else if (srcPtr[0] < 0x80) {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	else {
		*status = JIS7_STATUS_ASCII;
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_MIS;
	}
}

int Kanji_SJISToSJISChar(char *srcPtr, char *dstPtr, int *srcBytes, int *dstBytes) {
	/* ANK */
	if (srcPtr[0] >= 0x20 && srcPtr[0] <= 0x7E) {
		dstPtr[0] = srcPtr[0];
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_ANK;
	}
	/* CTRL */
	else if (srcPtr[0] < 0x80) {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_CTRL;
	}
	/* pJi */
	else if (srcPtr[0] >= 0xA1 && srcPtr[0] <= 0xDF) {
		dstPtr[0] = srcPtr[0];
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_KANA;
	}
	/*  */
	else if ((srcPtr[0] >= 0x81 && srcPtr[0] <= 0x9F) || srcPtr[0] >= 0xE0) {
		dstPtr[0] = srcPtr[0];
		dstPtr[1] = srcPtr[1];
		*dstBytes = 2;
		*srcBytes = 2;
		return CHAR_KIND_KANJI;
	}
	/* ~Xqbg */
	else {
		dstPtr[0] = '.';
		*dstBytes = 1;
		*srcBytes = 1;
		return CHAR_KIND_MIS;
	}
}

int Kanji_ASCIIToASCIIChar(char *srcPtr, char *dstPtr, int *srcBytes, int *dstBytes) {
	dstPtr[0] = srcPtr[0];
	*dstBytes = 1;
	*srcBytes = 1;
	/* ASCII */
	if (srcPtr[0] >= 0x20 && srcPtr[0] <= 0x7E)
		return CHAR_KIND_ANK;
	else if (srcPtr[0] < 0x80) {
		if (srcPtr[0] == '\t')
			dstPtr[0] = ' ';
		else
			dstPtr[0] = '.';
		return CHAR_KIND_CTRL;
	}
	/* ~Xqbg */
	else
		return CHAR_KIND_MIS;
}

