/*
	TMenu.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.12	1.0			Takeshi Yoneki
*/

#include "csystem.h"

void TMenu::Initial() {
	m_mref = NULL;
	m_command = NULL;
}

TMenu::TMenu() {
	Initial();
}

TMenu::TMenu(short resourceID) {
	Initial();
	GetMenu(resourceID);
}

TMenu::TMenu(short menuID, ConstStr255Param menuTitle) {
	Initial();
	NewMenu(menuID, menuTitle);
}

TMenu::~TMenu() {
	if (NULL == m_mref)
		return;
	DeleteMenu();
	if (m_sw == CMENU_GET)
		::ReleaseResource((Handle)m_mref);
	else
		::DisposeMenu(m_mref);
}

void TMenu::SetRelateToCommand(TMenuCommandBase *command) {
	m_command = command;
	m_command->m_menu = this;
}

void TMenu::GetMenu(short resourceID) {
	m_mref = ::GetMenu(resourceID);
	HNoPurge((Handle)m_mref);
	m_id = resourceID;
	m_sw = CMENU_GET;
}

void TMenu::NewMenu(short menuID, ConstStr255Param menuTitle) {
	m_mref = ::NewMenu(menuID, menuTitle);
	m_id = menuID;
	m_sw = CMENU_NEW;
}

void TMenu::EnableMenuItemByList(char *newList, char *oldList) {
	int n = strlen(newList);
	for (int i = 0; i < n; i++) {
		if (newList[i] != oldList[i]) {
			if (newList[i] == 'E')
				EnableItem(i);
			else
				DisableItem(i);
		}
	}
}

TString TMenu::GetMenuItemText(short item) {
	Str255 itemString;
	::GetMenuItemText(m_mref, item, itemString);
	return itemString;
}

TCFString TMenu::GetMenuItemTextCFString(MenuItemIndex item) {
	CFStringRef r = NULL;
	::CopyMenuItemTextAsCFString(m_mref, item, &r);
	if (r == NULL)
		return "";
	return r;
}

/*
	TMenuBar
*/

TMenu *TMenuBar::GetByID(short id) {
	TMenu *menu;
	for (menu = GetTop(); menu != NULL; menu = menu->GetNext()) {
		if(menu->m_id == id)
			return menu;
	}
	return NULL;
}

TMenuCommandBase *TMenuBar::GetCommandByID(short id) {
	TMenu *menu = GetByID(id);
	if (menu == NULL)
		return NULL;
	return menu->m_command;
}

void TMenuBar::AppendResMenu(short id, ResType rt) {
	if (rt == 'DRVR')
		return;
	TMenu *menu = GetByID(id);
	if (menu != NULL)
		menu->AppendResMenu(rt);
}

/*
	TMenuCommandBase
*/

void TMenuCommandBase::FixMenu() {
}

void TMenuCommandBase::Command(short item) {
}

/*
	TKeyMap
*/

void TKeyMap::GetKeys() {
#ifdef __MWERKS__
	::GetKeys((long *)m_keymap);
#else
	BigEndianLong keymap[4];
	::GetKeys(keymap);
	memcpy(m_keymap, keymap, 16);
#endif
}

Boolean TKeyMap::IsPressed(unsigned short key) {
	return ((m_keymap[key >> 3] >> (key & 7) ) & 1);
}

/*
	TPopup
*/

TPopup::TPopup(short id) {
	m_menu = new TMenu(id);
	m_menu->InsertMenuHier();
	m_value = 1;
}

TPopup::~TPopup() {
	m_menu->DeleteMenu();
	delete m_menu;
}

Boolean TPopup::TrackPopup(TRect& box) {
	TPoint leftTopPt(box.left, box.top);
	leftTopPt.LocalToGlobal();
	long code = m_menu->PopUpMenuSelect(leftTopPt.v, leftTopPt.h, m_value);
	short item = LoWord(code);
	if (item == 0)
		return FALSE;
	m_value = item;
	return TRUE;
}
