/*
	TProcess.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.04.15	1.0			Takeshi Yoneki
*/

#include "csystem.h"

/*
	TAEDesc
*/

void TAEDesc::Initial() {
	descriptorType = typeNull;
	dataHandle = NULL;
	m_error = noErr;
	m_created = FALSE;
}

TAEDesc::TAEDesc() {
	Initial();
}

TAEDesc::TAEDesc(const AEDesc *desc) {
	Initial();
	*(AEDesc*)this = *desc;
}

TAEDesc::~TAEDesc() {
	if (m_created && descriptorType != typeNull)
		AEDisposeDesc();
	m_created = FALSE;
}

OSErr TAEDesc::AECreateDesc(DescType typeCode, const void *dataPtr, Size dataSize) {
	m_error = ::AECreateDesc(typeCode, dataPtr, dataSize, this);
	m_created = m_error == noErr;
	return m_error;
}

OSErr TAEDesc::CreateObjSpecifier(DescType desiredClass, AEDesc *theContainer, DescType keyForm, AEDesc *keyData, Boolean disposeInputs) {
	m_error = ::CreateObjSpecifier(desiredClass, theContainer, keyForm, keyData, disposeInputs, this);
	m_created = m_error == noErr;
	return m_error;
}

OSErr TAEDesc::AEResolve(short callbackFlags, TAEDesc& token) {
	m_error = ::AEResolve(this, callbackFlags, &token);
	token.m_created = IsGood();
	return m_error;
}

Size TAEDesc::GetLength() {
	return ::AEGetDescDataSize(this);
}

void TAEDesc::GetData(void *dataPtr, Size maximumSize) {
	::AEGetDescData(this, dataPtr, maximumSize);
}

long TAEDesc::GetInteger() {
	if (dataHandle == NULL)
		return 0;
	long x;
	GetData(&x, sizeof x);
	return x;
}

Boolean TAEDesc::GetBoolean() {
	if (dataHandle == NULL)
		return FALSE;
	Boolean x;
	GetData(&x, sizeof x);
	return x;
}

DescType TAEDesc::GetDescType() {
	if (dataHandle == NULL)
		return 'null';
	DescType x;
	GetData(&x, sizeof x);
	return x;
}

TString TAEDesc::GetString() {
	if (dataHandle == NULL)
		return "";
	Size leng = GetLength();
	if (leng == 0)
		return "";
	char *s = new char [leng + 1];
	memset(s, 0, leng + 1);
	GetData(s, leng);
	TString cs;
	cs.CopyFrom(s, leng);
	return cs;
}

TFSSpec TAEDesc::GetFSSpec() {
//	TFSSpec spec;
	FSSpec x;
	GetData(&x, sizeof x);
	return x;
}

TFSRef TAEDesc::GetFSRef() {
	FSRef x;
	GetData(&x, sizeof x);
	return x;
}

/*
	TAEDescList
*/

TAEDescList::TAEDescList() {
}

TAEDescList::TAEDescList(const AEDescList *descList) {
	TAEDesc::Initial();
	*(AEDesc*)this = *descList;
}

OSErr TAEDescList::AECreateList(const void *factoringPtr, Size factoredSize, Boolean isRecord) {
	m_error = ::AECreateList(factoringPtr, factoredSize, isRecord, this);
	m_created = m_error == noErr;
	return m_error;
}

/*
	TAppleEvent
*/

TAppleEvent::TAppleEvent() {
}

TAppleEvent::TAppleEvent(const AppleEvent *aev)
		: TAEDesc(aev) {
}

OSErr TAppleEvent::AECreateAppleEvent(AEEventClass theAEEventClass, AEEventID theAEEventID, const AEAddressDesc *target, short returnID, long transactionID) {
	m_error = ::AECreateAppleEvent(theAEEventClass, theAEEventID, target, returnID, transactionID, this);
	m_created = m_error == noErr;
	return m_error;
}

OSErr TAppleEvent::AEGetParamDesc(AEKeyword theAEKeyword, DescType desiredType, TAEDesc& result) {
	m_error = ::AEGetParamDesc(this, theAEKeyword, desiredType, result);
	result.m_created = IsGood();
	return m_error;
}

OSErr TAppleEvent::AESend(TAppleEvent& reply, AESendMode sendMode, AESendPriority sendPriority, long timeOutInTicks, AEIdleUPP idleProc, AEFilterUPP filterProc) {
	m_error = ::AESend(this, reply, sendMode, sendPriority, timeOutInTicks, idleProc, filterProc);
	reply.m_created = IsGood();
	return m_error;
}

TString TAppleEvent::GetParamString(AEKeyword theAEKeyword, DescType desiredType) {
	TString s;
	TAEDesc strData;
	AEGetParamDesc(theAEKeyword, desiredType, strData);
	if (IsGood())
		s = strData.GetString();
	return s;
}

long TAppleEvent::GetParamInteger(AEKeyword theAEKeyword, DescType desiredType) {
	long n = 0;
	TAEDesc longData;
	AEGetParamDesc(theAEKeyword, desiredType, longData);
	if (IsGood())
		n = longData.GetInteger();
	return n;
}

Boolean TAppleEvent::GetParamBoolean(AEKeyword theAEKeyword, DescType desiredType) {
	Boolean f = 0;
	TAEDesc boolData;
	AEGetParamDesc(theAEKeyword, desiredType, boolData);
	if (IsGood())
		f = boolData.GetBoolean();
	return f;
}

TFSSpec TAppleEvent::GetParamFSSpec(AEKeyword theAEKeyword, DescType desiredType) {
	TFSSpec spec;
	TAEDesc specData;
	AEGetParamDesc(theAEKeyword, desiredType, specData);
	if (IsGood() == FALSE) {
		TString path = GetParamString(theAEKeyword, typeChar);
		if (IsGood())
			spec.FSMakeFSSpec(0, 0, path);
	}
	else
		spec = specData.GetFSSpec();
	return spec;
}

DescType TAppleEvent::GetParamDescType(AEKeyword theAEKeyword, DescType desiredType) {
	DescType t = typeNull;
	TAEDesc typeData;
	AEGetParamDesc(theAEKeyword, desiredType, typeData);
	if (IsGood())
		t = typeData.GetDescType();
	return t;
}

TAEResult::TAEResult() {
	m_keyword = typeNull;
}

OSErr TAEResult::PutDataWithKeyword(AEKeyword theAEKeyword, DescType typeCode, const void *dataPtr, Size dataSize) {
	m_keyword = theAEKeyword;
	return m_error = AECreateDesc(typeCode, dataPtr, dataSize);
}

OSErr TAEResult::PutDescWithKeyword(AEKeyword theAEKeyword, const AEDesc *theAEDesc) {
	m_keyword = theAEKeyword;
	return m_error = AEDuplicateDesc(theAEDesc);
}

/*
	TProcess
*/

void TProcess::Initial() {
	highLongOfPSN = 0;
	lowLongOfPSN = kNoProcess;
	m_error = noErr;
}

TProcess::TProcess() {
	Initial();
}

TProcess::TProcess(ProcessSerialNumber& psn) {
	Initial();
	*this = psn;
}

Boolean TProcess::IsApplHere(OSType creator) {
	Initial();
	Str63 name;
	FSSpec spec;
	m_pInfo.processInfoLength = sizeof (ProcessInfoRec);
	m_pInfo.processName = name;
	m_pInfo.processAppSpec = &spec;
	for (;;) {
		if (GetNextProcess() != noErr)
			break;
		if (GetProcessInformation() != noErr)
			break;
		if (m_pInfo.processSignature == creator)
			return TRUE;
	}
	m_error = -1;
	return FALSE;
}

OSErr TProcess::LaunchApplication(const FSSpec *spec, AppParameters *ap, int flag) {
	LaunchParamBlockRec lpb;
	memset(&lpb, 0, sizeof lpb);
	lpb.launchBlockID = extendedBlock;
	lpb.launchEPBLength = extendedBlockLen;
	lpb.launchFileFlags = 0;
	lpb.launchControlFlags = flag;
	lpb.launchAppSpec = (FSSpec *)spec;
	lpb.launchAppParameters = ap;
	m_error = ::LaunchApplication(&lpb);
	*this = lpb.launchProcessSN;
	return m_error;
}

Boolean TProcess::SendOpenDocument(OSType creator, TFilePath& filePath, long selStart, long selEnd) {
	if (theSystem->systemVersion < 0x0700)
		return FALSE;

	TLocalPingEvent ping;
	if (IsApplHere(creator) == FALSE) {
		TFSSpec spec;
		if (spec.FindApplication(creator) == FALSE)
			return FALSE;
		LaunchApplication(spec, NULL/*ping*/);
		if (IsGood() == FALSE)
			return FALSE;
	}
	else {
		SetFrontProcess();
	}

	TAEDesc target;
	target.AECreateDesc('sign', &creator, sizeof creator);
	if (target.IsGood() == FALSE)
		return FALSE;

	TFSSpec spec(filePath);
	if (spec.IsGood() == FALSE)
		return FALSE;

	TAEDescList fileList;
	fileList.AECreateList();
	if (fileList.IsGood() == FALSE)
		return FALSE;
	fileList.AEPutPtr(0, 'fss ', (const FSSpec *)spec, sizeof (FSSpec));
	if (fileList.IsGood() == FALSE)
		return FALSE;

	TAppleEvent ae;
	ae.AECreateAppleEvent('aevt', 'odoc', target);
	if (ae.IsGood() == FALSE)
		return FALSE;

	ae.AEPutParamDesc(keyDirectObject, fileList);
	if (ae.IsGood() == FALSE)
		return FALSE;

	if (selStart != -1 && theSystem->IsFriend(creator)) {
		ae.AEPutParamPtr('selS', 'long', &selStart, sizeof selStart);
		ae.AEPutParamPtr('selE', 'long', &selEnd, sizeof selEnd);
	}

	TAppleEvent aedmy;
	long startTick = TickCount();
	CEvent ev;
	OSErr err;
	for (; TickCount() - startTick < 600;) {
		err = ae.AESend(aedmy, kAENoReply);
		if (err != -600)
			break;
		ev.WaitNextEvent(0, 30);
	}

	return err == noErr;
}

/*
	CNotification
*/

CNotification::CNotification() {
	Init();
}

CNotification::CNotification(TString s) {
	Init();
	SetText(s);
}

CNotification::~CNotification() {
	Remove();
}

void CNotification::Init() {
	m_bInstalled = FALSE;
	memset(&m_nm, 0, sizeof (m_nm));
	m_nm.qType = nmType;
	m_nm.nmMark = 1;
}

void CNotification::SetText(TString s) {
	m_text = s;
	m_nm.nmStr = (StringPtr)(ConstStr255Param)m_text;
}

OSErr CNotification::Install() {
	m_error = ::NMInstall(&m_nm);
	if (m_error == noErr)
		m_bInstalled = TRUE;
	return m_error;
}

void CNotification::Remove() {
	if (m_bInstalled)
		::NMRemove(&m_nm);
	m_bInstalled = FALSE;
}

/*
	Ping
*/

TLocalPingEvent::TLocalPingEvent() {
	memset((AppParameters *)this, 0, sizeof (AppParameters));
	theMsgEvent.what = kHighLevelEvent;
	theMsgEvent.message = 'HunX';
	theMsgEvent.when = TickCount();
	*(OSType *)(&theMsgEvent.where) = 'ping';
	theMsgEvent.modifiers = 0;
	eventRefCon = 0;
	messageLength = 0;
}

OSErr TLocalPingEvent::Post(OSType target) {
	m_error = noErr;
	return IsGood();
}
