/*
	TString.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.03	1.0			Takeshi Yoneki
*/

#include "csystem.h"
#include "ckanji.h"

static Str15 safetyEmptyString = "\p";
static char sprintBufferInstance[1024];
int sprintfBufferSize = 1024;
static char *sprintfBuffer = sprintBufferInstance;
static UniChar safetyEmptyUniString[16] = { 0, 0 };

void TString::Init() {
	m_dataLength = 0;
	m_allocLength = 0;
	m_data = (char *)safetyEmptyString;
	m_scriptCode = ::IntlScript();
}

void TString::SafeDelete() {
	if (m_data != (char *)safetyEmptyString && m_data != NULL) {
		delete [] m_data;
	}
}

void TString::AllocCopy(const char *p, int len) {
	if (m_allocLength < len || len == 0 || p == NULL) {
		SafeDelete();
		Init();
	}
	if (len > 0) {
		if (m_data == (char *)safetyEmptyString || m_data == NULL) {
			m_data = new char[len + 2];
			m_allocLength = len;
		}
		memcpy(&m_data[1], p, len);
		m_data[len + 1] = '\0';
		m_dataLength = len;
		m_data[0] = m_dataLength < 256 ? m_dataLength : 255;
	}
}

TString::TString() {
	Init();
}

TString::TString(char *s) {
	Init();
	*this = s;
}

TString::TString(const char *s) {
	Init();
	*this = s;
}

TString::TString(ConstStr255Param ps) {
	Init();
	*this = ps;
}

TString::TString(const TString& cs) {
	Init();
	*this = cs;
}

/*
TString::TString(OSType sig)
{
	Init();
	*this = sig;
}
*/

TString::TString(const TCFString &sr) {
	Init();
	*this = sr;
}

TString::TString(const TUniString &us) {
	Init();
	*this = us;
}

/*
TString::TString(const CUniString& us)
{
	Init();
	*this = us;
}
*/

TString::~TString() {
	SafeDelete();
}

TString::operator char *() const {
	return &m_data[1];
}

TString::operator ConstStr255Param() const {
	return (ConstStr255Param)m_data;
}

/*
TString::operator StringPtr() const
{
	return (StringPtr)m_data;
}
*/

/*
TString::operator OSType() const
{
	OSType sig = 0;
	int shift = 24;
	for (int i = 0; i < 4; i++)
	{
		unsigned long x = (GetAt(i) & 0x000000FF);
		x = (x == 0 ? 0x20 : x);
		x = (x << shift);
		sig |= x;
		shift -= 8;
	}
	return sig;
}
*/

TString::operator CFStringRef() const {
	return ::CFStringCreateWithCString(NULL, *this, m_scriptCode);
}

void TString::operator=(const TString& cs) {
	AllocCopy(cs, cs.GetLength());
	m_scriptCode = cs.m_scriptCode;
}

void TString::operator=(char* s) {
	if (s == NULL)
		Init();
	else
		AllocCopy(s, strlen(s));
}

void TString::operator=(const char* s) {
	if (s == NULL)
		Init();
	else
		AllocCopy(s, strlen(s));
}

void TString::operator=(ConstStr255Param ps) {
	if (ps == NULL)
		Init();
	else
		AllocCopy((char *)&ps[1], StrLength(ps));
}

void TString::operator=(OSType sig) {
	char bf[5];
	int shift = 24;
	for (int i = 0; i < 4; i++) {
		bf[i] = ((sig >> shift) & 0x000000FF);
		shift -= 8;
	}
	bf[4] = '\0';
	*this = bf;
}

void TString::operator=(CFStringRef s) {
	Boolean result = CFStringGetCString(s, sprintBufferInstance, sizeof(sprintBufferInstance), m_scriptCode);
	if (result)
		*this = sprintBufferInstance;
}

/*
void TString::operator=(const CUniString& us)
{
	if (theSystem->hasTextEncodingConverter == FALSE)
		return;
	// K؂UnicodeϊȂƁuvȂǂ2B
	// TUnicodeTextEncodingInfo::UnicodeToTextł1̂Ŗ肪B
	// TString͒̌EႢ̂ňꊇϊ\B
	int sz = us.GetLength() * 2 + 16;
	char *buffer = new char [sz];
	memset(buffer, 0, sz);
	ByteCount oOutputLen = 0;
	OSStatus result = theSystem->m_encInfo.ConvertFromUnicodeToText(us.GetLength() * sizeof(UniChar), (UniChar *)us, sz, &oOutputLen, buffer);
	if (result == noErr)
		*this = buffer;
	delete [] buffer;
}
*/

void TString::operator=(const TCFString &sr) {
	sr.CopyToCSTR(sprintfBuffer, sprintfBufferSize);
	//CFStringGetCString(sr, sprintfBuffer, (CFIndex)sprintfBufferSize, m_scriptCode);
	AllocCopy(sprintfBuffer, strlen(sprintfBuffer));
	m_scriptCode = sr.GetScriptCode();
}

void TString::operator=(const TUniString &us) {
	TCFString cs = us;
	*this = cs;
}

OSType TString::GetOSType() {
	OSType sig = 0;
	int shift = 24;
	for (int i = 0; i < 4; i++) {
		unsigned long x = (GetAt(i) & 0x000000FF);
		x = (x == 0 ? 0x20 : x);
		x = (x << shift);
		sig |= x;
		shift -= 8;
	}
	return sig;
}

void TString::SetOSType(OSType sig) {
	char bf[5];
	int shift = 24;
	for (int i = 0; i < 4; i++) {
		bf[i] = ((sig >> shift) & 0x000000FF);
		shift -= 8;
	}
	bf[4] = '\0';
	AllocCopy(bf, strlen(bf));
}

void TString::CopyTo(StringPtr ps, int n) {
	if (ps == NULL)
		return;
	if (m_data == (char *)safetyEmptyString) {
		ps[0] = 0;
	}
	else if (n < 0 || m_dataLength < n) {
		memcpy(ps, m_data, m_dataLength + 1);
	}
	else {
		memcpy(ps, m_data, n);
		ps[0] = n - 1;
	}
}

void TString::CopyTo(char *s, int n) {
	if (s == NULL)
		return;
	if (m_data == (char *)safetyEmptyString)
		s[0] = '\0';
	else if (n < 0 || m_dataLength + 1 <= n)
		memcpy(s, &m_data[1], m_dataLength + 1);
	else
		memcpy(s, &m_data[1], n);
}

/*
CFStringRef TString::GetCFString(CFStringEncoding enc)
{
	return ::CFStringCreateWithCString(NULL, *this, enc);
}
*/

void TString::CopyFrom(char *s, int n) {
	if (s == NULL || n < 0) {
		*this = s;
	}
	else {
		int count = n;
		for (int i = 0; i < n; i++) {
			if (s[i] == 0) {
				count = i;
			}
		}
// 2006.02.10 Shift-JISÕR[h͔p~
/*
		int slen = strlen(s);
		int count = 0;
		for (; count < slen && count < n && s[count] != 0;) {
			// System 6łCharacterByteTypeCharBytegƂɂȂA
			// JgOt|[gIɗLӂłȂĂ͂ȂB
			// 2004.09.13
			int charByte = 1;
			if (theSystem->systemVersion >= 0x0700)
				charByte = (CharacterByteType(&s[count], 0, m_scriptCode) == smFirstByte ? 2 : 1);
			else
				charByte = theSystem->m_charByteData[s[count] & 0xFF];
			int nextCount = charByte + count;
			if (nextCount > n || nextCount > slen)
				break;
			count += charByte;
		}
*/
		AllocCopy(s, count);
	}
}

/*
void TString::SetCFString(CFStringRef s, CFStringEncoding enc)
{
	Boolean result = CFStringGetTString(s, sprintBufferInstance, sizeof(sprintBufferInstance), enc);
	if (result)
		*this = sprintBufferInstance;
}
*/

char TString::GetAt(int n) const {
	return n >= m_dataLength ? '\0' : m_data[n + 1];
}

char *TString_GetBuffer(int sz) {
	if (sz > sprintfBufferSize) {
		if (sprintfBuffer != sprintBufferInstance)
			delete [] sprintfBuffer;
		sprintfBuffer = new char[sz];
		sprintfBufferSize = sz;
	}
	return sprintfBuffer;
}

/*
void TString_SetBuffer(char *buf, int sz) {
	if (buf != NULL) {
		sprintfBuffer = buf;
		sprintfBufferSize = sz;
	}
}
*/

int TString::sprintf(const char *format, ...) {
	va_list	args;
	va_start(args, format);
	int ret = vsprintf(sprintfBuffer, format, args);
	*this = sprintfBuffer;
	va_end(args);
	return ret;
}

char *TString::strtok_first(const char *delim) {
	char *p = m_data == (char *)safetyEmptyString ? NULL : &m_data[1];
	return ::strtok(p, delim);
}

char *TString::strtok_next(const char *delim) {
	return ::strtok(NULL, delim);
}

int TString::strcmp(const char *s) {
	const char *p = m_data == (char *)safetyEmptyString ? "" : &m_data[1];
	s = s == NULL ? "" : s;
	return ::strcmp(p, s);
}

int TString::strncmp(const char *s, int n) {
	char *p = m_data == (char *)safetyEmptyString ? (char *)"" : &m_data[1];
	return ::strncmp(p, s, n);
}

int TString::stricmp(const char *s) {
	TString t = (char *)s;
	TString u = *this;
	t.MakeUpper();
	u.MakeUpper();
	return u.strcmp(t);
}

int TString::strnicmp(const char *s, int n) {
	TString t = (char *)s;
	TString u = *this;
	t.MakeUpper();
	u.MakeUpper();
	return u.strncmp(t, n);
}

Boolean TString::Equal(const char *s, int n) {
	const char *p = m_dataLength == 0 ? "" : &m_data[1];
	s = s == NULL ? "" : s;
	for (int i = 0;; i++) {
		if (n >= 0 && i >= n)
			break;
		if (s[i] == '\0' && p[i] == '\0')
			break;
		if (s[i] == '\0' || p[i] == '\0' || s[i] != p[i])
			return FALSE;
	}
	return TRUE;
}

Boolean TString::operator==(const TString& cs) {
	const char *q = cs.m_data == (char *)safetyEmptyString ? "" : &cs.m_data[1];
	return Equal(q);
}

Boolean TString::operator==(char *s) {
	return Equal(s);
}

Boolean TString::operator==(ConstStr255Param ps) {
	TString cs = ps;
	const char *q = cs.m_data == (char *)safetyEmptyString ? "" : &cs.m_data[1];
	return Equal(q);
}

Boolean TString::IsASCII() {
	char *p = *this;
	for (int i = 0; i < m_dataLength; i++) {
		if (p[i] & 0x80)
			return FALSE;
	}
	return TRUE;
}

TString TString::GetForMenu() {
	TString s;
	char *p = (char *)*this;
	s.sprintf("%s%s", p[0] == '-' ? "\r" : "", p);
	return s;
}

void TString::UppercaseText(TWindow * win) {
	TSavePort savePort(win);
	::UppercaseText(*this, GetLength(), FontScript());
}

void TString::MakeUpper(TWindow * win) {
	if (theSystem->systemVersion < 0x0700)
		UpperString();
	else
		UppercaseText(win);
}

int TString::atoi() {
	return ::atoi(&m_data[1]);
}

void TString::CutRearSpace() {
	if (IsEmpty())
		return;
	int n = GetLength();
	int length = 0;
	char *p = *this;
	for (int i = 0; i < n;) {
		int sz = theSystem->GetCharSize(p[i]);
		if (sz == 2 && i + 2 <= n) {
			if (!(p[i] == 0x81 && p[i + 1] == 0x40))/* {̃Xy[XȊO */
				length = i + sz;
		}
		else {
			if (p[i] != 0x20)
				length = i + sz;
		}
		i += sz;
	}
	CopyFrom(p, length);
}

void TString::StripSpace() {
	CutRearSpace();
	if (IsEmpty())
		return;
	char *p = *this;
	int i;
	int n = GetLength();
	for (i = 0; i < n;) {
		int sz = theSystem->GetCharSize(p[i]);
		if (sz == 2) {
			if (!(p[i] == 0x81 && p[i + 1] == 0x40))/* {̃Xy[XȊO */
				break;
		}
		else {
			if (p[i] != 0x20)
				break;
		}
		i += sz;
	}
	TString s = &p[i];
	*this = s;
}

void TString::LoadSTRNum(short id, short n) {
	TResource res('STR#', id);
	TLockHandle lockHandle(res.m_h);
	STRNumRec *rp = (STRNumRec *)*res.m_h;
	if (rp->m_number < n)
		return;
	StringPtr p = rp->m_string;
	for (int i = 1;; i++) {
		if (i == n) {
			*this = p;
			return;
		}
		p = p + p[0] + 1;
	}
}

Boolean TString::CutShortcut(TString *body, TString *shortcut) {
	*body = *this;
	*shortcut = "";
	int n = GetLength();
	if (n >= 80)
		return FALSE;
	if (n > 2 && GetAt(n - 2) == '/') {
		char *p = *this;
		body->CopyFrom(p, n - 2);
		shortcut->CopyFrom(&p[n - 2], 2);
		return TRUE;
	}
	return FALSE;
}

TString TString::ConvertToSJIS(int kanjiCode, TUnicodeTextEncodingInfo *encInfo) {
	if (kanjiCode == KANJI_UTF8) {
		Boolean bCreate = FALSE;
		if (encInfo == NULL) {
			encInfo = new TUnicodeTextEncodingInfo(m_scriptCode);
			bCreate = TRUE;
		}
		TString s = encInfo->UTF8ToText(*this);
		if (bCreate)
			delete encInfo;
		return s;
	}

	char *srcText = (char *)*this;
	char *buffer = TString_GetBuffer(GetLength() * 2);
	char *dstText = buffer;
	int rest = GetLength();

	int status = JIS7_STATUS_ASCII;

	for (; rest > 0;) {
		int charKind;
		int srcBytes = rest;
		int dstBytes = 0;

		for (;;) {
			if (kanjiCode == KANJI_EUC)
				charKind = Kanji_EUCToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
			else if (kanjiCode == KANJI_JIS7)
				charKind = Kanji_JIS7ToSJISChar(srcText, dstText, &srcBytes, &dstBytes, &status);
			//			else if (kanjiCode == KANJI_SJIS)
			//				charKind = Kanji_SJISToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
			else
				charKind = Kanji_ASCIIToASCIIChar(srcText, dstText, &srcBytes, &dstBytes);
			if (srcBytes != 0)
				break;
		}

		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	*dstText = '\0';

	return (TString)buffer;
}

Boolean TString::CheckConvertToSJIS(int kanjiCode, int *misHit, int *kanjiHit, TUnicodeTextEncodingInfo *encInfo) {
	if (IsEmpty())
		return TRUE;

	if (kanjiCode == KANJI_UTF8) {
		if (theSystem->hasTextEncodingConverter == FALSE)
			return FALSE;

		Boolean bCreate = FALSE;
		if (encInfo == NULL) {
			encInfo = new TUnicodeTextEncodingInfo(m_scriptCode);
			bCreate = TRUE;
		}
		TString s = encInfo->UTF8ToText(*this);
		if (bCreate)
			delete encInfo;
		// Ă璷ς\
		if (GetLength() != s.GetLength() && kanjiHit != NULL)
			(*kanjiHit)++;
		// Ro[gs\͋Ԃ̂
		if (s.IsEmpty() && misHit != NULL)
			(*misHit)++;
		return (s.IsEmpty() == FALSE);
	}

	char *srcText = (char *)*this;
	char *buffer = TString_GetBuffer(GetLength() * 2);
	char *dstText = buffer;
	int rest = GetLength();

	int status = JIS7_STATUS_ASCII;
	Boolean result = TRUE;

	for (; rest > 0;) {
		int charKind;
		int srcBytes = rest;
		int dstBytes = 0;

		for (;;) {
			if (kanjiCode == KANJI_EUC)
				charKind = Kanji_EUCToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
			else if (kanjiCode == KANJI_JIS7)
				charKind = Kanji_JIS7ToSJISChar(srcText, dstText, &srcBytes, &dstBytes, &status);
			else if (kanjiCode == KANJI_SJIS)
				charKind = Kanji_SJISToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
			else
				charKind = Kanji_ASCIIToASCIIChar(srcText, dstText, &srcBytes, &dstBytes);

			if (charKind == CHAR_KIND_MIS && misHit != NULL) {
				(*misHit)++;
				result = FALSE;
			}
			if (charKind == CHAR_KIND_KANJI && kanjiHit != NULL)
				(*kanjiHit)++;

			if (srcBytes != 0)
				break;
		}

		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	*dstText = '\0';

	return result;
}

TString TString::MakeJIS7() {
	char *srcText = (char *)*this;
	char *buffer = TString_GetBuffer(GetLength() * 2);
	char *dstText = buffer;
	int rest = GetLength();
	for (; rest > 0;) {
		int srcBytes = rest;
		int dstBytes = 0;
		int charKind = Kanji_SJISToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
		if (charKind == CHAR_KIND_KANJI)
			Kanji_ToJIS7(srcText, dstText);
		else if (charKind == CHAR_KIND_KANA)
			*dstText = *srcText - 0x80;
		else
			*dstText = *srcText;
		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	*dstText = '\0';

	return (TString)buffer;
}

TString TString::MakeEUC() {
	char *srcText = (char *)*this;
	char *buffer = TString_GetBuffer(GetLength() * 2);
	char *dstText = buffer;
	int rest = GetLength();
	for (; rest > 0;) {
		int srcBytes = rest;
		int dstBytes = 0;
		int charKind = Kanji_SJISToSJISChar(srcText, dstText, &srcBytes, &dstBytes);
		if (charKind == CHAR_KIND_KANJI)
			Kanji_ToEUC(srcText, dstText);
		else if (charKind == CHAR_KIND_KANA) {
			*dstText = 0x8E;
			dstText++;
			*dstText = *srcText;
		}
		else
			*dstText = *srcText;
		srcText += srcBytes;
		dstText += dstBytes;
		rest -= srcBytes;
	}
	*dstText = '\0';

	return (TString)buffer;
}

// for only Japanese
TString TString::MakeUTF8() {
	TUnicodeTextEncodingInfo encInfo(m_scriptCode);
	return encInfo.TextToUTF8(*this);
}

OSErr TString::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		char *p = *this;
		result->PutDataWithKeyword(keyAEResult, typeChar, (Ptr)p, GetLength());
		return noErr;
	}
	else if (eventID == kAESetData) {
		*this = ae.GetParamString();
		return noErr;
	}
	else if (eventID == kAEDelete) {
		return noErr;
	}
	return errAEEventNotHandled;
}

Boolean TString::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	if (obj->GetObjectType() != classString)
		return FALSE;
	TString s = *(TString *)obj;
	int r = strcmp(s);
	if (oper == kAEGreaterThan)
		return r > 0;
	else if (oper == kAEGreaterThanEquals)
		return r >= 0;
	else if (oper == kAEEquals)
		return r == 0;
	else if (oper == kAELessThan)
		return r < 0;
	else if (oper == kAELessThanEquals)
		return r <= 0;
	else
		return FALSE;
}

/*
	TStringList
*/

void TStringList::Init() {
	m_scriptCode = ::IntlScript();
}

TStringList::TStringList() {
	Init();
}

TStringList::TStringList(TFilePath &filePath) {
	Init();
	LoadFile(filePath);
}

TStringList::~TStringList() {
	RemoveDeleteAll();
}

void TStringList::AppendStr(char *p) {
	TString *str = new TString(p);
	str->SetScriptCode(GetScriptCode());
	Append(str);
}

void TStringList::LoadFile(TFilePath &filePath) {
	TFileIO file(filePath);
	if (file.IsGood() == FALSE)
		return;
	long count = file.GetFileSize();
	TMemory buffer(count + 16);
	file.Read(count, (char *)buffer);
	char *p = (char *)buffer;
	int n = 0;
	char eol = 0;
	for (int i = 0; i < count; i++) {
		char c = buffer.GetAt(i);
		if ((c == '\r' || c == '\n') && eol == 0)
			eol = c;
		if (c == '\r' || c == '\n') {
			buffer.SetAt(i, 0);
			if (c == eol) {
				TString *str = new TString(p + n);
				str->SetScriptCode(GetScriptCode());
				Append(str);
			}
			n = i + 1;
		}
	}
	if (n < count) {
		TString *str = new TString(p + n);
		str->SetScriptCode(GetScriptCode());
		Append(str);
	}
}

void TStringList::SaveFile(TFilePath &filePath, OSType creator, OSType fileType) {
	TFileIO file(filePath, creator, fileType);
	if (file.IsGood() == FALSE)
		return;
	file.BeEmpty();
	int count = GetCount();
	TString s;
	for (int i = 0; i < count; i++) {
		s = *GetNth(i);
		file.Write(s);
		s = "\r";
		file.Write(s);
	}
}

void TStringList::LoadStrNumAll(short id) {
	for (int i = 1;; i++) {
		TString *s = new TString;
		s->LoadSTRNum(id, i);
		if (s->IsEmpty()) {
			delete s;
			break;
		}
		s->SetScriptCode(GetScriptCode());
		Append(s);
	}
}

/*
	GfB^̃eLXgf[^𕶎񃊃Xgɂ
	NULLĂ炻܂ŁAƂŖȂł傤
*/

#ifdef CONFIG_USE_CEDITOR

void TStringList::CopyFromEditor(TEditor *ed) {
	m_container = ed;
	CharsHandle h = ed->TEGetText();
	TLockHandle lockHandle(h);
	CopyFrom(*h, ed->GetTextLength());
}

void TStringList::CopyToEditor(TEditor *ed) {
	m_container = ed;
	TMemory buffer;
	int length = CopyTo(&buffer);
	ed->SetText((char *)buffer, length);
}

#endif

void TStringList::CopyFrom(TMemory *buffer) {
	RemoveDeleteAll();
	char *bf = *buffer;
	char *linep = bf;
	for (int i = 0;; i++) {
		Boolean eof = (bf[i] == '\0');
		if (eof && linep == &bf[i])
			break;
		if (bf[i] == '\r' || bf[i] == '\0') {
			bf[i] = '\0';
			TString *line = new TString(linep);
			line->SetScriptCode(GetScriptCode());
			line->m_container = this;
			Append(line);
			linep = &bf[i + 1];
		}
		if (eof)
			break;
	}
}

void TStringList::CopyFrom(char *data, int length) {
	RemoveDeleteAll();
	TMemory buffer(length + 16);
	char *bf = buffer;
	memcpy(bf, data, length);
	CopyFrom(&buffer);
}

int TStringList::CopyTo(TMemory *buffer) {
	if (buffer == NULL)
		return 0;
	int n = GetCount();
	int length = 0;
	TString *s;
	for (s = GetTop(); s != NULL; s = s->GetNext())
		length += (s->GetLength() + 1);
	buffer->ChangeLength(length + 16);
	char *linep = *buffer;
	for (s = GetTop(); s != NULL; s = s->GetNext()) {
		s->CopyTo(linep);
		linep[s->GetLength()] = '\r';
		linep = &linep[s->GetLength() + 1];
	}
	return length;
}

/*
	L[[hŎn܂s̎c̕𓾂
*/
TString TStringList::GetStringWithKeyword(TString keyword) {
	for (TString *s = GetTop(); s != NULL; s = s->GetNext()) {
		TString line = *s;
		if (line.strnicmp(keyword, keyword.GetLength()) == 0) {
			char *p = line;
			for (int i = keyword.GetLength(); i < line.GetLength(); i++) {
				if (p[i] != 0x20)
					return &p[i];
			}
		}
	}
	return "";
}

OSErr TStringList::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		TMemory buffer;
		int length = CopyTo(&buffer);
		result->PutDataWithKeyword(keyAEResult, typeChar, (char *)buffer, length);
		return noErr;
	}
	else if (eventID == kAESetData) {
		TAEDesc textData;
		ae.AEGetParamDesc(keyAEData, typeChar, textData);
		if (ae.IsGood() == FALSE)
			return errAEParamMissed;
		Size length = textData.GetLength();
		TMemory buffer(length + 1);
		textData.GetData((char *)buffer, length);
		CopyFrom(&buffer);
		//		CopyFrom((char *)textData.HLock(), textData.GetLength());
		//		textData.HUnlock();
		return noErr;
	}
	return errAEEventNotHandled;
}

OSErr TStringList::OnAdhocForContainer(TToken *token, AEEventID eventID, const AppleEvent *aev, TAEResult *result) {
	if (m_container != NULL)
		return m_container->OnAdhocForContainer(token, eventID, aev, result);
	return noErr;
}

////////////////////////////////////

void TUniString::Init() {
	m_dataLength = 0;
	m_allocLength = 0;
	m_data = (UniChar *)safetyEmptyUniString;
	m_scriptCode = ::IntlScript();
}

void TUniString::SafeDelete() {
	if (m_data != safetyEmptyUniString && m_data != NULL) {
		delete [] m_data;
	}
}

void TUniString::AllocCopy(const UniChar *p, int len) {
	if (len < 0) {
		len = unistrlen(p);
	}
	if (m_allocLength < len || len == 0 || p == NULL) {
		SafeDelete();
		Init();
	}
	int bom = BOM_BIG;
	if (*p == BOM_BIG || *p == BOM_LITTLE) {
		bom = *p;
		p++;
		len--;
	}
	if (len > 0) {
		if (m_data == safetyEmptyUniString || m_data == NULL) {
			m_data = new UniChar[len + 1];
			m_allocLength = len;
		}
		m_dataLength = len;
		for (int i = 0; i <len ; i++) {
			if (bom == BOM_BIG)
				m_data[i] = p[i];
			else
				m_data[i] = SwapInt16(p[i]);
		}
//		memcpy(m_data, p, len * sizeof(UniChar));
		m_data[m_dataLength] = 0;
	}
}

TUniString::TUniString() {
	Init();
}

TUniString::~TUniString() {
	SafeDelete();
}

/* Rs[RXgN^ */

TUniString::TUniString(const TUniString& us) {
	Init();
	*this = us;
}

TUniString::TUniString(const TCFString& cs) {
	Init();
	*this = cs;
}

TUniString::TUniString(const char *cstr) {
	Init();
	*this = cstr;
}

TUniString::TUniString(const TString &s) {
	Init();
	*this = s;
}

TUniString::TUniString(ConstStr255Param pStr) {
	Init();
	*this = pStr;
}

TUniString::TUniString(const CFStringRef theString) {
	Init();
	*this = theString;
}

TUniString::TUniString(const CFURLRef anURL) {
	Init();
	*this = anURL;
}

TUniString::TUniString(const HFSUniStr255 &us) {
	Init();
	*this = us;
}

TUniString::TUniString(const THFSUniStr255 &us) {
	Init();
	*this = us;
}

TUniString::TUniString(const UniChar *ustr) {
	Init();
	*this = ustr;
}

/* Rs[ */

void TUniString::operator=(const TUniString& us) {
	CopyFromUniString(us);
}

void TUniString::operator=(const TCFString& cs) {
	CopyFromTCFString(cs);
}

void TUniString::operator=(const char *cstr) {
	CopyFromCSTR(cstr, -1, -1);
}

void TUniString::operator=(const TString &s) {
	CopyFromCSTR((const char *)s, -1, s.GetScriptCode());
}

void TUniString::operator=(ConstStr255Param pStr) {
	CopyFromPSTR(pStr);
}

void TUniString::operator=(const CFStringRef theString) {
	TCFString cs = theString;
	CopyFromTCFString(cs);
}

void TUniString::operator=(const CFURLRef anURL) {
	TCFString cs = anURL;
	CopyFromTCFString(cs);
}

void TUniString::operator=(const HFSUniStr255 &us) {
	CopyFromHFSUniStr255(us);
}

void TUniString::operator=(const THFSUniStr255 &us) {
	CopyFromHFSUniStr255(us);
}

void TUniString::operator=(const UniChar *ustr) {
	CopyFromUSTR(ustr);
}

Boolean TUniString::IsEmpty() {
	if (m_dataLength == 0)
		return TRUE;
	else if (m_dataLength == 1 && (m_data[0] == BOM_LITTLE || m_data[0] == BOM_BIG))
		return TRUE;
	return FALSE;
}

int TUniString::Compare(const TUniString &s, CFOptionFlags option) {
	TCFString cs1 = *this;
	TCFString cs2 = s;
	return cs1.Compare(cs2, option);
}

TUniString TUniString::Append(const TUniString &s) {
	TCFString cs1 = *this;
	TCFString cs2 = s;
	return cs1 + cs2;
}

void TUniString::CutRearSpace() {
	if (IsEmpty())
		return;
	int n = GetLength();
	int length = 0;
	UniChar *p = *this;
	for (int i = 0; i < n;) {
		if (p[i] != ntohs(0x20))
			length = i + 1;
		i += 1;
	}
	m_dataLength = length;
	m_data[m_dataLength] = 0;
}

int TUniString::unistrlen(const UniChar *p) {
	int len;
	for (len = 0; p[len] != 0; len++) {
	}
	return len;
}

void TUniString::CopyFromUniString(const TUniString &us, int n) {
	int nn = us.GetLength();
	if (n < 0 || nn <= n)
		AllocCopy(us, us.GetLength());
	else
		AllocCopy(us, n);
	SetScriptCode(us.GetScriptCode());
}

void TUniString::CopyFromTCFString(const TCFString &cs) {
	int len = cs.GetLength();
	SafeDelete();
	Init();
	m_data = new UniChar[len + 1];
	m_allocLength = len;
	CFRange range = {0, len};
	CFStringGetCharacters(cs.GetCFStringRef(), range, m_data);
	m_dataLength = len;
	m_data[m_dataLength] = 0;
	m_scriptCode = cs.GetScriptCode();
}

void TUniString::CopyFromCSTR(const char *cstr, int length, int scriptCode) {
	char *p;
	if (length >= 0) {
		p = new char[length + 1];
		int i;
		for (i = 0; i < length && cstr[i] != 0; i++) {
			p[i] = cstr[i];
		}
		p[i] = 0;
	}
	else {
		p = (char *)cstr;
	}
	TCFString cs;
	cs.CopyFromCSTR(p, scriptCode);
	CopyFromTCFString(cs);
}

void TUniString::CopyFromPSTR(ConstStr255Param pStr, int scriptCode) {
	TCFString cs;
	cs.CopyFromPSTR(pStr, scriptCode);
	CopyFromTCFString(cs);
}

void TUniString::CopyFromUSTR(const UniChar *ustr, int length) {
/*
	if (length < 0)
		length = unistrlen(ustr);
	TCFString cs;
	cs.CopyFromUSTR(ustr, length);
	CopyFromTCFString(cs);
*/
	AllocCopy(ustr, length);
}

void TUniString::CopyFromHFSUniStr255(const HFSUniStr255 &hu) {
	AllocCopy(hu.unicode, hu.length);
}

void TUniString::CopyToHFSUniStr255(HFSUniStr255 *hu) {
	memset(hu, 0, sizeof(HFSUniStr255));
	hu->length = GetLength();
	memcpy(hu->unicode, m_data, GetLength() * sizeof(UniChar));
}

/*
void TUniString::CopyToTString(TString *s) {
	TCFString cs = *this;
	*s = cs;
}
*/

short SwapInt16(short inData) {
	return (((inData >> 8) & 0xFF) | ((inData << 8) & 0xFF00));
}

OSErr TUniString::OnAppleEvent(AEEventID eventID, DescType desiredClass, const AppleEvent *aev, TAEResult *result) {
	TAppleEvent ae(aev);
	if (eventID == kAEGetData) {
		TString s = *this;
		char *p = s;
		result->PutDataWithKeyword(keyAEResult, typeChar, (Ptr)p, GetLength());
		return noErr;
	}
	else if (eventID == kAESetData) {
		*this = ae.GetParamString();
		return noErr;
	}
	else if (eventID == kAEDelete) {
		return noErr;
	}
	return errAEEventNotHandled;
}

Boolean TUniString::CompareObjects(DescType oper, TAppleObject *obj) {
	if (obj == NULL)
		return FALSE;
	if (obj->GetObjectType() != GetObjectType())
		return FALSE;
	TUniString s = *(TUniString *)obj;
	int r = Compare(s);
	if (oper == kAEGreaterThan)
		return r > 0;
	else if (oper == kAEGreaterThanEquals)
		return r >= 0;
	else if (oper == kAEEquals)
		return r == 0;
	else if (oper == kAELessThan)
		return r < 0;
	else if (oper == kAELessThanEquals)
		return r <= 0;
	else
		return FALSE;
}

