/*
	TSystem.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.12	1.0			Takeshi Yoneki
*/

#include "csystem.h"

#define	OUT_OF_MEMORY_DLG		1000
#define	FRIEND_APPL_STR_NUM		1000

#define WHITE_PIXPAT_ID			1000
#define GRAY_PIXPAT_ID			1001
#define DEEP_PIXPAT_ID			1002
#define BLACK_PIXPAT_ID			1003

long pascal GrowZoneExit(long need);

void FatalErrorDlg(int errorNumber) {
	WindowRef port;
	port = ::GetDialogWindow(::GetNewDialog(1001, NULL, (WindowRef)-1));
	::SetPortWindowPort(port);
	TRect box;
	GetPortBounds(::GetWindowPort(port), box);
	box.LocalToGlobal();
	TRect sr = theSystem->GetDesktopRect();
	short left = (sr.right - box.GetWidth()) / 2;
	::MoveWindow(port, left, box.top, FALSE);
	::ShowWindow(port);
	Str255 errStr;
	::NumToString(errorNumber, errStr);
	::ParamText(errStr, "\p", "\p", "\p");
	short itemHit;
	::ModalDialog(NULL, &itemHit);
	::ExitToShell();
}

long pascal GrowZoneExit(long need) {
	for (int i = 0; i < 3; i++)
		SysBeep(0);

	TDialog dlg(OUT_OF_MEMORY_DLG);
	dlg.ModalDialog();
	ExitToShell();

	return 0;
}

void TSystem::InitToolbox(int masters, long stack) {
	for (int i = 0; i < masters; i++)
		MoreMasters();

	InitCursor();
	FlushEvents(everyEvent,0);
}

TSystem::TSystem(int masters, long stack, long configFlags) {
	OSErr err;
	long response;

	InitToolbox(masters, stack);

	applDirectory.GetVol();

	Gestalt(gestaltSystemVersion, &systemVersion);
	if (systemVersion >= 0x0700) {
		FindFolder(kOnSystemDisk, kSystemFolderType, FALSE, &sysDirectory.m_volume, &sysDirectory.m_dirID);
	}

	hasColorQD = FALSE;
	if (Gestalt(gestaltQuickdrawVersion, &response) == noErr) {
		if (response >= gestalt8BitQD)
			hasColorQD = TRUE;
		if (response >= gestalt32BitQD)
			has32ColorQD = TRUE;
	}

	hasAppleEvents = Gestalt(gestaltAppleEventsAttr, &response) == noErr;
	hasAppleScript = FALSE;
	m_scriptingComponent = NULL;
	if (hasAppleEvents) {
		if ((response & (1 << gestaltScriptingSupport)) != 0 || systemVersion >= 0x1000) {
			err = AEObjectInit();
			if (err == noErr) {
				hasAppleScript = TRUE;
				m_scriptingComponent = OpenDefaultComponent(kOSAComponentType, kOSAGenericScriptingComponentSubtype);
			}
		}
	}

	hasTSMTE = FALSE;
	tsmteVersion = 0;
	if (Gestalt(kTSMTESignature, &response) == noErr) {
		if ((response & (1 << gestaltTSMTE)) != 0) {
			hasTSMTE = TRUE;
		}
	}

	hasDragMgr = FALSE;
	if (Gestalt(gestaltDragMgrAttr, &response) == noErr) {
		if ((response & (1 << gestaltDragMgrPresent)) != 0)
			hasDragMgr = TRUE;
	}

	hasBaloonHelp = FALSE;
	if (Gestalt(gestaltHelpMgrAttr, &response) == noErr) {
		if ((response & (1 << gestaltHelpMgrPresent)) != 0)
			hasBaloonHelp = TRUE;
	}

	hasThreadMgr = FALSE;
	if (Gestalt(gestaltThreadMgrAttr, &response) == noErr) {
		if ((response & (1 << gestaltThreadMgrPresent)) != 0)
			hasThreadMgr = TRUE;
	}

	hasAppearance = FALSE;
	appearanceVersion = 0;
	if (Gestalt(gestaltAppearanceAttr, &response) == noErr) {
		if ((response & (1 << gestaltAppearanceExists)) != 0) {
			err = RegisterAppearanceClient();
			hasAppearance = TRUE;
			if (Gestalt(gestaltAppearanceVersion, &response) == noErr)
				appearanceVersion = response;
		}
	}

	hasTempMemory = FALSE;
	if (Gestalt(gestaltOSAttr, &response) == noErr) {
		if ((response & (1 << gestaltTempMemSupport)) != 0 &&
		        (response & (1 << gestaltRealTempMemory)) != 0 &&
		        (response & (1 << gestaltTempMemTracked)) != 0) {
			hasTempMemory = TRUE;
		}
	}

	hasHFSPlus = FALSE;
	if (Gestalt(gestaltFSAttr, &response) == noErr) {
		if ((response & (1 << gestaltHasHFSPlusAPIs)) != 0) {
			hasHFSPlus = TRUE;
		}
	}

	hasNavigation = FALSE;
	if (NavServicesAvailable())
		hasNavigation = NavLoad() == noErr;

	carbonVersion = 0;
	if (Gestalt(gestaltCarbonVersion, &response) == noErr)
		carbonVersion = response;

	hasTextEncodingConverter = FALSE;
	hasTextEncodingConverter = (CreateTextEncoding != NULL);
	if (hasTextEncodingConverter)
		m_encInfo.Create();

	theCursor = new TCursorList;
	theCursor->Load(watchCursor);
	theCursor->Load(iBeamCursor);
	theCursor->Load(plusCursor);
	theCursor->Load(crossCursor);

	//	theUndo = new TEditorUndo;
	theWindowArray = new TWindowArray;

	short foundVRefNum;
	long foundDirID;

	sysPrefDirectory = sysDirectory;
	if (systemVersion >= 0x0700)
		if (noErr == FindFolder(kOnSystemDisk, kPreferencesFolderType, FALSE, &foundVRefNum, &foundDirID))
			sysPrefDirectory = TDirectory(foundVRefNum, foundDirID);

	m_uppGrowZoneProcInfo.SetUPP(GrowZoneExit);
	SetGrowZone((GrowZoneUPP)m_uppGrowZoneProcInfo.GetUPP());

	TFilePath filePath("debug.log", applDirectory);
	theDebug = new TDebug(filePath);

	m_friendAppl.LoadStrNumAll(FRIEND_APPL_STR_NUM);
	for (int i = 0; i < 256; i++)
		m_charByteData[i] = 1;

	if (systemVersion >= 0x0700) {
		whitePixPat = new TPixPat(WHITE_PIXPAT_ID);
		grayPixPat = new TPixPat(GRAY_PIXPAT_ID);
		deepPixPat = new TPixPat(DEEP_PIXPAT_ID);
		blackPixPat = new TPixPat(BLACK_PIXPAT_ID);
	}
	else {
		whitePixPat = NULL;
		grayPixPat = NULL;
		deepPixPat = NULL;
		blackPixPat = NULL;
	}

	if (systemVersion < 0x0700)
		m_safetyMemorySize = 32 * 1024;
	else if (systemVersion < 0x0800)
		m_safetyMemorySize = 64 * 1024;
	else
		m_safetyMemorySize = 128 * 1024;

	if (FreeMem() < m_safetyMemorySize)
		UnloadScrap();

	m_useNavigation = TRUE;
}

TSystem::~TSystem() {
	delete theWindowArray;

	if (hasNavigation)
		NavUnload();
	if (m_scriptingComponent != NULL)
		CloseComponent(m_scriptingComponent);
}

TRgn TSystem::GetDesktopRgn() {
	if (hasColorQD)
		return GetGrayRgn();
	else
		return GetDesktopRect();
}

TRect TSystem::GetDesktopRect() {
	BitMap screenBits;
	GetQDGlobalsScreenBits(&screenBits);
	return screenBits.bounds;
}

Boolean TSystem::IsFriend(OSType creator) {
	TString s;
	s = creator;
	TString *cmp;
	for (cmp = m_friendAppl.GetTop(); cmp != NULL; cmp = cmp->GetNext()) {
		if (*cmp == s)
			return TRUE;
	}
	return FALSE;
}

void TSystem::MakeCharSize(TWindow *port) {
	TSavePort savePort(port);
	char bf[2] = "\0";
	for (int i = 0; i < 256; i++) {
		bf[0] = i;
		int flag;
		flag = CharacterByteType(bf, 0, ::IntlScript());
		if (flag == smFirstByte)
			m_charByteData[i] = 2;
		else
			m_charByteData[i] = 1;
	}
}

int TSystem::GetMainDepth() {
	if (hasColorQD) {
		GDHandle h = GetMainDevice();
		if (h != NULL)
			return (*(*h)->gdPMap)->pixelSize;
	}
	return 1;
}

