/*
	TWindow.cpp
 
	Ten Tool Class Library
	Copyright (C) 1996 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 1996.02.12	1.0			Takeshi Yoneki
*/

#include "csystem.h"

TWindow *TWindowArray::GetByWindowRef(WindowRef wref) {
	for (int i = 0; i < GetCount(); i++) {
		TWindow *win = (TWindow *)GetNth(i);
		if (win != NULL) {
			if (*win == wref)
				return win;
		}
	}
	return NULL;
}

pascal OSErr TWindow_DragReceiveHandler(WindowPtr wp, void *refCon, DragReference dragRef);
pascal OSErr TWindow_DragReceiveHandler(WindowPtr wp, void *refCon, DragReference dragRef) {
	if (refCon == NULL)
		return -1;
	TWindow *win = (TWindow *)refCon;
	return win->DragReceiveHandler(wp, refCon, dragRef);
}

pascal OSErr TWindow_DragTrackingHandler(DragTrackingMessage message, WindowPtr wp, void *refCon, DragReference dragRef);
pascal OSErr TWindow_DragTrackingHandler(DragTrackingMessage message, WindowPtr wp, void *refCon, DragReference dragRef) {
	if (refCon == NULL)
		return -1;
	TWindow *win = (TWindow *)refCon;
	return win->DragTrackingHandler(message, wp, refCon, dragRef);
}

OSErr TWindow::DragReceiveHandler(WindowPtr wp, void *refCon, DragReference dragRef) {
	return -1;
}

OSErr TWindow::DragTrackingHandler(DragTrackingMessage message, WindowPtr wp, void *refCon, DragReference dragRef) {
	return -1;
}

TWindow::TWindow() {
	TRect sr = theSystem->GetDesktopRect();
	m_growRect.SetRect(MIN_WIN_SIZE, MIN_WIN_SIZE, sr.GetWidth(), sr.GetHeight());
	m_dragRect.SetRect(WIN_RECT_PIX, 24, sr.GetWidth() - WIN_RECT_PIX, sr.GetHeight() - WIN_RECT_PIX);
	m_zoomRect.SetRect(WIN_RECT_PIX, WIN_RECT_TOP, sr.GetWidth() - WIN_RECT_PIX - 70, sr.GetHeight() - WIN_RECT_PIX);
	m_port = NULL;
	m_hasGrowIcon = FALSE;
	m_name.m_container = this;
	m_depth = CalcDepth();
	m_prevClickTime = 0;
	theWindowArray->Append(this);
	m_enableDrag = TRUE;
}

TWindow::~TWindow() {
	theWindowArray->Remove(this);
	if (m_port != NULL) {
		if (theSystem->hasDragMgr && m_enableDrag) {
			if (m_uppDragReceiveHandler.GetUPP() != NULL)
				RemoveReceiveHandler((DragReceiveHandlerUPP)m_uppDragReceiveHandler.GetUPP(), m_port);
			if (m_uppDragTrackingHandler.GetUPP() != NULL)
				RemoveTrackingHandler((DragTrackingHandlerUPP)m_uppDragTrackingHandler.GetUPP(), m_port);
		}
		DisposeWindow();
	}
	m_port = NULL;
}

void TWindow::GetNewWindow(short windowID, WindowRef behind) {
	if (theSystem->hasColorQD)
		m_port = ::GetNewCWindow(windowID, NULL, behind);
	else
		m_port = ::GetNewWindow(windowID, NULL, behind);
	if (theSystem->hasDragMgr && m_enableDrag) {
		m_uppDragReceiveHandler.SetUPP(TWindow_DragReceiveHandler);
		m_uppDragTrackingHandler.SetUPP(TWindow_DragTrackingHandler);
		InstallReceiveHandler((DragReceiveHandlerUPP)m_uppDragReceiveHandler.GetUPP(), m_port, this);
		InstallTrackingHandler((DragTrackingHandlerUPP)m_uppDragTrackingHandler.GetUPP(), m_port, this);
	}
	m_activate = FALSE;	/* \[XłvisibleOFFɂ邱 */
}

TRect TWindow::GetPortRectGlobal() {
	TSavePort savePort(this);
	TRect wr = GetPortRect();
	wr.LocalToGlobal();
	return wr;
}

TPoint TWindow::GetPositionGlobal() {
	TRect r = GetPortRectGlobal();
	return TPoint(r.left, r.top);
}

void TWindow::SelectWindow() {
	::SelectWindow(m_port);
	m_activate = TRUE;
}

void TWindow::ShowWindow() {
	::ShowWindow(m_port);
	m_activate = TRUE;
}

void TWindow::HideWindow() {
	::HideWindow(m_port);
	m_activate = FALSE;
}

void TWindow::AppendChildControl(TControlBase *child) {
	child->SetParentWindow(this);
	Append(child);
}

void TWindow::OnUpdate() {
	TSavePort savePort(this);
	m_depth = CalcDepth();
	BeginUpdate();
	DrawControls();
	TControlBase *control;
	for (control = GetTop(); control != NULL; control = control->GetNext())
		control->OnUpdate();
	if (m_hasGrowIcon)
		DrawGrowIconOnly();
	EndUpdate();
}

void TWindow::OnActivate(Boolean act) {
	TSavePort savePort(this);
	m_activate = act;
	TControlBase *control;
	for (control = GetTop(); control != NULL; control = control->GetNext())
		control->OnActivate(act);
	if (m_hasGrowIcon)
		DrawGrowIconOnly();
}

void TWindow::OnDrag(Point where) {
	TSavePort savePort(this);
	DragWindow(where, m_dragRect);
	if (m_port == FrontWindow())
		SelectWindow();
	SetWindowUserState(GetPortRectGlobal());
}

TControlBase *TWindow::WhichControl(TPoint mousePt) {
	TControlBase *control;
	for (control = GetTop(); control != NULL; control = control->GetNext())
		if (control->IsInside(mousePt))
			return control;
	return NULL;
}

void TWindow::OnContent(EventRecord *ev) {
	TSavePort savePort(this);
	TPoint mousePt = ev->where;
	mousePt.GlobalToLocal();
	TControlBase* target = WhichControl(mousePt);
	if (target == NULL)
		return;
	target->OnContent(ev);
}

void TWindow::OnChangeSize() {
	TSavePort savePort(this);
	TControlBase *control;
	short top = 0;
	TPoint pt(0, 0);
	for (control = GetTop(); control != NULL; control = control->GetNext()) {
		TPoint xy = control->OnChangeSize(pt);
		pt.v += xy.v;
		pt.h += xy.h;
	}
}

void TWindow::OnGrow(Point where) {
	TSavePort savePort(this);
	long sz = GrowWindow(where, m_growRect);
	short width = LoWord(sz);
	short height = HiWord(sz);
	TRect r = GetPortRect();
	if (sz != 0 && (width != r.GetWidth() || height != r.GetHeight())) {
		SizeWindow(width, height);
		OnChangeSize();
		SetWindowUserState(GetPortRectGlobal());
	}
}

Boolean TWindow::OnClose() {
	return TRUE;
}

Boolean TWindow::OnClose(const AppleEvent *aev) {
	return OnClose();
}

Boolean TWindow::OnGoAway(Point where) {
	TSavePort savePort(this);
	if (TrackGoAway(where))
		return OnClose();
	else
		return FALSE;
}

void TWindow::OnZoom(Point where, int partCode) {
	TSavePort savePort(this);
	if (TrackBox(where, partCode)) {
		ZoomWindow(partCode);
		OnChangeSize();
	}
}

void TWindow::OnTask() {
	TSavePort savePort(this);
	if (m_activate) {
		TPoint mousePt;
		mousePt.GetMouse();
		Boolean set
			= FALSE;
		if (set
		        == FALSE) {
			TControlBase* target = WhichControl(mousePt);
			if (target != NULL)
				target->OnMouseCursor(mousePt);
			else
				theCursor->Set(arrowCursor);
		}
	}
	TControlBase *control;
	for (control = GetTop(); control != NULL; control = control->GetNext()) {
		control->OnTask();
	}
}

void TWindow::DrawGrowIconOnly() {
	TRgn saveRgn;
	saveRgn.GetClip();
	TRect r = GetPortRect();
	r.top = GetPortRect().bottom - SCROLL_BAR_WIDTH + 1;
	r.left = GetPortRect().right - SCROLL_BAR_WIDTH + 1;
	r.ClipRect();
	DrawGrowIcon();
	saveRgn.SetClip();
}

void TWindow::SetFont(short fontNum) {
	TSavePort savePort(this);
	::TextFont(fontNum);
}

void TWindow::SetFontSize(short fontSize) {
	TSavePort savePort(this);
	::TextSize(fontSize);
}

void TWindow::RGBBackColor(RGBColor *color) {
	if (theSystem->hasColorQD) {
		TSavePort savePort(this);
		::RGBBackColor(color);
	}
}

int TWindow::CalcDepth() {
	if (theSystem->hasColorQD) {
		GDHandle mh = NULL;
		if (m_port == NULL) {
			GDHandle h = GetDeviceList();
			mh = h;
			for (; h != NULL;) {
				if (((*h)->gdFlags >> mainScreen) & 1)
					mh = h;
				h = GetNextDevice(h);
			}
		}
		else {
			long square = 0;
			TRect winRect = GetPortRectGlobal();
			GDHandle h = GetDeviceList();
			mh = h;
			for (; h != NULL;) {
				TRect hr((*h)->gdRect);
				TRect r = winRect.SectRect(hr);
				long sq = r.GetHeight() * r.GetWidth();
				if (sq > square) {
					square = sq;
					mh = h;
				}
				h = GetNextDevice(h);
			}
		}
		if (mh != NULL)
			return (*(*mh)->gdPMap)->pixelSize;
	}
	return 1;
}

TRect TWindow::CalcNewWindowRect(int n, TRect userRect) {
	TRgn desktop = theSystem->GetDesktopRgn();
	TRgn wrgn = userRect;
	TRect wr = userRect;
	if (desktop.UnionRgn(wrgn) != desktop)	/* ʂOĂ */
		return m_zoomRect;
	wrgn.OffsetRgn(SCROLL_BAR_WIDTH * n, SCROLL_BAR_WIDTH * n);
	if (desktop.UnionRgn(wrgn) == desktop)	/* ʂɓĂ */
	{
		wr.OffsetRect(SCROLL_BAR_WIDTH * n, SCROLL_BAR_WIDTH * n);
		return wr;
	}
	/* ړʉʂnYꍇ */
	/* ړ\ */
	wrgn = userRect;
	int m;
	for (m = 0;; m++) {
		wrgn.OffsetRgn(SCROLL_BAR_WIDTH, SCROLL_BAR_WIDTH);
		if (desktop.UnionRgn(wrgn) != desktop)
			break;
	}
	n = n % (m + 1);
	wr.OffsetRect(SCROLL_BAR_WIDTH * n, SCROLL_BAR_WIDTH * n);
	return wr;
}

Boolean TWindow::IsDoubleClick(Point where, long when) {
	long dbl;
	Rect r;
	int click = 1;
	dbl = GetDblTime();
	if (m_prevClickTime != 0) {
		if (when - m_prevClickTime < dbl) {
			Pt2Rect(m_prevClickPoint, where, &r);
			if (r.right - r.left < 5 && r.bottom - r.top < 5)
				click++;
		}
	}
	m_prevClickTime = when;
	m_prevClickPoint = where;
	return click == 2;
}

void TWindow::FlushPortBuffer(TRect r) {
	m_flushRgn.RectRgn(r);
	QDFlushPortBuffer(::GetWindowPort(m_port), m_flushRgn);
}

/*
	Window̏͂Zobt@
*/
long TWindow::GetObjectNumber() {
	if (m_container == NULL)
		return 0;
	int count = m_container->CountContainedObjects(cWindow);
	for (int i = 0; i < count; i++) {
		TWindow *win = (TWindow *)m_container->GetContainedObjectByNumber(cWindow, i + 1);
		if (*this == *win)
			return i + 1;
	}
	return 0;
}

TAppleObject *TWindow::GetObjectProperty(DescType desiredClass, DescType keyForm, const AEDesc *keyData) {
	TAppleObject *obj = NULL;
	if (desiredClass == pName)
		return &m_name;
	if (obj != NULL)
		obj->BeTemporary();
	return obj;
}

OSErr TWindow::OnAdhocForContainer(TToken *token, AEEventID eventID, const AppleEvent *aev, TAEResult *result) {
	return noErr;
}

/*
	TSavePort
*/

TSavePort::TSavePort(TWindow *cw) {
	GetPort(&m_savePort);
	if (cw != NULL)
		cw->SetPort();
}

TSavePort::TSavePort(GrafPtr gp) {
	GetPort(&m_savePort);
	if (gp != NULL)
		::SetPort(gp);
}

TSavePort::~TSavePort() {
	SetPort(m_savePort);
}

/*
	TSaveClipRgn
*/

TSaveClipRgn::TSaveClipRgn() {
	m_savePort = NULL;
	m_saveRgn.GetClip();
}

TSaveClipRgn::TSaveClipRgn(TWindow *cw) {
	GetPort(&m_savePort);
	cw->SetPort();
	m_saveRgn.GetClip();
}

TSaveClipRgn::~TSaveClipRgn() {
	m_saveRgn.SetClip();
	if (m_savePort != NULL)
		SetPort(m_savePort);
}

/*
	Rg[x[XNX
*/

void TControlBase::OnContent(EventRecord *ev) {
}

void TControlBase::OnActivate(Boolean act) {
}

void TControlBase::OnUpdate() {
}

short TControlBase::OnChangeSize(short top) {
	return 0;
}

TPoint TControlBase::OnChangeSize(TPoint pt) {
	return TPoint(0, OnChangeSize(pt.v));
}

void TControlBase::OnTask() {
}

Boolean TControlBase::IsInside(TPoint mousePt) {
	return FALSE;
}

void TControlBase::OnMouseCursor(TPoint mousePt) {
	theCursor->Set(arrowCursor);
}

/*
	ėpItXN[NX
*/

TOffScreen::TOffScreen() {
	m_world = NULL;
	m_good = FALSE;
}

TOffScreen::~TOffScreen() {
	DisposeGWorld();
}

QDErr TOffScreen::NewGWorld(short PixelDepth, const Rect *boundsRect, CTabHandle cTable, GDHandle aGDevice, GWorldFlags flags) {
	m_pixelDepth = PixelDepth;
	m_boundsRect = *boundsRect;
	DisposeGWorld();
	QDErr err = ::NewGWorld(&m_world, PixelDepth, boundsRect, cTable, aGDevice, flags);
	m_good = err == noErr;
	return err;
}

void TOffScreen::DisposeGWorld() {
	if (m_world != NULL)
		::DisposeGWorld(m_world);
	m_world = NULL;
}

void TOffScreen::ChangeSize(const Rect *r) {
	if (m_world == NULL)
		return;

	DisposeGWorld();
	NewGWorld(m_pixelDepth, r);
}

GrafPtr TOffScreen::BeginDraw() {
	GetGWorld(&m_savePort, &m_saveDevice);
	if (m_world == NULL)
		return NULL;
	SetGWorld(m_world, 0L);
	if (LockPixels(GetPortPixMap(m_world)) == FALSE)
		return NULL;
	m_good = TRUE;
	return (GrafPtr)m_world;
}

void TOffScreen::EndDraw() {
	SetGWorld(m_savePort, m_saveDevice);
	if (m_world == NULL)
		return;
	UnlockPixels(GetPortPixMap(m_world));
}

