/*
	NTag.h
 
	ID3TagEdit
	Copyright (C) 2002 Takeshi Yoneki, OSTRACISM CO.
 
	Make : 2002.01.30	1.0			Takeshi Yoneki
*/

#ifndef	_NTAG_H_
#define	_NTAG_H_

class NView;
class NTagList;

#define	CODE_AS_IS		(-1)
#define	CODE_LATIN1		0
#define	CODE_UTF16		1

#define	TAG_VERSION_1_0		0
#define	TAG_VERSION_1_1		1
#define	TAG_VERSION_2_2		2
#define	TAG_VERSION_2_3		3
#define	TAG_VERSION_2_4		4

#define	TAG_V2_HEAD_FLAG_UNSYNC			0x80
#define	TAG_V2_HEAD_FLAG_EXTENDED		0x40
#define	TAG_V2_HEAD_FLAG_EXPERIMENTAL	0x20
#define	TAG_V2_HEAD_FLAG_FOOTER			0x10

#pragma options align=packed

struct ID3_V1 {
	char m_tag[3];
	char m_title[30];
	char m_artist[30];
	char m_album[30];
	char m_year[4];
	union {
		struct {
			char m_comment[30];
		}
		v10;
		struct {
			char m_comment[28];
			char m_null;
			char m_track;
		}
		v11;
	} u;
	char m_genre;
};

struct ID3_V2_HEAD {
	char m_tag[3];
	char m_major;
	char m_minor;
	char m_flags;
	char m_size[4];
};

struct ID3_V2_2_FRAME {
	char m_frameID[3];
	char m_size[3];
};

struct ID3_V2_2_FRAME_TEXT : public ID3_V2_2_FRAME {
	char m_encoding;
};

struct ID3_V2_2_FRAME_COMMENT : public ID3_V2_2_FRAME_TEXT {
	char m_language[3];
};

struct ID3_V2_3_FRAME {
	char m_frameID[4];
	char m_size[4];
	char m_flags[2];
};

struct ID3_V2_3_FRAME_TEXT : public ID3_V2_3_FRAME {
	char m_encoding;
};

struct ID3_V2_3_FRAME_COMMENT : public ID3_V2_3_FRAME_TEXT {
	char m_language[3];
};

typedef ID3_V2_HEAD ID3_V2_FOOT;

#pragma options align=reset

class NTagString : public TAppleObject {
public:
	TMemory m_org;
	TUniString m_str;
	int m_encoding;
	NTagString();
	void operator=(const NTagString& ts);
	void Set(char *data, int count, int encoding = CODE_AS_IS);
	void DecodeString(int as, int scriptCode);
	void EncodeString(int as, int scriptCode, int limit = -1);
	int CheckString();
private:
	int UnicodeToSJISChar(unsigned short in, char *out);
};

#define FOR_BROWSE		0
#define FOR_TSV			1

struct NTagRecord {
	TFilePath m_filePath;
	TUniString m_fileName;
	NTagString m_title;
	NTagString m_artist;
	NTagString m_album;
	NTagString m_comment;
	TUniString m_year;
	int m_genre;
	int m_version;
	int m_track;
	int m_trackMax;
	int m_loadEnc;
	int m_saveEnc;
	/**/
	Boolean m_modify;
};


class NTag : public TAppleObject, public NTagRecord {
public:
	TIconRef m_icon;
	TFilePath m_tempPath;
	TFinderInfo m_info;

	NTagRecord m_undoBuffer;
	NTagRecord m_redoBuffer;

	long m_soundStartPos;
	long m_soundSize;

	NTag(NTagList *parent);
	NTag *CloneForModify();
	NTag *GetNext() {
		return (NTag *)TBase::GetNext();
	}
	void CopyToUndoBuffer() {
		m_undoBuffer = *(NTagRecord *)this;
	}
	void CopyToRedoBuffer() {
		m_redoBuffer = *(NTagRecord *)this;
	}
	void CopyFromUndoBuffer() {
		*(NTagRecord *)this = m_undoBuffer;
	}
	void CopyFromRedoBuffer() {
		*(NTagRecord *)this = m_redoBuffer;
	}
	TString GetVersionString();
	TString GetGenreString(int forWhat = FOR_BROWSE);
	TString GetTrackString(DataBrowserPropertyID property);
	TString GetCodingString(int coding);
	int CheckLimit();
	TCFString GetStringByPropertyID(DataBrowserPropertyID property, int forWhat = FOR_BROWSE);
	void SetStringByPropertyID(DataBrowserPropertyID property, TCFString s);
	int LoadTag(TFilePath filePath);
	void DecodeStrings();
	Boolean LoadV1Tag();
	Boolean LoadV2Tag();
	Boolean LoadV22Tag(TFileIO *file, long tagSize);
	Boolean LoadV23Tag(TFileIO *file, long tagSize);
	long CalcV2TagSize(ID3_V2_HEAD *head);
	long CalcV22FrameSize(ID3_V2_2_FRAME *frame);
	long CalcV23FrameSize(ID3_V2_3_FRAME *frame);
	void EncodeStrings();
	void MakeTag(TMemory *buffer);
	void MakeV1TagSub(char *dst, TMemory &src);
	void MakeV1Tag(TMemory *buffer);
	void MakeV2Tag(TMemory *buffer);
	void SetV2TagSize(ID3_V2_HEAD *head, int sz);
	int MakeV2Frame(char *frame, TString frameID, int sz);
	int MakeV2TextFrame(char *frame, TString frameID, NTagString &data);
	int MakeV2CommentFrame(char *frame, TString frameID, NTagString &data);
	int MakeV2ASCIIFrame(char *frame, TString frameID, TString &data);
	int GetViewScriptCode();
	TUnicodeTextEncodingInfo *GetUniEncInfo();
	void UpdateDataBrowser();
	void CopyFileName2Title();
	void CopyTitle2FileName();
	void CopyParent2Album();
	void CopyGranpa2Artist();
	void PlusTrackFileName();
	void PlusTrackTitle();
	void WriteTSVLine(TFileWriter *file);
	void ReadTSVLine(TString &line);
	void SetUndo(int cmd);
	void CorrectFileName();
};

class NTagList : public TAppleObject {
public:
	NView *m_view;
	NTag** m_array;
	NTagList();
	~NTagList();
	int MakeOrderArray();
	void SaveFiles();
	void OnImportTSV();
	void OnExportTSV();
	void LoadTagFiles(TListBase *list, TStandardFileReply *file);
	void AppendFiles(TStandardFileReplyList *fileList);
	NTag *GetBySelection();
	NTag *GetByFilePath(TFilePath filePath);
	NTag *GetTop() {
		return (NTag *)TListBase::GetTop();
	}
	NTag *GetNth(int n) {
		return (NTag *)TListBase::GetNth(n);
	}
	void CopyFileName2Title();
	void CopyTitle2FileName();
	void CopyParent2Album();
	void CopyGranpa2Artist();
	void PlusTrackFileName();
	void PlusTrackTitle();
	Boolean ImportTSV(TFilePath &path);
	Boolean ExportTSV(TFilePath &path);
	void SetUndo(int cmd);
};

#endif
